namespace eval FEditor {

    variable used_functions ""
    variable type "Basic"

    # Functions
    variable available_functions [list sqrt abs ln log fac sin cos \
	    tan asin acos atan sinh cosh tanh exp srand if_then \
	    if_else interpolate int]
    variable example_functions {"sqrt()" "abs()" "ln()" "log()" "fac()" \
	    "sin()" "cos()" "tan()" "asin()" "acos()" "atan()" "sinh()" \
	    "cosh()" "tanh()" "exp()" "srand()" "if()then()else()endif" \
	    "if()then()elif()then()else()endif" "interpolate(# # )" "int()"}

    # Basic
    variable basic_variables "t it tm rand dt e pi"
    variable example_basic_variables "t it tm rand dt exp(1) pi"
    variable basic_operators "+ - * / ^ mod div rmod rdiv max min ~ < <= >\
	    >= = != & |"

    # Chargables
    variable variables ""
    variable example_variables ""
    variable operators ""

    variable fninput
    variable ff_names

}

proc FEditor::init { wn predefined {field ""} fieldname {fieldvalue ""} fieldslist tags } {

    global tkPriv tcl_platform

    variable used_functions

    # Functions
    variable available_functions
    variable example_functions

    # Chargables
    variable variables
    variable example_variables
    variable operators

    # Basic
    variable basic_variables
    variable example_basic_variables
    variable basic_operators

    variable fninput
    variable ff_names

    
    # If no FIELD is provided, read Fields Names to insert into combobox
    if { $field == "" } {
	set ff_names ""
	foreach {ff_field ff_name} $fieldslist {
	    append ff_names " {$ff_name}"
	}
    }

    # If lists of predefined variables and operators are provided
    # in $predefined ($predefined is a list of three lists), assign
    # them to comboboxex. If not, use basic_ values.
    foreach {ivars iexvars iopers} $predefined {
	if { [llength $ivars] > 0 } {
	    set variables $ivars
	} else {
	    set variables $basic_variables
	}
	if { [llength $iexvars] > 0 } {
	    set example_variables $iexvars
	} else {
	    set example_variables $example_basic_variables
	}
	if { [llength $iopers] > 0 } {
	    set operators $iopers
	} else {
	    set variables $basic_operators
	}
    }

    set w $wn.feditor
    InitWindow $w "Function Editor" PreTdynFunctionEditorWindowGeom ""

    GidHelp $w "Insert or edit the function and press OK to update material data"
    grid columnconf $w 0 -weight 1
    grid rowconf $w 0 -weight 1

    # Main Frame
    set mfr [frame $w.f -bd 2 -relief groove]

    # Buttons bar
    set upbar [frame $mfr.upbar]
    set upbarwest [frame $upbar.west]
    set upbareast [frame $upbar.east]
    # Top buttons
    set btncb1 [button $upbarwest.cb1 -image [imagefromtdyn sym-fx.gif] \
	    -command "FEditor::openfunclist $w"]
    set btncb2 [button $upbarwest.cb2 -image [imagefromtdyn sym-x.gif] \
	    -command "FEditor::openvarlist $w"]
    set btncb3 [button $upbarwest.cb3 -image [imagefromtdyn sym-up.gif] \
	    -command "FEditor::openoperlist $w"]
    if { [llength $used_functions] > 0 } {
	set btncb4 [button $upbareast.cb4 -text "History" \
		-command "FEditor::openhistlist $w" -highlightthickness 0]
    }
    set btncb5 [button $upbareast.cb5 -image [imagefromtdyn questionarrow.gif] \
	    -command "PickHelp $w" -highlightthickness 0]
    GidHelp $btncb1 "List of available functions" 
    GidHelp $btncb2 "List of available variables and constants" 
    GidHelp $btncb3 "List of available operators"
    if { [llength $used_functions] > 0 } {
	GidHelp $btncb4 "History of used funcions"
    }
    if { $field == "" || $field == "SELECT FIELD" } {
	# Combo box with fields names
	set combobox [ComboBox $upbareast.combobox -editable no \
		-values $ff_names -text "SELECT A FIELD" -textvariable FEditor::fninput]
    } else {
	set l_fn [Label $upbareast.l_cprs -text $fieldname\
		-helptext "Edited function will be used for $fieldname."]
    }

    # Text window and scroolbars
    set textwindow [text $mfr.t -xscroll "$w.f.hscr set" \
	    -yscroll "$w.f.vscr set" -width 60]
    set vertscroll [scrollbar $mfr.vscr -orient vertical -comm "$textwindow yview"]
    set horiscroll [scrollbar $mfr.hscr -orient horizontal -comm "$textwindow xview"]
    bind $textwindow <KeyPress> {
	if { [regexp {[a-zA-Z]} %A] } {
	    catch {
		if {[%W compare sel.first <= insert]
		&& [%W compare sel.last >= insert]} {
		    %W delete sel.first sel.last
		}
	    }
	    %W insert insert %A ""
	    break
	}
    }
    GidHelp $textwindow "Insert or edit the function and press OK to update material data"
    # Insert Field Value into the text window
    $textwindow ins end $fieldvalue

    # Down buttons
    set downbar [frame $w.downbar]
    set btnok [button $downbar.ok -text OK -width 5 \
	    -command \
	    "FEditor::sendfunction \"$w\" \"$field\" \"$fieldslist\" \"$textwindow\" \"$tags\"" \
	    -und 0]
    set btnclear [button $downbar.clear -text Clear -width 5 \
	    -command "$textwindow del 1.0 end" -und 1]
    set btncancel [button $downbar.cancel -text Cancel -width 5 \
	    -command "destroy $w" -und 0]
    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $btncancel"

    # BINDS
    bind $btnok <Return> "tkButtonInvoke $btnok"
    bind $w <Escape> "tkButtonInvoke $btncancel"
    bind $textwindow <Alt-KeyPress-o> "tkButtonInvoke $btnok"
    bind $textwindow <Alt-KeyPress-l> "tkButtonInvoke $btnclear"
    bind $textwindow <Alt-KeyPress-c> "tkButtonInvoke $btncancel"

    # GRIDS
    # Main frame
    grid $mfr -row 0 -column 0  -pady 5 -sticky ewns -padx 2
    # Top frames and buttons
    grid $upbar -row 0 -column 0 -sticky ew -pady 5 -columnspan 2
    grid $upbarwest -row 0 -column 0 -sticky w
    grid $upbareast -row 0 -column 1 -sticky e
    grid columnconf $upbar {0 1} -weight 1
    grid $btncb1 -row 0 -column 0 -sticky w -pady 2 -padx 5 
    grid $btncb2 -row 0 -column 1 -sticky w -pady 2 -padx 5
    grid $btncb3 -row 0 -column 2 -sticky w -pady 2 -padx 5

    # Select field combo box
    if { $field == "" || $field == "SELECT FIELD" } {
	# Combo box with fields names
	grid $combobox -row 0 -column 3 -sticky w -pady 2 -padx 5
    } else {
	grid $l_fn -row 0 -column 3 -sticky w -pady 2 -padx 50
    }
    if { [llength $used_functions] > 0 } {
	grid $btncb4  -row 0 -column 4 -sticky e -pady 2 -padx 10
    }
    grid $btncb5 -row 0 -column 5 -sticky e -pady 2 -padx 5

    # Textwindow
    grid $textwindow -row 1 -column 0 -padx 2 -sticky nsew
    grid $vertscroll -row 1 -column 1 -sticky ns
    grid $horiscroll -row 2 -column 0 -sticky ew -padx 2 
    grid columnconf $mfr 0 -weight 1
    grid rowconf $mfr 1 -weight 1

    # Bottom frame and buttons
    grid $downbar -row 1 -column 0
    grid $btnok -row 0 -column 0 -sticky ew -padx 3 -pady 2
    grid $btnclear -row 0 -column 1 -sticky ew -padx 3 -pady 2
    grid $btncancel -row 0 -column 2 -sticky ew -padx 3 -pady 2

    focus $mfr
}

proc FEditor::openfunclist { w } {

    variable available_functions

    ListBoxWindow $w {Function List} {Select the function to be inserted} \
	    "$available_functions" "FEditor::sendfuncselc $w.f.t"
}

proc FEditor::openvarlist { w } {

    variable variables

    ListBoxWindow $w {Variable and Constant List} \
	    {Select the variable or constant to be inserted} \
	    "$variables" "FEditor::sendvarselc $w.f.t"
}

proc FEditor::openoperlist { w } {

    variable operators

    ListBoxWindow $w {Operator List} \
	    {Select the variable or constant to be inserted} \
	    "$operators" "FEditor::sendoperselc $w.f.t"
}

proc FEditor::openhistlist { w } {

    variable used_functions

    ListBoxWindow $w {List od used functions} \
	    {Select one of the functions to be inserted} \
	    "$used_functions" "FEditor::sendhistselc $w.f.t"
}

proc FEditor::sendhistselc { w idx } {
    if { $idx != "" } {
	$w insert insert [lindex $FEditor::used_functions $idx]
    } else {
	WarnWin "Error in sendhistlist $idx"
    }
}

proc FEditor::sendfuncselc { w idx } {
    if { $idx != "" } {
	$w insert insert [lindex $FEditor::available_functions $idx]
    } else {
	WarnWin "Error in sendfunclist $idx"
    }
}

proc FEditor::sendvarselc { w idx } {
    if { $idx != "" } {
	$w insert insert [lindex $FEditor::variables $idx]
    } else {
	WarnWin "Error in sendvarselc $idx"
    }
}

proc FEditor::sendoperselc { w idx } {
    if { $idx != "" } {
	$w insert insert [lindex $FEditor::operators $idx]
    } else {
	WarnWin "Error in sendoperselcs $idx"
    }
}

proc FEditor::sendfunction { parent field fieldslist textwidget {tags ""} } {


    variable fninput
    variable ff_names

    set retval [$textwidget get 1.0 end]

    if { !([string length $field] > 0) } {
	set field ""
    }

    if { $field == "" } {
	foreach {ff_field ff_name} $fieldslist {
	    if { "$ff_name" == "$FEditor::fninput" } {
		set field $ff_field
		break
	    }
	}
    }

    if { $field == "" } {

	WarnWin "Please select a Field" $parent

    } else {

	if { [string length [string trim $retval]] > 0 } {


	    regsub -all {\n} $retval {} retval
	    regsub -all {;} $retval {} retval
	    set retval [string trim $retval]

  	    set checkres false
	    set params ""
  	    set params [list "-2D" "-name" "checkres" "-tool" "check_function" "$retval;"]
  	    set checkres [TUI_exectdyn $params]

	    if { $checkres == "" } {
		if { $field != "" && $retval != "" && [lsearch $FEditor::used_functions $retval] == -1} {
		    lappend FEditor::used_functions $retval
		}
		set $field $retval
		destroy $parent
	    } else {
		WarnWin "The function you are trying to insert is not correct. The following details are available:\n$checkres"
	    }

	} else {

	    destroy $parent

	}
    }
}
