namespace eval matfluid {

    global ProgramName
    global tkPriv tcl_platform
    variable WinPath

    # Tdyn settings
    variable show_ransol [IsVarOnTdynDefaults show_ransol 1]
    variable show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    variable show_advect [IsVarOnTdynDefaults show_advect 1]
    variable show_ursolver [IsVarOnTdynDefaults show_ursolver 1]

    # Selected material name
    variable Material_Name

    # Advect Species variables
    variable Species_Rebuild 

    # Advect Species variables
    variable Species_Names
    variable Species_Values
    variable Species_UValues
    # Advect Species variables
    variable Species_Valnames \
	    [list \
	    [list "f1 " F1_Spc func 1 ""] \
	    [list "f2 " F2_Spc func 1 ""] \
	    [list "f3 " F3_Spc func 1 ""] \
	    [list "f4 " F4_Spc func 1 ""] \
	    [list "f5 " F5_Spc func 1 ""] \
	    ]
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    variable Species_UValnames \
	    [list \
	    [list "F2 Units" F2_Units unit 1 concentration] \
	    [list "F3 Units" F3_Units unit 1 concentration] \
	    [list "F4 Units" F4_Units unit 1 concentration] \
	    [list "F5 Units" F5_Units unit 1 concentration] \
	    ]
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    # Advect Species support variables
    variable spcdelerrwarn 0
    variable spcdellasterrwarn 0
    variable spcnlastsel -1
    variable snlist
    variable sname

    # URSolver Species variables
    variable URVariables_Rebuild 
    # URSolver Species variables
    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_UValues
    # URSolver Species variables
    variable URVariables_Valnames \
	    [list \
	    [list "f1t " F1t_Phi func 1] \
	    [list "f1c " F1c_Phi func 1] \
	    [list "f2 " F2_Phi func 1] \
	    [list "f3 " F3_Phi func 1] \
	    [list "f4 " F4_Phi func 1] \
	    ]
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    variable URVariables_UValnames \
	    [list \
	    [list "F1t Phi Units" F1t_Phi_Units unit 1 concentration] \
	    [list "F1c Phi Units" F1c_Phi_Units unit 1 concentration] \
	    [list "F2 Phi Units" F2_Phi_Units unit 1 concentration] \
	    [list "F3 Phi Units" F3_Phi_Units unit 1 concentration] \
	    [list "F4 Phi Units" F4_Phi_Units unit 1 concentration] \
	    ]
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    # URSolver Species support variables
    variable ursdelerrwarn 0
    variable ursdellasterrwarn 0
    variable ursnlastsel -1
    variable ursnlist
    variable ursname

    # List of available Fluid Material Questions in Mat File.
    # Variables names are local script variables and in some 
    # cases do not correspond the MAT file QUESTION name
    if { $ProgramName == "Tdyn2D" } {
	variable Fluid_Questions [list \
		[list Density func 1 ransol]\
		[list Density_Units unit 1 ransol density] \
		[list Viscosity func 1 ransol] \
		[list Viscosity_Units unit 1 ransol viscosity] \
		[list Compressibility func 1 ransol] \
		[list Compressibility_Units unit 1 ransol compressibility] \
		[list AccOX_Field func 1 ransol] \
		[list AccOY_Field func 1 ransol] \
		[list Acc_Units unit 1 ransol acceleration] \
		[list Specific_Heat_Cp func 1 heatrans] \
		[list Cp_Units unit 1 heatrans heat] \
		[list Thermal_Conductivity_k func 1 heatrans] \
		[list k_Units unit 1 heatrans conductivity] \
		[list Flotability func 1 heatrans] \
		[list Heat_Source_Field func 1 heatrans] \
		[list Heat_Source_Units unit 1 heatrans heat] \
		[list Heat_Reaction_Field func 1 heatrans] \
		[list Heat_Reaction_Units unit 1 heatrans heat] \
		[list Concentration_Properties func 0 advect] \
		[list Spc_Units unit 0 advect] \
		[list UR_Variables func 0 ursolver] \
		[list URV_Units unit 0 ursolver] \
		]
    } elseif { $ProgramName == "Tdyn3D" } {
	variable Fluid_Questions [list \
		[list Density func 1 ransol]\
		[list Density_Units unit 1 ransol density] \
		[list Viscosity func 1 ransol] \
		[list Viscosity_Units unit 1 ransol viscosity] \
		[list Compressibility func 1 ransol] \
		[list Compressibility_Units unit 1 ransol compressibility] \
		[list AccOX_Field func 1 ransol] \
		[list AccOY_Field func 1 ransol] \
		[list AccOZ_Field func 1 ransol] \
		[list Acc_Units unit 1 ransol acceleration] \
		[list Specific_Heat_Cp func 1 heatrans] \
		[list Cp_Units unit 1 heatrans heat] \
		[list Thermal_Conductivity_k func 1 heatrans] \
		[list k_Units unit 1 heatrans conductivity] \
		[list Flotability func 1 heatrans] \
		[list Heat_Source_Field func 1 heatrans] \
		[list Heat_Source_Units unit 1 heatrans heat] \
		[list Heat_Reaction_Field func 1 heatrans] \
		[list Heat_Reaction_Units unit 1 heatrans heat] \
		[list Concentration_Properties func 0 advect] \
		[list Spc_Units unit 0 advect] \
		[list UR_Variables func 0 ursolver] \
		[list URV_Units unit 0 ursolver] \
		]
    }
    foreach item $Fluid_Questions {
	variable [lindex $item 0]
    }
    variable Fluid_UnitsRead [list \
	    [list Density_Units "Fluid_Density_Units#CB#"] \
	    [list Viscosity_Units "Fluid_Viscosity_Units#CB#"] \
	    [list Compressibility_Units "Compressibility_Units#CB#"] \
	    [list Acc_Units "Acc_Units#CB#"] \
	    [list Density_Units "Density_Units#CB#"] \
	    [list Cp_Units "Cp_Units#CB#"] \
	    [list k_Units "k_Units#CB#"] \
	    [list Heat_Source_Units "Heat_Source_Units#CB#"] \
	    [list Heat_Reaction_Units "Heat_Reaction_Units#CB#"] \
    ]
    foreach item $Fluid_UnitsRead {
	variable [lindex $item 0]
    }

    # Function Editor variables lists
    switch $ProgramName {
	"Tdyn2D" {
	    variable ransol_ff \
		    [list \
		    {matfluid::Density} {Density} \
		    {matfluid::Viscosity} {Viscosity} \
		    {matfluid::Compressibility} {Compressibility} \
		    {matfluid::AccOX_Field} {Acceleration Field OX} \
		    {matfluid::AccOY_Field} {Acceleration Field OY} \
		    ]
	}
	"Tdyn3D" {
	    variable ransol_ff \
		    [list \
		    {matfluid::Density} {Density} \
		    {matfluid::Viscosity} {Viscosity} \
		    {matfluid::Compressibility} {Compressibility} \
		    {matfluid::AccOX_Field} {Acceleration Field OX} \
		    {matfluid::AccOY_Field} {Acceleration Field OY} \
		    {matfluid::AccOZ_Field} {Acceleration Field OZ} \
		    ]
	}
    }
    variable heatrans_ff \
	    [ list \
	    {matfluid::Specific_Heat_C} {Specific Heat C} \
	    {matfluid::Thermal_Conductivity_k} {Thermal Conductivity k} \
	    {matfluid::Flotability} {Flotability} \
	    {matfluid::Heat_Source_Field} {Heat Source Field} \
	    {matfluid::Heat_Reaction_Field} {Heat Reaction Field} \
	    ]
    variable advect_ff \
	    [ list \
	    {matfluid::F1_Spc} {f1} \
	    {matfluid::F2_Spc} {f2} \
	    {matfluid::F3_Spc} {f3} \
	    {matfluid::F4_Spc} {f4} \
	    {matfluid::F5_Spc} {f5} \
	    ]
    variable ursolver_ff \
	    [ list \
	    {matfluid::F1t_Phi} {f1t} \
	    {matfluid::F1c_Phi} {f1c} \
	    {matfluid::F2_Phi} {f2} \
	    {matfluid::F3_Phi} {f3} \
	    {matfluid::F4_Phi} {f4} \
	    ]
    variable fluid_variables "x y z nx ny nz t it rand dt exp(1) pi vx vy vz pr tm\
	    vt v_ tau wc ds ke ep om kt pk dn bt sp# ph#"
    variable example_fluid_variables "x y z nx ny nz t it rand dt exp(1)\
	    pi vx vy vz pr tm vt v_ tau wc ds ke ep om kt pk dn bt sp#"
    variable fluid_operators "+ - * / ^ mod div rmod rdiv max min Dx() Dy() Dz()\
	    ~ < <= > >= = != & |"
    variable predefined_ff [list $fluid_variables $example_fluid_variables $fluid_operators]
}



################################################################################
#
#   Initialize Fluid Material Window
#
################################################################################

proc matfluid::init { BASE selmatname } {

    global ProgramName
    global tkPriv tcl_platform

    variable WinPath $BASE

    # Tdyn settings
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    # Selected Material Name
    variable Material_Name $selmatname
    # Variables Lists
    variable Fluid_Questions
    foreach item $Fluid_Questions {
	variable [lindex $item 0]
    }
    # Function Editor Variables
    variable ransol_ff
    variable heatrans_ff
    variable advect_ff
    variable ursolver_ff
    variable fluid_variables
    variable example_fluid_variables
    variable fluid_operators
    variable predefined_ff

    # Advect Species support variables
    variable Species_Rebuild 
    variable Species_Names
    variable Species_Values
    variable Species_Valnames
    variable snlist

    # URSolver support variables
    variable URVariables_Rebuild 
    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_Valnames
    variable ursnlist

    # Initialise support variables
    set Species_Rebuild ""
    set URVariables_Rebuild ""

    # Read values of selected material
    matfluid::getvalues

    #####  HELP Text variables #####
    set help_btn_rnsic "Define Initial Conditions."
    set help_btn_solvdat "Define Solver Data."
    set help_btn_func "Custom Function Editor.\n\
	    Edit a function and transfer it automatically to a field.\n\
	    You can also RIGHT-CLICK a field to open the Function Editor."
    # RANSOL Fields
    set help_density "Density of the fluid."
    set help_viscosity "Viscosity of the fluid.\n\
	    May be a constant or a funtion ( i.e. 0.001*Tm ).\n\
	    Function editor can be started by pressing 'fx' button."
    set help_compressibility "Compressibility parameter defined by 1/c\u00b2,\n\
	    being c the velocity of sound in the fluid."
    switch $ProgramName {
	"Tdyn2D" {
	    set ht_acc "External acceleration (NOT GRAVITY) acting on fluid (OX OY OZ component).\n\
		    The units are inserted in the Data>Problem_Data>UNITS menu.\n\
		    May be a constant or a function ( i.e. 1.0-x*tt )\n\
		    Function editor can be started by pressing 'fx' button."
	    set labeltextacc "Acceleration Field (X Y)"
	}
	"Tdyn3D" {
	    set ht_acc "External acceleration (NOT GRAVITY) acting on fluid (OX OY component).\n\
		    The units are inserted in the Problem_Data menu.\n\
		    May be a constant or a function ( i.e. 1.0-x*tt )\n\
		    Function editor can be started by pressing 'fx' button."
	    set labeltextacc "Acceleration Field (X Y Z)"
	}
    }
    # HEATRANS Fields
    set help_specificheatcp "Specific Heat (Cp) of the Fluid."
    set help_thermalconductivity "Thermal Conductivity (k) of the Fluid."
    set help_thermalexpansion "Flotability of the Fluid\n\
	    ( to take into account a thermal flotability effect, i.e. 0.001*(Tm-25) )"
    set help_heatsource "Heat source for the thermal analysis.\n\
	    May be a constant or a funtion ( i.e. 100*sin(tt) ).\n\
	    Function editor can be started by pressing 'fx' button.\
	    Units are defined in Problem Data>UNITS>General Units window."
    set help_heatreaction "Heat reaction for the thermal analysis.\n\
	    May be a constant or a funtion ( i.e. 1.1e-5*tm^3 ).\n\
	    Function editor can be started by pressing 'fx' button.\
	    Units are defined in Problem Data>UNITS>General Units window."
    # ADVECT Fields
    set help_Advect "ADVECT module data.\n\
	    Select species from the list and edit their properties.\n\
	    New species may be created by pressing '+' button.\n\
	    It is also possible to delete current species by pressing '-' button.\n\
	    Changing species names or their number will affect all existing fluid materials."
    set help_Advect_Pic "ADVECT module data.\n\
	    Properties of the current species may be inserted below.\n\
	    Properties names correspond to those given in the equation."
    # URSOLVER Fields
    set help_URSolver "URSOLVER module data.\n\
	    Select variables from the list and edit their properties.\n\
	    New variables may be created by pressing '+' button.\n\
	    It is also possible to delete current variables by pressing '-' button.\n\
	    Changing variable names or their number will affect all existing fluid materials."
    set help_URSolver_Pic "URSOLVER module data.\n\
	    Properties of the current variable may be inserted below.\n\
	    Properties names correspond to those given in the equation."

    # Main Notebook pages GRID
    set pages [NoteBook $BASE.pages -arcradius 0]
    if { $show_ransol == 1 } {
	set page1 [$pages insert 0 page1 -text "RANSOL"]
	set fpage1 [$pages getframe page1]
    }
    if { $show_heatrans == 1 } {
	set page2 [$pages insert 1 page2 -text "HEATRANS"]
	set fpage2 [$pages getframe page2]
    }
    if { $show_advect == 1 } {
	set page3 [$pages insert 2 page3 -text "ADVECT"]
	set fpage3 [$pages getframe page3]
    }
    if { $show_ursolver == 1 } {
	set page4 [$pages insert 3 page4 -text "URSOLVER"]
	set fpage4 [$pages getframe page4]
    }

    if { $show_ransol == 1 } {
	$pages raise page1
    } elseif { $show_heatrans == 1 } {
	$pages raise page2
    } elseif { $show_advect == 1 } {
	$pages raise page3
    } elseif { $show_ursolver == 1 } {
	$pages raise page4
    }

    # GRIDS PAGES
    grid $pages -row 1 -column 0 -sticky nwes -pady 5
    if { $show_ransol == 1 } {
	grid columnconf $fpage1 10 -weight 1
	grid rowconf $fpage1 10 -weight 1
    }
    if { $show_heatrans == 1 } {
	grid columnconf $fpage2 10 -weight 1
	grid rowconf $fpage2 10 -weight 1
    }
    if { $show_advect == 1 } {
	grid columnconf $fpage3 1 -weight 1
	grid rowconf $fpage3 1 -weight 1
    }
    if { $show_ursolver == 1 } {
	grid columnconf $fpage4 1 -weight 1
	grid rowconf $fpage4 1 -weight 1
    }


    # Density Units is outside of the show_ransol condition because 
    # it is also used in HEATRANS if RANSOL is hidden. 
    lappend units_dens "Kg/m\u00b3" "ton/m\u00b3" "g/lt" "Kg/l" "Kg/cc" "g/cc"

    if { $show_ransol == 1 } {

	#--------------------------- PAGE RANSOL ---------------------------#
	
	# Combo boxes Units definitions
	lappend units_visc "Kg/m.s" "Pa.s" "ton/m.s" "g/m.s"
	lappend units_cprs "s\u00b2/m\u00b2" "s\u00b2/cm\u00b2" "cs\u00b2/m\u00b2"
	lappend units_acc "m/s\u00b2" "G"
	
	# Invisible size label
	set sizelabel [label $fpage1.l_sizelabel \
		-text "                                   " -bd 0 ]
	
	# Density Field
	set l_dens [Label $fpage1.l_dens -text "Density " -helptext $help_density]
	set e_dens [entry $fpage1.e_dens -textvariable matfluid::Density]
	set cbx_dens [ComboBox $fpage1.lb_dens -values $units_dens \
		-editable true -width 11 -textvariable matfluid::Density_Units]
	
	# Viscosity Field
	set l_visc [Label $fpage1.l_visc -text "Viscosity " -helptext $help_viscosity]
	set e_visc [entry $fpage1.e_visc -textvariable matfluid::Viscosity]
	set cbx_visc [ComboBox $fpage1.lb_visc -values $units_visc \
		-editable true -width 11 -textvariable matfluid::Viscosity_Units]
	
	# Compressibility Field
	set l_cprs [Label $fpage1.l_cprs -text "Compressibility " -helptext $help_compressibility]
	set e_cprs [entry $fpage1.e_cprs -textvariable matfluid::Compressibility]
	set cbx_cprs [ComboBox $fpage1.lb_cprsc -values $units_cprs \
		-editable true -width 11 -textvariable matfluid::Compressibility_Units]

	# Acceleration Field
	set f_acc [frame $fpage1.f_acc ]
	set l_acc [Label $fpage1.l_acc -text $labeltextacc -helptext $ht_acc ]
	set e_accx [entry $f_acc.e_accx -textvariable matfluid::AccOX_Field -width 6]
	set e_accy [entry $f_acc.e_accy -textvariable matfluid::AccOY_Field -width 6]
	if { $ProgramName == "Tdyn3D" } {
	    set e_accz [entry $f_acc.e_accz -textvariable matfluid::AccOZ_Field -width 6]
	}
	set cbx_acc [ComboBox $fpage1.lb_acc -values $units_acc \
		-editable true -width 11 -textvariable matfluid::Acc_Units]

	# Buttons frame
	set btn_frame [frame $fpage1.btn_frame]

	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$ransol_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions
	set btn_rnsic [Button $btn_frame.btn_rnsic -underline 0 \
		-command "matfluid::callinitdata $BASE \"\" ransol" \
		-image $buttons::ibtn_initcond -helptext $help_btn_rnsic -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matfluid::callsolverdata $BASE \"\" ransol" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]
	
	# GRIDS
	# SizeLabel
	grid $sizelabel -row 0 -column 0 -sticky en
	# Density
	grid $l_dens -row 1 -column 0 -sticky en -pady 2
	grid $e_dens -row 1 -column 1 -sticky wn -pady 2
	grid $cbx_dens -row 1 -column 2 -sticky wn -pady 1 -padx 7
	# Viscosity
	grid $l_visc -row 2 -column 0 -sticky en -pady 2
	grid $e_visc -row 2 -column 1 -sticky wn -pady 2
	grid $cbx_visc -row 2 -column 2 -sticky wn -pady 1 -padx 7
	# Compressibility
	grid $l_cprs -row 3 -column 0 -sticky en -pady 2
	grid $e_cprs -row 3 -column 1 -sticky wn -pady 2
	grid $cbx_cprs -row 3 -column 2 -sticky wn -pady 1 -padx 7
	# Acc Field
	grid $l_acc -row 4 -column 0 -sticky en -pady 2
	grid $f_acc -row 4 -column 1 -sticky wn -pady 2
	grid $e_accx -row 0 -column 0 -sticky wn -padx 1
	grid $e_accy -row 0 -column 1 -sticky wn -padx 1
	if { $ProgramName == "Tdyn3D" } {
	    grid $e_accz -row 0 -column 2 -sticky wn -padx 1
	}
	grid $cbx_acc -row 4 -column 2 -sticky wn -pady 1 -padx 7
	
	# Buttons frame
	grid $btn_frame -row 5 -column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -pady 2 -padx 1
	# Initial Conditions Button
	grid $btn_rnsic -row 0 -column 1 -sticky en -pady 2 -padx 1
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Right Click menus
	bind $e_dens <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Density Density \"\" 1 %X %Y"
	bind $e_visc <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Viscosity Viscosity \"\" 1 %X %Y"
	bind $e_cprs <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Compressibility Compressibility \"\" 1 %X %Y"
	bind $e_accx <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::AccOX_Field \"Acceleration (OX)\" \"\" 1 %X %Y"
	bind $e_accy <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::AccOY_Field \"Acceleration (OY)\" \"\" 1 %X %Y"
	if { $ProgramName == "Tdyn3D" } {
	    bind $e_accz <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::AccOZ_Field \"Acceleration (OZ)\" \"\" 1 %X %Y"
	}
    }

    if { $show_heatrans == 1 } {

	#--------------------------- PAGE HEATRANS ---------------------------#
	
	# Combo boxes Units definitions
	lappend units_shc "J/Kg.Gt" "J/g.Gt"
	lappend units_thc "W/m.Gt" "KW/m.Gt"
	lappend units_heatsrc "Kcal/hour.m\u00b3" "KW/m\u00b3" "cal/hour.m\u00b3" "W/m\u00b3" "cal/s.m\u00b3"
	lappend units_heatrct "Kcal/hour.m\u00b3.Gt" "KW/m\u00b3.Gt" "cal/hour.m\u00b3.Gt" "W/m\u00b3.Gt" "cal/s.m\u00b3.Gt"

	# Invisible label to keep size
	set sizelabel [label $fpage2.l_sizelabel \
		-text "                                   " -bd 0 ]

	# Density Field
	set l_dens [Label $fpage2.l_dens -text "Density " -helptext $help_density]
	set e_dens [entry $fpage2.e_dens -textvariable matfluid::Density]
	set cbx_dens [ComboBox $fpage2.lb_dens -values $units_dens \
		-editable true -width 11 -textvariable matfluid::Density_Units]
	bind $e_dens <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Density \"Density\" \"\" 1 %X %Y"

	# Specific Heat Cp Field
	set l_shc [Label $fpage2.l_shc -text "Specific Heat Cp" -helptext $help_specificheatcp]
	set e_shc [entry $fpage2.e_shc -textvariable matfluid::Specific_Heat_Cp]
	set cbx_shc [ComboBox $fpage2.lb_shc -values  $units_shc \
		-editable true -width 11 -textvariable matfluid::Cp_Units]
	
	# Thermal Conductivity Field
	set l_thc [Label $fpage2.l_thc -text "Thermal Conductivity k" -helptext $help_thermalconductivity]
	set e_thc [entry $fpage2.e_thc -textvariable matfluid::Thermal_Conductivity_k]
	set cbx_thc [ComboBox $fpage2.lb_thc -values $units_thc \
		-editable true -width 11 -textvariable matfluid::k_Units]
	
	# Flotability Field
	set l_tex [Label $fpage2.l_tex -text "Flotability" -helptext $help_thermalexpansion]
	set e_tex [entry $fpage2.e_tex -textvariable matfluid::Flotability]
	
	# Heat Source Field
	set l_heatsrc [Label $fpage2.l_heatsrc -text "Heat Source Field" -helptext $help_heatsource]
	set e_heatsrc [Entry $fpage2.e_heatsrc -textvariable matfluid::Heat_Source_Field]
	set cbx_heatsrc [ComboBox $fpage2.lb_heatsrc -values $units_heatsrc \
		-editable true -width 11 -textvariable matfluid::Heat_Source_Units]
	
	# Heat Reaction Field
	set l_heatrct [Label $fpage2.l_heatrct -text "Heat Reaction Field" -helptext $help_heatreaction]
	set e_heatrct [Entry $fpage2.e_heatrct -textvariable matfluid::Heat_Reaction_Field]
	set cbx_heatrct [ComboBox $fpage2.lb_heatrct -values $units_heatrct \
		-editable true -width 11 -textvariable matfluid::Heat_Reaction_Units]
	
	# Buttons frame
	set btn_frame [frame $fpage2.btn_frame]
	
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -justify left -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$heatrans_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions
	set btn_rnsic [Button $btn_frame.btn_rnsic -underline 0 \
		-command "matfluid::callinitdata $BASE \"\" heatrans" \
		-image $buttons::ibtn_initcond -helptext $help_btn_rnsic -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matfluid::callsolverdata $BASE \"\" heatrans" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# GRIDS
	# SizeLabel
	grid $sizelabel -row 0 -column 0 -sticky en
	# Density
	grid $l_dens -row 1 -column 0 -sticky en -pady 2
	grid $e_dens -row 1 -column 1 -sticky wn -pady 2
	grid $cbx_dens -row 1 -column 2 -sticky wn -pady 1 -padx 7
	# Specific Heat Cp
	grid $l_shc -row 2 -column 0 -sticky en -pady 2
	grid $e_shc -row 2 -column 1 -sticky wn -pady 2
	grid $cbx_shc -row 2 -column 2 -sticky wn -pady 1 -padx 7
	# Thermal Conductivity
	grid $l_thc -row 3 -column 0 -sticky en -pady 2
	grid $e_thc -row 3 -column 1 -sticky wn -pady 2
	grid $cbx_thc -row 3 -column 2 -sticky wn -pady 1 -padx 7
	# Flotability
	grid $l_tex -row 4 -column 0 -sticky en -pady 2
	grid $e_tex -row 4 -column 1 -sticky wn -pady 2
	# Heat Source Field
	grid $l_heatsrc -row 5 -column 0 -sticky en -pady 2
	grid $e_heatsrc -row 5 -column 1 -sticky wn -pady 2
	grid $cbx_heatsrc -row 5 -column 2 -sticky wn -pady 1 -padx 7
	# Heat Reaction Field
	grid $l_heatrct -row 6 -column 0 -sticky en -pady 2
	grid $e_heatrct -row 6 -column 1 -sticky wn -pady 2
	grid $cbx_heatrct -row 6 -column 2 -sticky wn -pady 1 -padx 7
	# Buttons frame
	grid $btn_frame -row 7 -column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -padx 1 -pady 2
	# Initial Conditions Button
	grid $btn_rnsic -row 0 -column 1 -sticky en -padx 1 -pady 2
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	# Right click menus
	bind $e_shc <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Specific_Heat_Cp \"Specific Heat Cp\" \"\" 1 %X %Y"
	bind $e_thc <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Thermal_Conductivity_k \"Thermal Conductivity k\" \"\" 1 %X %Y"
	bind $e_tex <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Flotability \"Flotability\" \"\" 1 %X %Y"
	bind $e_heatsrc <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Heat_Source_Field \"Heat Source Field\" \"\" 1 %X %Y"
	bind $e_heatrct <ButtonPress-3> "matfluid::popupmenu $BASE matfluid::Heat_Reaction_Field \"Heat Reaction Field\" \"\" 1 %X %Y"
    }

    if { $show_advect == 1 } {

	#--------------------------- PAGE ADVECT ---------------------------#

	# Top Label Frame
	set topfr [frame $fpage3.topfr -relief flat]
	# Information label
	set infotext "Changing Species names or their number will affect all existing fluid materials."
	set l_info [Label $topfr.l_info -text $infotext -justify left -wraplength 150]
	GidHelp $l_info $help_Advect

	# Equation label with image
	set l_ieqdf [Label $topfr.l_ieqdf -image $TUI_images::i_fladveq]
	GidHelp $l_ieqdf $help_Advect_Pic

	# Title Frame Species List
	set tfr_spclist [TitleFrame $fpage3.tfr_spclist -relief ridge -bd 3\
		-text "Defined Species" -side left ]
	set frame_spclist [$tfr_spclist getframe]

	# Paned window
	set pw [PanedWindow $frame_spclist.pw]
	set pane1 [$pw add -weight 1]
	set pane2 [$pw add -weight 0]

	# Species Names in scrolled window
	set spc_scrw [ScrolledWindow $pane1.spc_scrw -scrollbar both]
	set snlist [listbox $spc_scrw.snlist -selectbackground lightblue -selectforeground black \
		-selectborderwidth 1 -selectmode extended -height 5 -bg white -width 10]
	$spc_scrw setwidget $snlist

	# Selected Specie values frame
	set svlist [frame $pane2.svlist -relief flat]

	# Buttons frame
	set btn_frame [frame $svlist.btn_frame]
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$advect_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions
	set btn_rnsic [Button $btn_frame.btn_rnsic -underline 0 \
		-command "matfluid::callinitdata $BASE $snlist advect" \
		-image $buttons::ibtn_initcond -helptext $help_btn_rnsic -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matfluid::callsolverdata $BASE $snlist advect" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# Buttons PLUS MINUS - under the Species Names listbox
	set btnbox [ButtonBox $frame_spclist.btnbox -spacing 0 -padx 1 -pady 1 -homogeneous 0]
	$btnbox add -image $buttons::ibtn_plus -width 20 -height 20 \
		-highlightthickness 0 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext "Add New Specie" \
		-command "matfluid::addspecie $snlist $svlist \"advect\""
	$btnbox add -image $buttons::ibtn_minus -width 20 -height 20 \
		-highlightthickness 1 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext "Delete Selected Specie" \
		-command "matfluid::removespecie $snlist $svlist \"advect\""

	# GRIDS
	grid $topfr -row 0 -column 0 -columnspan 3 -sticky nw -pady 10
	grid $l_info -row 0 -column 0 -sticky nw -padx 5
	grid $l_ieqdf -row 0 -column 1 -sticky nw -padx 5
	# Title Frame
	grid $tfr_spclist -row 1 -column 0 -columnspan 5 -sticky wnes
	grid $pw -row 0 -column 0 -sticky wnes
	grid $spc_scrw -row 0 -column 0 -sticky wnes
	grid $btnbox -row 1 -column 0 -sticky wn
	# Buttons frame
	grid $btn_frame -row [expr [llength $Species_Valnames] + 1] \
		-column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -padx 1 -pady 2
	# Initial Conditions Button
	grid $btn_rnsic -row 0 -column 1 -sticky en -padx 1 -pady 2
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	grid rowconf $pane1 0 -weight 1
	grid columnconf $pane1 0 -weight 1
	grid rowconf $pane2 0 -weight 1
	grid columnconf $pane2 0 -weight 1
	grid rowconf $frame_spclist 1 -weight 1
	grid columnconf $frame_spclist 1 -weight 1

	GidHelpRecursive $tfr_spclist $help_Advect

	# BINDS 
	bind $snlist <<ListboxSelect>> ";
	matfluid::valsavespecie $snlist \"last\" \"advect\"; 
	matfluid::displayspecievalues $snlist \"advect\";
	matfluid::displayspeciesnames $snlist \"advect\";
	"
	bind $snlist <ButtonPress-3> "+matfluid::popupmenu $snlist \"\" \"\" \"\" 2 %X %Y"

	# Insert Species Names to listbox
	matfluid::displayspeciesnames $snlist "advect"
	# Selecting first Specie
	matfluid::updatespcselection $snlist 0 "advect"
	# Create Species Values widgets
	matfluid::makespecie $snlist $svlist "advect"
	# Put specie values into entries
	matfluid::displayspecievalues $snlist "advect"

    }

    if { $show_ursolver == 1 } {

	#--------------------------- PAGE URSOLVER ---------------------------#

	# Top Label Frame
	set topfr [frame $fpage4.topfr -relief flat]

	# Information label
	set infotext "Changing Variables names or their number will affect all existing fluid materials."
	set l_info [Label $topfr.l_info -text $infotext -justify left -wraplength 150]
	GidHelp $l_info $help_URSolver

	# Equation label with image
	set l_ieqdf [Label $topfr.l_ieqdf -image $TUI_images::i_flurseq]
	GidHelp $l_ieqdf $help_URSolver_Pic

	# Title Frame Species List
	set tfr_spclist [TitleFrame $fpage4.tfr_spclist -relief ridge -bd 3\
		-text "Defined Variables" -side left ]
	set frame_spclist [$tfr_spclist getframe]

	# Paned window
	set upw [PanedWindow $frame_spclist.pw]
	set upane1 [$upw add -weight 1]
	set upane2 [$upw add -weight 0]

	# Variables Names in scrolled window
	set ursv_scrw [ScrolledWindow $upane1.ursv_scrw -scrollbar both]
	set ursnlist [listbox $ursv_scrw.ursnlist -selectbackground lightblue -selectforeground black \
		-selectborderwidth 1 -selectmode extended -height 5 -bg white -width 10]
	$ursv_scrw setwidget $ursnlist

	# Selected Specie values frame
	set ursvlist [frame $upane2.ursvlist -relief flat]

	# Buttons frame
	set btn_frame [frame $ursvlist.btn_frame]
	# Function Editor Button
	set btn_functions [Button $btn_frame.btn_func -underline 0 \
		-command "FEditor::init $BASE \"$predefined_ff\" \"\" \"SELECT FIELD\" \"\" \"$ursolver_ff\" \"\"" \
		-image $buttons::ibtn_func -helptext $help_btn_func -takefocus 1]
	# Initial Conditions button
	set btn_rnsic [Button $btn_frame.btn_rnsic -underline 0 \
		-command "matfluid::callinitdata $BASE $ursnlist ursolver" \
		-image $buttons::ibtn_initcond -helptext $help_btn_rnsic -takefocus 1]
	# Solver Data button
	set btn_solvdat [Button $btn_frame.btn_solvdat -underline 0 \
		-command "matfluid::callsolverdata $BASE $ursnlist ursolver" \
		-image $buttons::ibtn_solvdat -helptext $help_btn_solvdat -takefocus 1]

	# Buttons PLUS MINUS - under the Species Names listbox
	set btnbox [ButtonBox $frame_spclist.btnbox -spacing 0 -padx 1 -pady 1 -homogeneous 0]
	$btnbox add -image $buttons::ibtn_plus -width 20 -height 20 \
		-highlightthickness 0 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext "Add New Variable" \
		-command "matfluid::addspecie $ursnlist $ursvlist ursolver"
	$btnbox add -image $buttons::ibtn_minus -width 20 -height 20 \
		-highlightthickness 1 -takefocus 0 -relief link -borderwidth 1 \
		-padx 4 -pady 1 -helptext "Delete Selected Variable" \
		-command "matfluid::removespecie $ursnlist $ursvlist ursolver"

	# GRIDS
	grid $topfr -row 0 -column 0 -columnspan 3 -sticky nw -pady 10
	grid $l_info -row 0 -column 0 -sticky nw -padx 5
	grid $l_ieqdf -row 0 -column 1 -sticky nw -padx 5
	# Title Frame
	grid $tfr_spclist -row 1 -column 0 -columnspan 5 -sticky wnes
	grid $upw -row 0 -column 0 -sticky wnes
	grid $ursv_scrw -row 0 -column 0 -sticky wnes
	grid $btnbox -row 1 -column 0 -sticky wn
	# Buttons frame
	grid $btn_frame -row [expr [llength $Species_Valnames] + 1] \
		-column 1 -columnspan 2 -sticky wn
	# Function Editor Button
	grid $btn_functions -row 0 -column 0 -sticky en -padx 1 -pady 2
	# Initial Conditions Button
	grid $btn_rnsic -row 0 -column 1 -sticky en -padx 1 -pady 2
	# Solver Data Button
	grid $btn_solvdat -row 0 -column 2 -sticky en -padx 1 -pady 2
	grid rowconf $upane1 0 -weight 1
	grid columnconf $upane1 0 -weight 1
	grid rowconf $upane2 0 -weight 1
	grid columnconf $upane2 0 -weight 1
	grid rowconf $frame_spclist 1 -weight 1
	grid columnconf $frame_spclist 1 -weight 1

	# BINDS 
	bind $ursnlist <<ListboxSelect>> ";
	matfluid::valsavespecie $ursnlist \"last\" \"ursolver\"; 
	matfluid::displayspecievalues $ursnlist \"ursolver\";
	matfluid::displayspeciesnames $ursnlist \"ursolver\";
	"
	bind $ursnlist <ButtonPress-3> "+matfluid::popupmenu $ursnlist \"\" \"\" \"\" 2 %X %Y"

	GidHelpRecursive $tfr_spclist $help_URSolver

	# Insert Species Names to listbox
	matfluid::displayspeciesnames $ursnlist "ursolver"
	# Selecting first Specie
	matfluid::updatespcselection $ursnlist 0 "ursolver"
	# Create Species Values widgets
	matfluid::makespecie $ursnlist $ursvlist "ursolver"
	# Put specie values into entries
	matfluid::displayspecievalues $ursnlist "ursolver"
    }

    if { [info exists snlist] == 1 && [info exists ursnlist] == 1 } {
	$pages bindtabs <Button-1> "matfluid::selectpage $snlist $ursnlist"
    } elseif { [info exists snlist] == 1 && [info exists ursnlist] != 1 } {
	$pages bindtabs <Button-1> "matfluid::selectpage $snlist \"\""
    } elseif { [info exists snlist] != 1 && [info exists ursnlist] == 1 } {
	$pages bindtabs <Button-1> "matfluid::selectpage \"\" $ursnlist"
    }
}

proc matfluid::selectpage { snlist ursnlist {pg ""} } {
    
    variable spcnlastsel
    variable ursnlastsel

    set nlist ""

    if { $pg == "page3" } {
	set type "advect"
	set nlist $snlist
	set setsel $spcnlastsel
	set leavenlist $ursnlist
    } elseif { $pg == "page4" } {
	set type "ursolver"
	set nlist $ursnlist
	set setsel $ursnlastsel
	set leavenlist $snlist
    }

    if { [string length $nlist] > 0 && ( $type == "advect" || $type == "ursolver") } {
	matfluid::valsavespecie $leavenlist "" $type
	matfluid::updatespcselection $nlist $setsel $type
	matfluid::displayspecievalues $nlist $type
    }

    return 1
}


proc matfluid::callinitdata { parent nlist type } {

    variable spcnlastsel
    variable ursnlastsel

    if { [string length $nlist] > 0 && ($type == "advect" || $type == "ursolver") } {
	matfluid::valsavespecie $nlist "actual" $type
    }
    set datachange [matfluid::checkclose $parent 0 0 0]

    if { $datachange != 0 } {
	set answer ""
	set answer [tk_dialogRAM $parent "Save Data?" \
		"To open the Initial Data Window, edited values have to be saved. Do you want to save them?"\
		question 0 Yes No]
    } else {
	set answer 0
    }

    if { $answer == 0 } {
	set saveres ""
	if { $datachange != 0 } {
	    # Save edited data
	    set saveres [matfluid::checkclose $parent 1 0 0]
	}

	if { $saveres != "dataerror" } {
	    switch $type {
		"ransol" {
		    initdatwin::init .gid $type ""
		}
		"heatrans" {
		    initdatwin::init .gid $type ""
		}
		"advect" {
		    initdatwin::init .gid $type $spcnlastsel
		}
		"ursolver" {
		    initdatwin::init .gid $type $ursnlastsel
		}
	    }
	}
    }

    return 1
}

proc matfluid::callsolverdata { parent nlist type } {

    variable spcnlastsel
    variable ursnlastsel

    if { [string length $nlist] > 0 && ($type == "advect" || $type == "ursolver") } {
	matfluid::valsavespecie $nlist "actual" $type
    }
    set datachange [matfluid::checkclose $parent 0 0 0]

    if { $datachange != 0 } {
	set answer ""
	set answer [tk_dialogRAM $parent "Save Data?" \
		"To open the Solver Data Window, edited values have to be saved. Do you want to save them?"\
		question 0 Yes No]
    } else {
	set answer 0
    }

    if { $answer == 0 } {
	set saveres ""
	if { $datachange != 0 } {
	    # Save edited data
	    set saveres [matfluid::checkclose $parent 1 0 0]
	}

	if { $saveres != "dataerror" } {
	    switch $type {
		"ransol" {
		    solvdatwin::init .gid $type ""
		}
		"heatrans" {
		    solvdatwin::init .gid $type ""
		}
		"advect" {
		    solvdatwin::init .gid $type $spcnlastsel
		}
		"ursolver" {
		    solvdatwin::init .gid $type $ursnlastsel
		}
	    }
	}
    }

    return 1
}




################################################################################
#
#   Species Manipulation
#   The following functions are used to manipulate the ADVECT Species Data
#   as well as the URSolver Variables Data.
#
################################################################################

proc matfluid::makespecie { nlist vlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    # Combo boxes Units definitions
    # ADVECT
    lappend units_F2_Units "m\u00b2/s" "m\u00b2/hour"
    lappend units_F3_Units "1/s" "1/hour"
    lappend units_F4_Units "1/s" "1/hour"
    lappend units_F5_Units "1/s" "1/hour"
    # URSOLVER
    lappend units_F1t_Phi_Units "1/m\u00b3" "1/cm\u00b2"
    lappend units_F1c_Phi_Units "1/m\u00b3" "1/cm\u00b2"
    lappend units_F2_Phi_Units "1/m.s" "1/cm.s"
    lappend units_F3_Phi_Units "1/m\u00b3.s" "1/cm\u00b3.s"
    lappend units_F4_Phi_Units "pu/m\u00b3.s" "pu/cm\u00b3.s"

    # Grid Values Frame
    grid $vlist -row 0 -column 1 -sticky wnes -padx 15

    switch $type {
	"advect" {
	    # Specie Name
	    set l_sname [Label $vlist.l_sname -text "   Specie Name "]
	    set e_sname [entry $vlist.e_sname -width 21 -textvariable matfluid::sname]
	}
	"ursolver" {
	    # Variable Name
	    set l_sname [Label $vlist.l_vname -text "Variable Name "]
	    set e_sname [entry $vlist.e_vname -width 21 -textvariable matfluid::ursname]
	}
    }

    # GRIDS
    # Label
    grid $l_sname -row 0 -column 0 -sticky en -pady 5
    # Entry
    grid $e_sname -row 0 -column 1 -columnspan 2 -sticky wn -pady 5

    # BINDS
    bind $e_sname <Return> ";
    matfluid::valsavespecie $nlist \"actual\" \"$type\";
    matfluid::displayspeciesnames $nlist \"$type\";
    "
    bind $e_sname <ButtonPress-3> "+matfluid::popupmenu $vlist \"\" \"\" \"\" 2 %X %Y"

    switch $type {
	"advect" {
	    set type_Valnames $Species_Valnames
	    set type_UValnames $Species_UValnames
	    set cbmove 2
	}
	"ursolver" {
	    set type_Valnames $URVariables_Valnames
	    set type_UValnames $URVariables_UValnames
	    set cbmove 1
	}
    }

    # Creating widgets
    for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	set tvar [lindex [lindex $type_Valnames $i] 1]
	set tname [lindex [lindex $type_Valnames $i] 0]
	# Defining a Label
	set lvar l_$tvar
	set $lvar [Label $vlist.$lvar -text $tname]
	# Defining an Entry
	set evar e_$tvar
	set $evar [entry $vlist.$evar -width 11 -textvariable matfluid::$tvar]

	# GRIDS
	# Label
	grid [eval set temp $$lvar] -row [expr $i+1] -column 0 -sticky en -pady 1
	# Entry
	grid [eval set temp $$evar] -row [expr $i+1] -column 1 -sticky wn -pady 1

	# BINDS
	bind [eval set temp $$evar] <Return> ";
	matfluid::valsavespecie $nlist \"actual\" \"$type\";
	matfluid::displayspeciesnames $nlist \"$type\";
	"
	bind [eval set temp $$evar] <ButtonPress-3> "+matfluid::popupmenu $vlist matfluid::$tvar \"$tname\" \"\" 1 %X %Y"
    }
    for { set i 0 } { $i < [llength $type_UValnames] } { incr i } {
	set tuvar [lindex [lindex $type_UValnames $i] 1]
	set cuvar "units_"
	append cuvar [lindex [lindex $type_UValnames $i] 1]
	# Defining Units Combobox
	set cvar c_$tuvar
	set $cvar [ComboBox $vlist.$cvar -values [eval set temp $$cuvar] \
		-editable 1 -width 7 -textvariable matfluid::$tuvar -takefocus 1]

	# GRIDS
	# Units Combo
	grid [eval set temp $$cvar] -row [expr $i+$cbmove] -column 2 -sticky wn -padx 2
    }

    return 1
}

proc matfluid::displayspecievalues { nlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    if { [lindex [$nlist curselection] 0] > -1 } {
	set sel [lindex [$nlist curselection] 0]
    } else {
	matfluid::updatespcselection $nlist 0 $type
	set sel [lindex [$nlist curselection] 0]
    }

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set type_Values $Species_Values
	    set type_Valnames $Species_Valnames
	    set type_UValnames $Species_UValnames
	    set type_UValues $Species_UValues
	    set vartype_name "sname"
	    set vartype_lastsel "spcnlastsel"
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set type_Values $URVariables_Values
	    set type_Valnames $URVariables_Valnames
	    set type_UValues $URVariables_UValues
	    set type_UValnames $URVariables_UValnames
	    set vartype_name "ursname"
	    set vartype_lastsel "ursnlastsel"
	}
    }

    # Display Specie / Variable name and set last selection
    set $vartype_name [lindex $type_Names $sel]
    set $vartype_lastsel $sel

    set currvals [lindex $type_Values $sel]
    for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	# Entries Values
	set tvar [lindex [lindex $type_Valnames $i] 1]
	set $tvar [lindex $currvals $i]
    }
    set currvals [lindex $type_UValues $sel]
    for { set i 0 } { $i < [llength $type_UValnames] } { incr i } {
	# Combo boxes Units values
	set tvar [lindex [lindex $type_UValnames $i] 1]
	set $tvar [lindex $currvals $i]
    }

    return 1
}

proc matfluid::displayspeciesnames { nlist type } {

    variable Species_Names
    variable URVariables_Names

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	}
    }

    if { [lindex [$nlist curselection] 0] > -1 } {
	set savesel [lindex [$nlist curselection] 0]
    } else {
	set savesel ""
    }
    set p 0
    $nlist delete 0 end
    foreach n $type_Names {
	$nlist insert $p $n
	incr p
    }

    if { $savesel > -1 } {
	$nlist selection clear 0 end
	$nlist selection set $savesel $savesel
    }

    return 1
}

proc matfluid::updatespcselection { nlist setsel type } {

    variable Species_Names
    variable URVariables_Names
    variable spcnlastsel
    variable ursnlastsel
    variable sname
    variable ursname

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	}
    }

    if { $setsel < 0 || $setsel > [expr [llength $type_Names] -1] } {
	set setsel "end"
    }

    $nlist selection clear 0 end
    $nlist selection set $setsel $setsel

    switch $type {
	"advect" {
	    set spcnlastsel $setsel
	    set sname [lindex $type_Names $setsel]
	}
	"ursolver" {
	    set ursnlastsel $setsel
	    set ursname [lindex $type_Names $setsel]
	}
    }

    return 1
}

proc matfluid::valsavespecie { nlist sel type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable Species_Valnames
    variable Species_UValnames
    for { set i 0 } { $i < [llength $Species_Valnames] } { incr i } {
	set tvar [lindex [lindex $Species_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $Species_UValnames] } { incr i } {
	set tvar [lindex [lindex $Species_UValnames $i] 1]
	variable $tvar
    }
    variable spcnlastsel
    variable sname
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable URVariables_Valnames
    variable URVariables_UValnames
    for { set i 0 } { $i < [llength $URVariables_Valnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_Valnames $i] 1]
	variable $tvar
    }
    for { set i 0 } { $i < [llength $URVariables_UValnames] } { incr i } {
	set tvar [lindex [lindex $URVariables_UValnames $i] 1]
	variable $tvar
    }
    variable ursnlastsel
    variable ursname

    if { $sel == "actual" && [string length [$nlist curselection]] > 0 } {
	set sel [lindex [$nlist curselection] 0]
    } else {
	switch $type {
	    "advect" {
		set sel $spcnlastsel
	    }
	    "ursolver" {
		set sel $ursnlastsel
	    }
	}
    }

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set vartype_Names "Species_Names"
	    set type_Values $Species_Values
	    set vartype_Values "Species_Values"
	    set type_Valnames $Species_Valnames
	    set type_UValnames $Species_UValnames
	    set type_UValues $Species_UValues
	    set vartype_UValues "Species_UValues"
	    set type_name $sname
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set vartype_Names "URVariables_Names"
	    set type_Values $URVariables_Values
	    set vartype_Values "URVariables_Values"
	    set type_Valnames $URVariables_Valnames
	    set type_UValnames $URVariables_UValnames
	    set type_UValues $URVariables_UValues
	    set vartype_UValues "URVariables_UValues"
	    set type_name $ursname
	}
    }

    if { $sel > -1 && !($sel > [expr [llength $type_Values] -1]) } {

	# Saving Specie / UR Variable Name
	set $vartype_Names [lreplace $type_Names $sel $sel [string trim [DWSpace2Under $type_name]]]

	# Saving Specie / UR Variable values
	set newvals ""
	for { set i 0 } { $i < [llength $type_Valnames] } { incr i } {
	    set tvar [lindex [lindex $type_Valnames $i] 1]
	    lappend newvals [eval set temp $$tvar]
	}
	set $vartype_Values [lreplace $type_Values $sel $sel $newvals]
	# Saving Specie / UR Variable Units values
	set newvals ""
	for { set i 0 } { $i < [llength $type_UValnames] } { incr i } {
	    set tvar [lindex [lindex $type_UValnames $i] 1]
	    lappend newvals [eval set temp $$tvar]
	}
	set $vartype_UValues [lreplace $type_UValues $sel $sel $newvals]
    }

    return 1
}

proc matfluid::addspecie { nlist vlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable spcnlastsel
    variable sname
    variable Species_Rebuild
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable ursnlastsel
    variable ursname
    variable URVariables_Rebuild

    matfluid::valsavespecie $nlist "last" $type

    set sr [$nlist curselection]

    switch $type {
	"advect" {
	    # Update Species Values
	    lappend Species_Values [lindex $Species_Values $sr]
	    # Update Species Units Values
	    lappend Species_UValues [lindex $Species_UValues $sr]
	    
	    # Update Species Names
	    set nn "Specie"
	    set ns [expr $sr + 1]
	    lappend Species_Rebuild "a" $sr
	    set nt $nn$ns
	    while { [lsearch $Species_Names $nt]!=-1 } {
		set ns [expr $ns + 1]
		set nt $nn$ns
	    }
	    set nt $nn$ns
	    lappend Species_Names $nt
	    $nlist insert end $nt
	}
	"ursolver" {
	    # Update URSolver Variables Values
	    lappend URVariables_Values [lindex $URVariables_Values $sr]
	    # Update URSolver Units Values
	    lappend URVariables_UValues [lindex $URVariables_UValues $sr]
	    
	    # Update URSolver Variables Names
	    set nn "Variable"
	    set ns [expr $sr + 1]
	    lappend URVariables_Rebuild "a" $sr
	    set nt $nn$ns
	    while { [lsearch $URVariables_Names $nt]!=-1 } {
		set ns [expr $ns + 1]
		set nt $nn$ns
	    }
	    set nt $nn$ns
	    lappend URVariables_Names $nt
	    $nlist insert end $nt
	}        
    }
    matfluid::updatespcselection $nlist "end" $type
    matfluid::displayspecievalues $nlist $type
}

proc matfluid::removespecie { nlist vlist type } {

    # Advect Variables
    variable Species_Values
    variable Species_UValues
    variable Species_Names
    variable spcnlastsel
    variable sname
    variable spcdelerrwarn
    variable spcdellasterrwarn
    variable Species_Rebuild
    # URSolver Variables
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Names
    variable ursnlastsel
    variable ursname
    variable ursdelerrwarn
    variable ursdellasterrwarn
    variable URVariables_Rebuild

    set sr [$nlist curselection]

    switch $type {
	"advect" {
	    set type_Names $Species_Names
	    set vartype_Names Species_Names
	    set type_Values $Species_Values
	    set vartype_Values Species_Values
	    set type_UValues $Species_UValues
	    set vartype_UValues Species_UValues
	    set displaykind "Specie"
	    set lastcheckvar $spcdellasterrwarn
	    set var_lastcheckvar spcdellasterrwarn
	    set noselvar $spcdelerrwarn
	    set var_noselvar spcdelerrwarn
	    set type_Rebuild Species_Rebuild
	}
	"ursolver" {
	    set type_Names $URVariables_Names
	    set vartype_Names URVariables_Names
	    set type_Values $URVariables_Values
	    set vartype_Values URVariables_Values
	    set type_UValues $URVariables_UValues
	    set vartype_UValues URVariables_UValues
	    set displaykind "Variable"
	    set lastcheckvar $ursdellasterrwarn
	    set var_lastcheckvar ursdellasterrwarn
	    set noselvar $ursdelerrwarn
	    set var_noselvar ursdelerrwarn
	    set type_Rebuild URVariables_Rebuild
	}
    }

    if { $sr > -1 && $sr < [llength $type_Names] } {
	# Check if the deleted Specie/Variable is not the last one
	if { [llength $type_Names] < 2 } {
	    if { $lastcheckvar == 1 } {
		return
	    } else {
		WarnWin "There must be at least one $displaykind defined."
		set $var_lastcheckvar 1
	    }
	} else {
	    lappend $type_Rebuild "r" $sr
	    # Updating Species/URVariables Names
	    set $vartype_Names [lreplace $type_Names [lindex $sr 0] [lindex $sr end]]
	    # Updating Species/URVariables Values
	    set $vartype_Values [lreplace $type_Values [lindex $sr 0] [lindex $sr end]]
	    # Updating Species/URVariables Units Values
	    set $vartype_UValues [lreplace $type_UValues [lindex $sr 0] [lindex $sr end]]

	    matfluid::displayspeciesnames $nlist $type
	    matfluid::updatespcselection $nlist "end" $type
	}
    } else {
	if { $noselvar == 1 } {
	    return
	} else {
	    set temp "Please select a "
	    append temp $displaykind "(s) you wish to remove."
	    WarnWin $temp
	    set $noselvar 1
	}
    }

    matfluid::displayspecievalues $nlist $type
}

proc matfluid::formatspecies { way } {

    variable show_advect
    variable show_ursolver

    # ADVECT Variables
    variable Concentration_Properties
    variable Spc_Units
    variable Species_Names
    variable Species_Values
    variable Species_UValues
    variable Species_Valnames
    variable Species_UValnames
    # URSolver Variables
    variable UR_Variables
    variable URV_Units
    variable URVariables_Names
    variable URVariables_Values
    variable URVariables_UValues
    variable URVariables_Valnames
    variable URVariables_UValnames

    switch $way {
	"saving" {
	    if { $show_advect == 1 } {
		# ADVECT Species values
		set tn [expr [llength $Species_Valnames] * [llength $Species_Values]]
		set Concentration_Properties "#N# $tn"
		foreach sv $Species_Values {
		    foreach svv $sv {
		        append Concentration_Properties " " [string trim $svv]
		    }
		}
		set Concentration_Properties [string trim $Concentration_Properties]

		# ADVECT Species units values
		set tn [expr [llength $Species_UValnames] * [llength $Species_UValues]]
		set Spc_Units "#N# $tn"
		foreach sv $Species_UValues {
		    foreach svv $sv {
		        append Spc_Units " " [string trim $svv]
		    }
		}
		set Spc_Units [string trim $Spc_Units]
	    }
	    if { $show_ursolver == 1 } {
		# URSOLVER Species values
		set tn [expr [llength $URVariables_Valnames] * [llength $URVariables_Values]]
		set UR_Variables "#N# $tn"
		foreach sv $URVariables_Values {
		    foreach svv $sv {
		        append UR_Variables " " [string trim $svv]
		    }
		}
		set UR_Variables [string trim $UR_Variables]

		# URSOLVER Species units values
		set tn [expr [llength $URVariables_UValnames] * [llength $URVariables_UValues]]
		set URV_Units "#N# $tn"
		foreach sv $URVariables_UValues {
		    foreach svv $sv {
		        append URV_Units " " [string trim $svv]
		    }
		}
		set URV_Units [string trim $URV_Units]
	    }
	}
	"retrieving" {
	    if { $show_advect == 1 } {
		# ADVECT Species values
		set valstr [lreplace $Concentration_Properties 0 1]
		set Species_Values ""
		for { set i 0 } { $i < [expr [llength $valstr] / [llength $Species_Valnames]] } { incr i } {
		    set aa [lrange $valstr [expr $i * [llength $Species_Valnames]] \
		            [expr [expr ($i+1) * [llength $Species_Valnames]] -1]]
		    lappend Species_Values $aa
		}
		# ADVECT Species units values
		set valstr [lreplace $Spc_Units 0 1]
		set Species_UValues ""
		for { set i 0 } { $i < [expr [llength $valstr] / [llength $Species_UValnames]] } { incr i } {
		    set aa [lrange $valstr [expr $i * [llength $Species_UValnames]] \
		            [expr [expr ($i+1) * [llength $Species_UValnames]] -1]]
		    lappend Species_UValues $aa
		}
	    }
	    if { $show_ursolver == 1 } {
		# URSOLVER Species values
		set valstr [lreplace $UR_Variables 0 1]
		set URVariables_Values ""
		for { set i 0 } { $i < [expr [llength $valstr] / [llength $URVariables_Valnames]] } { incr i } {
		    set aa [lrange $valstr [expr $i * [llength $URVariables_Valnames]] \
		            [expr [expr ($i+1) * [llength $URVariables_Valnames]] -1]]
		    lappend URVariables_Values $aa
		}
		# URSOLVER Species units values
		set valstr [lreplace $URV_Units 0 1]
		set URVariables_UValues ""
		for { set i 0 } { $i < [expr [llength $valstr] / [llength $URVariables_UValnames]] } { incr i } {
		    set aa [lrange $valstr [expr $i * [llength $URVariables_UValnames]] \
		            [expr [expr ($i+1) * [llength $URVariables_UValnames]] -1]]
		    lappend URVariables_UValues $aa
		}
	    }
	}
    }
}

################################################################################
#
#   PopUp Menus
#
################################################################################

proc matfluid::popupmenu { parent {field ""} {fieldname ""} tags content x y } {

    variable predefined_ff

    catch { 
	destroy $parent.menu
    }

    set menu [menu $parent.menu -type normal]

    switch $content {
	1 {
	    # $menu add command -label "Help" -command matfluid::definehelptagshere-stillmissing
	    if { [string length $field] > 0 } {
		eval set fval $$field
	    }
	    $menu add command -label "Define Function" \
		    -command "FEditor::init $parent \"$predefined_ff\" $field \"$fieldname\" \"$fval\" \"\" \"\""
	}
    }
    
    tk_popup $menu $x $y

    return 1
}



################################################################################
#
#   Format Units - unicode character to numbers and the other way
#
################################################################################

proc matfluid::formatunits { way } {

    variable Density_Units
    variable Viscosity_Units
    variable Compressibility_Units
    variable Cp_Units
    variable k_Units
    variable Acc_Units
    variable Heat_Source_Units
    variable Heat_Reaction_Units
    variable Species_UValues
    variable URVariables_UValues

    array set tempvalues [ list \
	    Density_Units $Density_Units \
	    Viscosity_Units $Viscosity_Units \
	    Compressibility_Units $Compressibility_Units \
	    Cp_Units $Cp_Units \
	    k_Units $k_Units \
	    Acc_Units $Acc_Units \
	    Heat_Source_Units $Heat_Source_Units \
	    Heat_Reaction_Units $Heat_Reaction_Units \
	    ]

    set tempnames [array names tempvalues]

    switch $way {
	"frommatfile" {
	    foreach item $tempnames {
		regsub -all "2" $tempvalues($item) "\u00b2" tempvalues($item)
		regsub -all "3" $tempvalues($item) "\u00b3" tempvalues($item)
		regsub -all {\[} $tempvalues($item) "" tempvalues($item)
		regsub -all {\]} $tempvalues($item) "" tempvalues($item)
		set $item $tempvalues($item)
	  }
	}
	"tomatfile" {
	    foreach item $tempnames {
		regsub -all "\u00b2" $tempvalues($item) "2" tempvalues($item)
		regsub -all "\u00b3" $tempvalues($item) "3" tempvalues($item)
		set $item "\[$tempvalues($item)\]"
	    }
	}
	"spc_frommatfile" {
	    set type_UValues $Species_UValues
	    set vartype_UValues Species_UValues
	}
	"spc_tomatfile" {
	    set type_UValues $Species_UValues
	    set vartype_UValues Species_UValues
	}
	"urs_frommatfile" {
	    set type_UValues $URVariables_UValues
	    set vartype_UValues URVariables_UValues
	}
	"urs_tomatfile" {
	    set type_UValues $URVariables_UValues
	    set vartype_UValues URVariables_UValues
	}
    }

    if { $way == "spc_frommatfile"  || $way == "urs_frommatfile" } {
	set ltemp ""
	foreach litem $type_UValues {
	    set temp ""
	    foreach item $litem {
		set tt ""
		regsub -all "2" $item "\u00b2" tt
		regsub -all "3" $tt "\u00b3" tt
		regsub -all {\[} $tt "" tt
		regsub -all {\]} $tt "" tt
		lappend temp $tt
	    }
	    lappend ltemp [string trim $temp]
	}
	set $vartype_UValues $ltemp
    }

    if { $way == "spc_tomatfile"  || $way == "urs_tomatfile" } {
	set ltemp ""
	foreach litem $type_UValues {
	    set temp ""
	    foreach item $litem {
		set tt ""
		regsub -all "\u00b2" $item "2" tt
		regsub -all "\u00b3" $tt "3" tt
		lappend temp "\[$tt\]"
	    }
	    lappend ltemp [string trim $temp]
	}
	set $vartype_UValues $ltemp
    }

    return 1
}



################################################################################
#
#   GiD Values Manipulation - GET Values, TSAVE and DUMP Values
#   - GETVALUES reads GiD values and puts them to the variables
#   - CHECKCLOSE goes through all variables and checks for changes. If changes 
#     are found, asks the user SAVE YES or NO. If YES, runs DUMP procedure.
#     This procedure also checks the temporal space of Initial Conditions.
#   - TICSAVE saves values from Initial Conditions entries to temporal variables
#     from where in case of accept saving it saves the values tom GiD memory.
#   - DUMP saves values of entries to GiD memory and in case of initial condition
#
################################################################################

proc matfluid::getvalues { } {

    global ProgramName

    # Tdyn settings
    variable show_advect
    variable show_ursolver

    variable Fluid_Questions
    variable Material_Name
    variable Species_Names
    variable URVariables_Names
    

    # Read all materials values
    set ValList [TUI_getmatvalueslist $Material_Name]

    # Index of values started from 1 and not from 0 because the 
    # first values is a material type is not modified here. 
    set vi 1
    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	variable $itt [lindex $ValList $vi]
	incr vi
    }

    if { $show_advect == 1 } {
	# Read Species Names from Problem Data
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "Species_Names:(SpecieName)"] + 1]]]
	set Species_Names [lreplace $temp 0 1]
    }

    if { $show_ursolver == 1 } {
	# Read URSolver Variables Names from Problem Data
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "URVariables_Names:(VariableName)"] + 1]]]
	set URVariables_Names [lreplace $temp 0 1]
    }

    if { $show_advect == 1 || $show_ursolver == 1 } {
	# Format Species
	matfluid::formatspecies retrieving
    }

    # Replace special characters by Unicode format
    matfluid::formatunits frommatfile
    if { $show_advect == 1 } {
	matfluid::formatunits spc_frommatfile
    }
    if { $show_ursolver == 1 } {
	matfluid::formatunits urs_frommatfile
    }
}

proc matfluid::checkclose { win {dumpit 1} {askdump 1} {close 1} } {

    global tkPriv tcl_platform

    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver

    variable Material_Name
    variable Fluid_Questions
    variable Species_Valnames
    variable Species_Names
    variable Species_Values
    variable URVariables_Valnames
    variable URVariables_Names
    variable URVariables_Values
    variable snlist
    variable ursnlist
    foreach item $Fluid_Questions {
	variable [lindex $item 0]
    }

    # Last unsaved changes of Species Values - save them
    if { $show_advect == 1 } {
	matfluid::valsavespecie $snlist "actual" "advect"
	matfluid::formatunits spc_tomatfile
    }
    if { $show_ursolver == 1 } {
	matfluid::valsavespecie $ursnlist "actual" "ursolver"
	matfluid::formatunits urs_tomatfile
    }

    matfluid::formatunits tomatfile

    if { $show_advect == 1 || $show_ursolver == 1 } {
	matfluid::formatspecies "saving"
    }

    set pv [TUI_getmatvalueslist $Material_Name]
    set valueschange 0
    set wtsave 0
    set vi 1
    set usr_err ""
    set num_usr_err 0
    set uerrpg_ran 0
    set uerrpg_htr 0
    set uerrpg_adv 0
    set uerrpg_urs 0

    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	set $itt [string trim [eval set nothing $$itt]]
	eval set cv $$itt
	if { [string trim $cv] != [string trim [lindex $pv $vi]] } {
	    set valueschange 1
	    if { [lindex $item 2] == 1 } {
		set errcheck [TUI_valuecheck [eval set nothing $$itt] [lindex $item 1] 0 [lindex $item 4]]
	    } else {
		set errcheck "ok"
	    }
	    if { $errcheck != "ok" && [lindex $item 2] == 1} {
		incr num_usr_err
		switch "[lindex $item 3]" {
		    "ransol" {
		        if { $uerrpg_ran != 1 } {
		            append usr_err "\nRANSOL PAGE\n"
		            set uerrpg_ran 1
		        }
		    }
		    "heatrans" {
		        if { $uerrpg_htr != 1 } {
		            append usr_err "\nHEATRANS PAGE\n"
		            set uerrpg_htr 1
		        }
		    }
		}
		append usr_err "$num_usr_err. " [DWUnder2Space $itt] ": $errcheck\n"
	    }
	}
	incr vi
    }

    if { $show_advect == 1 } {
	# Check also Species Names
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "Species_Names:(SpecieName)"] + 1]]]
	set temp [lreplace $temp 0 1]
	if { $temp != $Species_Names } {
	    set valueschange 1
	    #################### !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! #####################
	    # To check Species Names repetition, change "-exact" option to "-all" 
	    # in the first following IF condition.
	    #################### !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! #####################
	      set warnedspcnames ""
	      foreach item $Species_Names {
		  if { [llength [lsearch -exact $Species_Names $item]] > 1 && [lsearch -exact $warnedspcnames $item] == -1} {
		      lappend warnedspcnames $item
		      incr num_usr_err
		      if { $uerrpg_adv != 1 } {
		          append usr_err "\nADVECT PAGE\n"
		          set uerrpg_adv 1
		      }
		      append usr_err "$num_usr_err. Specie Name: \"$item\" is included more then once in the Species List. Species have to have unique names."
		  }
		set errcheck [TUI_valuecheck $item string 0]
		if { $errcheck != "ok"} {
		    incr num_usr_err
		    if { $uerrpg_adv != 1 } {
		        append usr_err "\nADVECT PAGE\n"
		        set uerrpg_adv 1
		    }
		    append usr_err "$num_usr_err. Specie Name: \"$item\" is not correct. Species names can only include alphabetical characters, numbers and underscoore.\n"
		}
	    }
	}
	if { [TUI_getmatfieldvalue $Material_Name "Concentration_Properties"] != $Concentration_Properties } {
	    set spc_pos 0
	    foreach item $Species_Values {
		set tspn [lindex $Species_Names $spc_pos]
		incr spc_pos
		set valnamepos 0
		foreach itt $item {
		    set valname [lindex $Species_Valnames $valnamepos]
		    incr valnamepos
		    if { [lindex $valname 3] == 1 } {
		        set errcheck [TUI_valuecheck $itt [lindex $valname 2] 0 [lindex $valname 4]]
		    } else {
		        set errcheck "ok"
		    }
		    if { $errcheck != "ok"} {
		        incr num_usr_err
		        if { $uerrpg_adv != 1 } {
		            append usr_err "\nADVECT PAGE\n"
		            set uerrpg_adv 1
		        }
		        append usr_err "$num_usr_err. " "$tspn; " [lindex $valname 0] ": $errcheck\n"
		    }
		}
	    }
	}
    }

    if { $show_ursolver == 1 } {
	# Check also URSolver Variables Names
	set temp [join [lindex [.central.s info gendata] \
		[expr [lsearch [.central.s info gendata] "URVariables_Names:(VariableName)"] + 1]]]
	set temp [lreplace $temp 0 1]
	if { $temp != $URVariables_Names } {
	    set valueschange 1
	    #################### !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! #####################
	    # To check URSolver Variables Names repetition, change "-exact" option to "-all" 
	    # in the first following IF condition.
	    #################### !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! #####################
	      set warnedurvnames ""
	      foreach item $URVariables_Names {
		  if { [llength [lsearch -exact $URVariables_Names $item]] > 1 && [lsearch -exact $warnedurvnames $item] == -1} {
		      lappend warnedurvnames $item
		      incr num_usr_err
		      if { $uerrpg_ran != 1 } {
		          append usr_err "\nURSOLVER PAGE\n"
		          set uerrpg_urs 1
		      }
		      append usr_err "$num_usr_err. URSolver Variable Name: \"$item\" is included more then once in the Species List. Species have to have unique names."
		  }
		set errcheck [TUI_valuecheck $item string 0]
		if { $errcheck != "ok"} {
		    incr num_usr_err
		    if { $uerrpg_urs != 1 } {
		        append usr_err "\nURSOLVER PAGE\n"
		        set uerrpg_urs 1
		    }
		    append usr_err "$num_usr_err. URSolver Variable Name: \"$item\" is not correct. URSolver Variables names can only include alphabetical characters, numbers and underscoore.\n"
		}
	    }
	}
	if { [TUI_getmatfieldvalue $Material_Name "UR_Variables"] != $UR_Variables } {
	    set spc_pos 0
	    foreach item $URVariables_Values {
		set tspn [lindex $URVariables_Names $spc_pos]
		incr spc_pos
		set valnamepos 0
		foreach itt $item {
		    set valname [lindex $URVariables_Valnames $valnamepos]
		    incr valnamepos
		    if { [lindex $valname 3] == 1 } {
		        set errcheck [TUI_valuecheck $itt [lindex $valname 2] 0  [lindex $valname 4]]
		    } else {
		        set errcheck "ok"
		    }
		    if { $errcheck != "ok"} {
		        incr num_usr_err
		        if { $uerrpg_urs != 1 } {
		            append usr_err "\nURSOLVER PAGE\n"
		            set uerrpg_urs 1
		        }
		        append usr_err "$num_usr_err. " "$tspn; " [lindex $valname 0] ": $errcheck\n"
		    }
		}
	    }
	}
    }

    # Units are formated back to unicode so that they are not seen 
    # in MAT File format in fields whild asking if changes shouldbe saved. 
    matfluid::formatunits frommatfile
    if { $show_advect == 1 } {
	matfluid::formatunits spc_frommatfile
    }
    if { $show_ursolver == 1 } {
	matfluid::formatunits urs_frommatfile
    }

    if { $valueschange == 1 } {
	if { $askdump == 0 } {
	    set answer 0
	} else {
	    set answer [tk_dialogRAM $win "Material: \"$Material_Name\"" \
		    "Material data has changed.\nDo you want to save changes?" question 0 Yes No]
	}

	if { $answer == 0 && $dumpit == 1} {
	    set wtsave 1
	    if { $num_usr_err > 0 } {
		if { $num_usr_err > 1 } {
		    set err_display "The following $num_usr_err error"
		    append err_display "(s) were found in the data input:\n"
		} else {
		    set err_display "The following error was found in the data input:\n"
		}
		append err_display $usr_err
		WarnWin $err_display
		set close 0
	    } else {
		matfluid::dump
	    }
	}
    }

    if { $close == 1 } {
	destroy $win
	return $valueschange
    } else {
	if { $num_usr_err == 0 } {
	    return $valueschange
	} else {
	    if { $wtsave == 1 } {
		return "dataerror"
	    } else {
		return "nosave.dataerror"
	    }
	}
    }

}

# Updates material data to file
proc matfluid::dump { } {

    variable show_advect
    variable show_ursolver
    variable Material_Name

    variable Species_Names
    variable URVariables_Names

    variable Species_Rebuild
    variable URVariables_Rebuild

    variable Fluid_Questions
    foreach item $Fluid_Questions {
	variable [lindex $item 0]
    }
    variable Fluid_UnitsRead
    foreach item $Fluid_UnitsRead {
	variable [lindex $item 0]
    }

    # Format units to MAT File format
    foreach item $Fluid_UnitsRead {
	set itt [lindex $item 0]
	set $itt [TUI_formatunits Unicode2Nums [eval set temp $$itt] 1]
    }

    TUI_setnewmatvalues $Material_Name [string trim [matfluid::getcurrentvalset]]
 
    if { $show_advect == 1 } {
	TUI_savespeciesnames "Species_Names:(SpecieName)" $Species_Names
	TUI_synchrospecies "advect" $Species_Rebuild $Material_Name
    }
    if { $show_ursolver == 1 } {
	TUI_savespeciesnames "URVariables_Names:(VariableName)" $URVariables_Names
	TUI_synchrospecies "ursolver" $URVariables_Rebuild $Material_Name
    }

    # Format units from MAT File format - for the case the user clicks only the MODIFY button
    foreach item $Fluid_UnitsRead {
	set itt [lindex $item 0]
	set $itt [TUI_formatunits Nums2Unicode [eval set temp $$itt] 0]
    }
}

proc matfluid::getcurrentvalset { } {
    
    variable Fluid_Questions
    foreach item $Fluid_Questions {
	variable [lindex $item 0]
    }

    set Vallist "Fluid "
    foreach item $Fluid_Questions {
	set itt [lindex $item 0]
	eval append Vallist $$itt
	append Vallist " "
    }

    return $Vallist
}