namespace eval materials {

    global ProgramName
}

namespace eval fluidmaterials {

    variable Materials_List
    variable Material_Name
    variable Last_Material_Name
}

namespace eval solidmaterials {
    
    variable Materials_List
    variable Material_Name
    variable Last_Material_Name
}

namespace eval fluidboundaries {
    
    variable Boundaries_List
    variable Boundary_Name
    variable Last_Boundary_Name
}

namespace eval solidboundaries {
    
    variable Boundaries_List
    variable Boundary_Name
    variable Last_Boundary_Name
}

proc materials::init { book } {

    if { $book == "MATERIALS_(FLUID)" } {
	set w ".gid.fluidmat"
	set wtitle "Fluid Material Window"
	set geomname PreTdynFluidMatWindowGeom
    } elseif { $book == "MATERIALS_(SOLID)" } {
	set w ".gid.solidmat"
	set wtitle "Solid Material Window"
	set geomname PreTdynSolidMatWindowGeom
    } elseif { $book == "BOUNDARIES_(FLUID)" } {
	set w ".gid.fluidbond"
	set wtitle "Fluid Boundaries Window"
	set geomname PreTdynFluidBondWindowGeom
    } elseif { $book == "BOUNDARIES_(SOLID)" } {
	set w ".gid.solidbond"
	set wtitle "Solid Boundaries Window"
	set geomname PreTdynSolidBondWindowGeom
    }

    # Closing other windows opened
    set owc [TUI_closeotherwindows $w]

    if { $owc == 1 } {
	InitWindow $w $wtitle $geomname ""

	grid columnconf $w 0 -weight 1
	grid rowconf $w 1 -weight 1

	# Window Title and Close Window Command
	if { $book == "MATERIALS_(FLUID)" } {
	    set closecmd "matfluid::checkclose $w 1 1 1"
	    wm protocol $w WM_DELETE_WINDOW "$closecmd"
	} elseif { $book == "MATERIALS_(SOLID)" } {
	    set closecmd "matsolid::checkclose $w 1 1 1"
	    wm protocol $w WM_DELETE_WINDOW "$closecmd"
	} elseif { $book == "BOUNDARIES_(FLUID)" } {
	    set closecmd "bondsolid::checkclose $w 1 1 1"
	    wm protocol $w WM_DELETE_WINDOW "$closecmd"
	} elseif { $book == "BOUNDARIES_(SOLID)" } {
	    set closecmd "bondfluid::checkclose $w 1 1 1"
	    wm protocol $w WM_DELETE_WINDOW "$closecmd"
	}

	if { $book == "MATERIALS_(FLUID)" } {
	    fluidmaterials::init $w $book
	} elseif { $book == "MATERIALS_(SOLID)" } {
	    solidmaterials::init $w $book
	} elseif { $book == "BOUNDARIES_(FLUID)" } {
	    fluidboundaries::init $w $book
	} elseif { $book == "BOUNDARIES_(SOLID)" } {
	    solidboundaries::init $w $book
	}
	

	# BINDS
	bind $w <Escape> "tkButtonInvoke $w.bf.btn_close"
	bind $w <Alt-KeyPress-c> "tkButtonInvoke $w.bf.btn_close"

	focus $w
    }
}

proc fluidmaterials::init { w book } {

    variable Materials_List
    variable Material_Name
    variable Last_Material_Name

    set Materials_List [TIU_getbooknames $book]

    # Finding out the longest material name
    set mcl 15
    foreach aa $Materials_List {
	if { [string length $aa] > $mcl } {
	    set mcl [string length $aa]
	} 
    }
    incr mcl

    set Material_Name [lindex $Materials_List 0]
    set Last_Material_Name $Material_Name

    # Top navigation frame
    set f [frame $w.f -bg lightblue]
    # Materials Bombo Box
    set l_icon [label $f.l_icon -image $TUI_images::i_fluiddrop -bg lightblue]
    set l_selmat [label $f.l_selmat -text "Select Material:" -bg lightblue]
    set cbx_matlist [ComboBox $w.f.matlist -values $Materials_List \
	    -editable false -width $mcl -takefocus 1 \
	    -textvariable fluidmaterials::Material_Name \
	    -modifycmd "materials::changematerial $w $w.f.matlist $book \$fluidmaterials::Material_Name 1"]
    $cbx_matlist setvalue first

    # Update, New Erase buttons
    set btn_mod [button $w.f.mod -image $buttons::ibtn_uchang \
	    -command "materials::modifymaterial $w $book \$fluidmaterials::Material_Name" -und 0 -bg lightblue]
    set btn_new [button $w.f.new -image $buttons::ibtn_new \
	    -command "materials::newmaterial $w $book $cbx_matlist \"$Materials_List\" \$fluidmaterials::Material_Name" \
	    -und 0 -bg lightblue]
    set btn_erase [button $w.f.erase -image $buttons::ibtn_erase \
	    -command "materials::deletematerial $w $book $cbx_matlist \"$Materials_List\" \$fluidmaterials::Material_Name" \
	    -und 0 -bg lightblue]

    GidHelp $w.f.mod "Update changes"
    GidHelp $w.f.new "Crete New Material"
    GidHelp $w.f.erase "Delete Material"

    # GRIDS
    grid $f -row 0 -column 0 -sticky wnes
    grid columnconf $f 10 -weight 1
    grid $l_icon -row 0 -column 0 -sticky w -padx 5
    grid $l_selmat -row 0 -column 1 -sticky w
    grid $cbx_matlist -row 0 -column 2 -sticky w -padx 5
    grid $btn_mod -row 0 -column 3 -sticky w -pady 1
    grid $btn_new -row 0 -column 4 -sticky w -pady 1
    grid $btn_erase -row 0 -column 5 -sticky w -pady 1

    # Make lower buttons
    materials::makelowerbuttons $w $book $Material_Name

    # Data entries into the window
    matfluid::init $w $Material_Name
}

proc solidmaterials::init { w book } {

    variable Materials_List
    variable Material_Name
    variable Last_Material_Name

    set Materials_List [TIU_getbooknames "MATERIALS_(SOLID)"]

    # Finding out the longest material name
    set mcl 15
    foreach aa $Materials_List {
	if { [string length $aa] > $mcl } {
	    set mcl [string length $aa]
	} 
    }
    incr mcl
    set Material_Name [lindex $Materials_List 0]
    set Last_Material_Name $Material_Name

    # Top navigation frame
    set f [frame $w.f -bg lightblue]
    # Materials Bombo Box
    set l_icon [label $f.l_icon -image $TUI_images::i_solidcolumn -bg lightblue]
    set l_selmat [label $f.l_selmat -text "Select Material:" -bg lightblue]
    set cbx_matlist [ComboBox $w.f.matlist -values $Materials_List \
	    -editable false -width $mcl -takefocus 1 \
	    -textvariable solidmaterials::Material_Name \
	    -modifycmd "materials::changematerial $w $w.f.matlist $book \$solidmaterials::Material_Name 1"]
    $cbx_matlist setvalue first

    # Update, New Erase buttons
    set btn_mod [button $w.f.mod -image $buttons::ibtn_uchang \
	    -command "materials::modifymaterial $w $book \$solidmaterials::Material_Name" -und 0 -bg lightblue]
    set btn_new [button $w.f.new -image $buttons::ibtn_new \
	    -command "materials::newmaterial $w $book $cbx_matlist \"$Materials_List\" \$solidmaterials::Material_Name" \
	    -und 0 -bg lightblue]
    set btn_erase [button $w.f.erase -image $buttons::ibtn_erase \
	    -command "materials::deletematerial $w $book $cbx_matlist \"$Materials_List\" \$solidmaterials::Material_Name" \
	    -und 0 -bg lightblue]

    GidHelp $w.f.mod "Update changes"
    GidHelp $w.f.new "Crete New Material"
    GidHelp $w.f.erase "Delete Material"

    # GRIDS
    grid $f -row 0 -column 0 -sticky wnes
    grid columnconf $f 10 -weight 1
    grid $l_icon -row 0 -column 0 -sticky w -padx 5
    grid $l_selmat -row 0 -column 1 -sticky w
    grid $cbx_matlist -row 0 -column 2 -sticky w -padx 5
    grid $btn_mod -row 0 -column 3 -sticky w -pady 1
    grid $btn_new -row 0 -column 4 -sticky w -pady 1
    grid $btn_erase -row 0 -column 5 -sticky w -pady 1

    # Make lower buttons
    materials::makelowerbuttons $w $book $Material_Name
    
    # Data entries into the window
    matsolid::init $w $Material_Name
}

proc fluidboundaries::init { w book } {

    variable Materials_List
    variable Material_Name
    variable Last_Material_Name

    set Materials_List [TIU_getbooknames $book]

    # Finding out the longest material name
    set mcl 15
    foreach aa $Materials_List {
	if { [string length $aa] > $mcl } {
	    set mcl [string length $aa]
	} 
    }
    incr mcl

    set Material_Name [lindex $Materials_List 0]
    set Last_Material_Name $Material_Name

    # Top navigation frame
    set f [frame $w.f -bg lightblue]
    # Materials Bombo Box
    set l_icon [label $f.l_icon -image $TUI_images::i_fluiddrop -bg lightblue]
    set l_selmat [label $f.l_selmat -text "Select Material:" -bg lightblue]
    set cbx_matlist [ComboBox $w.f.matlist -values $Materials_List \
	    -editable false -width $mcl -takefocus 1 \
	    -textvariable fluidmaterials::Material_Name \
	    -modifycmd "materials::changematerial $w $w.f.matlist $book \$fluidmaterials::Material_Name 1"]
    $cbx_matlist setvalue first

    # Update, New Erase buttons
    set btn_mod [button $w.f.mod -image $buttons::ibtn_uchang \
	    -command "materials::modifymaterial $w $book \$fluidmaterials::Material_Name" -und 0 -bg lightblue]
    set btn_new [button $w.f.new -image $buttons::ibtn_new \
	    -command "materials::newmaterial $w $book $cbx_matlist \"$Materials_List\" \$fluidmaterials::Material_Name" \
	    -und 0 -bg lightblue]
    set btn_erase [button $w.f.erase -image $buttons::ibtn_erase \
	    -command "materials::deletematerial $w $book $cbx_matlist \"$Materials_List\" \$fluidmaterials::Material_Name" \
	    -und 0 -bg lightblue]

    GidHelp $w.f.mod "Update changes"
    GidHelp $w.f.new "Crete New Material"
    GidHelp $w.f.erase "Delete Material"

    # GRIDS
    grid $f -row 0 -column 0 -sticky wnes
    grid columnconf $f 10 -weight 1
    grid $l_icon -row 0 -column 0 -sticky w -padx 5
    grid $l_selmat -row 0 -column 1 -sticky w
    grid $cbx_matlist -row 0 -column 2 -sticky w -padx 5
    grid $btn_mod -row 0 -column 3 -sticky w -pady 1
    grid $btn_new -row 0 -column 4 -sticky w -pady 1
    grid $btn_erase -row 0 -column 5 -sticky w -pady 1

    # Make lower buttons
    materials::makelowerbuttons $w $book $Material_Name

    # Data entries into the window
    bondfluid::init $w $Material_Name
}

proc materials::changematerial { w mlcbx book Material_Name {checkch 1} } {

    # Make the pages content again.
    switch $book {
	"MATERIALS_(FLUID)" {
	    if { $checkch == 1 } {
		set temp [matfluid::checkclose $w 1 1 0]
	    } else {
		set temp 1
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		matfluid::init $w $Material_Name
	    } else {
		$mlcbx setvalue @[lsearch $fluidmaterials::Materials_List $fluidmaterials::Last_Material_Name]
	    }
	}
	"MATERIALS_(SOLID)" {
	    if { $checkch == 1 } {
		set temp [matsolid::checkclose $w 1 1 0]
	    } else {
		set temp 1
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		matsolid::init $w $Material_Name
	    } else {
		$mlcbx setvalue @[lsearch $solidmaterials::Materials_List $solidmaterials::Last_Material_Name]
	    }
	}
	"BOUNDARIES_(FLUID)" {
	    if { $checkch == 1 } {
		set temp [bondfluid::checkclose $w 1 1 0]
	    } else {
		set temp 1
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		bondfluid::init $w $Material_Name
	    } else {
		$mlcbx setvalue @[lsearch $fluidboundaries::Materials_List $fluidboundaries::Last_Material_Name]
	    }
	}
	"BOUNDARIES_(SOLID)" {
	    if { $checkch == 1 } {
		set temp [bondsolid::checkclose $w 1 1 0]
	    } else {
		set temp 1
	    }
	    if { $temp != "dataerror" } {
		destroy $w.pages
		bondsolid::init $w $Material_Name
	    } else {
		$mlcbx setvalue @[lsearch $solidboundaries::Materials_List $solidboundaries::Last_Material_Name]
	    }
	}
    }

    # Make lower buttons again
    destroy $w.bf
    materials::makelowerbuttons $w $book $Material_Name

    switch $book {
	"MATERIALS_(FLUID)" {
	    set fluidmaterials::Last_Material_Name $Material_Name
	}
	"MATERIALS_(SOLID)" {
	    set solidmaterials::Last_Material_Name $Material_Name
	}
	"BOUNDARIES_(FLUID)" {
	    set fluidboundaries::Last_Material_Name $Material_Name
	}
	"BOUNDARIES_(SOLID)" {
	    set solidboundaries::Last_Material_Name $Material_Name
	}
    }

    return 1
}

proc materials::modifymaterial { w book Material_Name } {


    switch $book {
	"MATERIALS_(FLUID)" {
	    set temp [matfluid::checkclose $w 1 0 0]
	    set type Material
	}
	"MATERIALS_(SOLID)" {
	    set temp [matsolid::checkclose $w 1 0 0]
	    set type Material
	}
	"BOUNDARIES_(FLUID)" {
	    set temp [bondfluid::checkclose $w 1 0 0]
	    set type Boundary
	}
	"BOUNDARIES_(SOLID)" {
	    set temp [bondsolid::checkclose $w 1 0 0]
	    set type Boundary
	}
    }

    if { $temp == 1 } {
	FloatMessage $w.f.flm "$type \"$Material_Name\" was modified"
    } elseif { $temp == 0 } {
	FloatMessage $w.f.flm "No changes need to be saved."
    }

    return 1
}

proc materials::newmaterial { w book mlcbx Materials_List Material_Name } {

    # Save actual material?
    switch $book {
	"MATERIALS_(FLUID)" {
	    set savechanges [matfluid::checkclose $w 1 1 0]
	    set type Material
	}
	"MATERIALS_(SOLID)" {
	    set savechanges [matsolid::checkclose $w 1 1 0]
	    set type Material
	}
	"BOUNDARIES_(FLUID)" {
	    set savechanges [bondfluid::checkclose $w 1 1 0]
	    set type Boundary
	}
	"BOUNDARIES_(SOLID)" {
	    set savechanges [bondsolid::checkclose $w 1 1 0]
	    set type Boundary
	}
    }

    if { $savechanges != "dataerror" } {
	set aa [tk_dialogEntryRAM $w "Creating New Material" \
		"Enter new material name.           " "" any ""]

	if { $aa != "--CANCEL--" && $aa != "" } {

	    # removing quotes and converting spaces to underscores
	    regsub -all "\"" $aa "" aa
	    set aa [DWSpace2Under [string trim $aa]]
	    
	    if { [lsearch $Materials_List $aa] == -1 } {

		foreach val [TUI_getmatvalueslist $Material_Name] {
		    append values $val " "
		}
		
		GidUtils::DisableWarnLine
		.central.s process escape escape escape escape Data Materials \
		        NewMaterial $Material_Name $aa [string trim $values]
		.central.s process escape escape escape escape
		GidUtils::EnableWarnLine

		switch $book {
		    "MATERIALS_(FLUID)" {
			set fluidmaterials::Material_Name $aa
			set fluidmaterials::Materials_List [TIU_getbooknames $book]
			set t_matlist $fluidmaterials::Materials_List
		    }
		    "MATERIALS_(SOLID)" {
			set solidmaterials::Material_Name $aa
			set solidmaterials::Materials_List [TIU_getbooknames $book]
			set t_matlist $solidmaterials::Materials_List
		    }
		    "BOUNDARIES_(FLUID)" {
			set fluidboundaries::Material_Name $aa
			set fluidboundaries::Materials_List [TIU_getbooknames $book]
			set t_matlist $fluidboundaries::Materials_List
		    }
		    "BOUNDARIES_(SOLID)" {
			set solidboundaries::Material_Name $aa
			set solidboundaries::Materials_List [TIU_getbooknames $book]
			set t_matlist $solidboundaries::Materials_List
		    }
		}

		set dee [lsearch [TIU_getbooknames $book] $aa]
		materials::changematerial $w $mlcbx $book $aa 0

		$mlcbx configure -values $t_matlist

	    } else {
		WarnWin "A $type with this name already exists.\nPlease select another name."
	    }
	}
    }
}

proc materials::deletematerial { w book mlcbx Materials_List Material_Name } {

    switch $book {
	"MATERIALS_(FLUID)" {
	    set type Material
	    set types Materials
	}
	"MATERIALS_(SOLID)" {
	    set type Material
	    set types Materials
	}
	"BOUNDARIES_(FLUID)" {
	    set type Boundary
	    set types Boundaries
	}
	"BOUNDARIES_(SOLID)" {
	    set type Boundary
	    set types Boundaries
	}
    }
    set aa [tk_dialogRAM $w "Deleting $Material_Name" \
    "Sure you want to delete this $type and all its data?" question 1 Yes No]

    if { $aa == 0 } {
	if { [llength $Materials_List] < 2 } {
	    WarnWin "You cannot delete all $types defined.\nCreate another $type you wish to use and then delete the last one."
	} else {
	    GidUtils::DisableWarnLine
	    .central.s process escape escape escape escape Data Materials \
		    DeleteMaterial $Material_Name
	    .central.s process escape escape escape escape
	    GidUtils::EnableWarnLine

	    switch $book {
		"MATERIALS_(FLUID)" {
		    set fluidmaterials::Materials_List [TIU_getbooknames $book]
		    set t_matlist $fluidmaterials::Materials_List
		    set fluidmaterials::Material_Name [lindex $fluidmaterials::Materials_List 0]
		    materials::changematerial $w $mlcbx $book $fluidmaterials::Material_Name 0
		}
		"MATERIALS_(SOLID)" {
		    set solidmaterials::Materials_List [TIU_getbooknames $book]
		    set t_matlist $solidmaterials::Materials_List
		    set solidmaterials::Material_Name [lindex $solidmaterials::Materials_List 0]
		    materials::changematerial $w $mlcbx $book $solidmaterials::Material_Name 0
		}
		"BOUNDARIES_(FLUID)" {
		    set fluidboundaries::Materials_List [TIU_getbooknames $book]
		    set t_matlist $fluidboundaries::Materials_List
		    set fluidboundaries::Material_Name [lindex $fluidboundaries::Materials_List 0]
		    materials::changematerial $w $mlcbx $book $fluidboundaries::Material_Name 0
		}
		"BOUNDARIES_(SOLID)" {
		    set solidboundaries::Materials_List [TIU_getbooknames $book]
		    set t_matlist $solidboundaries::Materials_List
		    set solidboundaries::Material_Name [lindex $solidboundaries::Materials_List 0]
		    materials::changematerial $w $mlcbx $book $solidboundaries::Material_Name 0
		}
	    }

	    $mlcbx configure -values $t_matlist

	}
    }
}

proc materials::makelowerbuttons { w book Material_Name } {

    global ProgramName

    # Buttons frame
    set bf [frame $w.bf -relief raised -takefocus 1]

    # Close Window Command
    switch $book {
	"MATERIALS_(FLUID)" {
	    set closecmd "matfluid::checkclose $w 1 1 1"
	    set 2delements Surfaces
	    set 3delements Volumes
	    set type Material
	    set types Materials
	}
	"MATERIALS_(SOLID)" {
	    set closecmd "matsolid::checkclose $w 1 1 1"
	    set 2delements Surfaces
	    set 3delements Volumes
	    set type Material
	    set types Materials
	}
	"BOUNDARIES_(FLUID)" {
	    set closecmd "bondfluid::checkclose $w 1 1 1"
	    set 2delements Lines
	    set 3delements Surfaces
	    set type Boundary
	    set types Boundaries
	}
	"BOUNDARIES_(SOLID)" {
	    set closecmd "bondsolid::checkclose $w 1 1 1"
	    set 2delements Lines
	    set 3delements Surfaces
	    set type Boundary
	    set types Boundaries
	}
    }
    # Assign button
    switch $ProgramName {
	"Tdyn2D" {
	    set assigncmd "materials::assignmaterial $w $2delements $Material_Name"
	}
	"Tdyn3D" {
	    set assigncmd "materials::assignmaterial $w $3delements $Material_Name"
	}
    }
    set btn_assign [button $bf.btn_assign -text Assign -relief raised \
	    -command $assigncmd -bd 1 -highlightthickness 1 -takefocus 1]

    # Draw button
    set MenuDraw $bf.btn_draw.menu
    set btn_draw [menubutton $bf.btn_draw -text Draw -relief raised \
	    -menu $MenuDraw -bd 1.08 -highlightthickness 1 -takefocus 1]
    set menu_draw [menu $MenuDraw -bd 1 -activeborderwidth 1]
    $MenuDraw add command -label "This $type \"[DWUnder2Space $Material_Name]\"" \
	    -command "materials::drawmaterial $w thismaterial $Material_Name"
    $MenuDraw add command -label "All Materials" \
	    -command "materials::drawmaterial $w allmaterials $Material_Name"

    # Unassign button
    set MenuUnassign $bf.btn_unassign.menu
    set btn_unassign [menubutton $bf.btn_unassign -text Unassign -relief raised \
	-menu $MenuUnassign -bd 1 -highlightthickness 1 -takefocus 1]
    set menu_unassign [menu $MenuUnassign -bd 1 -activeborderwidth 1]
    switch $ProgramName {
	"Tdyn2D" {
	    $MenuUnassign add command -label $2delements -command "materials::unassignmaterial $w $book $2delements $Material_Name"
	}
	"Tdyn3D" {
	    $MenuUnassign add command -label $3delements -command "materials::unassignmaterial $w $book $3delements $Material_Name"
	}
    }
    $MenuUnassign add command -label "All of $type \"[DWUnder2Space $Material_Name]\"" \
	    -command "materials::unassignmaterial $w $book allthismat $Material_Name"
    if { $book != "Default" } {
	$MenuUnassign add command -label "All Book \"[DWUnder2Space $book]\"" \
		-command "materials::unassignmaterial $w $book allthisbook $Material_Name"
    }
    $MenuUnassign add command -label "All $types" \
		-command "materials::unassignmaterial $w $book allmaterials $Material_Name"

    # Import/Export button
    set btn_impexp [button $bf.btn_impexp -text "Import/Export" -relief raised \
	    -bd 1 -highlightthickness 1 -takefocus 1 \
	    -command [list DWMatImportExport $w $book]]
    # Close button
    set btn_close [button $bf.btn_close -text Close -underline 0 -relief raised \
	    -bd 1 -highlightthickness 1 -bg lightblue -takefocus 1 \
	    -command "$closecmd"]
    
    # GRIDS
    grid $bf -row 2 -column 0 -sticky wnes
    grid columnconf $bf 4 -weight 1
    grid $btn_assign -row 0 -column 0 -sticky w -pady 1
    grid $btn_draw -row 0 -column 1 -sticky w -pady 1
    grid $btn_unassign -row 0 -column 2 -sticky w -pady 1
    grid $btn_impexp -row 0 -column 3 -sticky w -pady 1
    grid $btn_close -row 0 -column 4 -sticky e -pady 1

}

proc materials::assignmaterial { w {type Elements} Material_Name } {

    set _MatName [DWSpace2Under $Material_Name]

    if { $type != "Elements" } {
	.central.s process escape escape escape escape data mat assign \
		$_MatName $type
    } else {
	.central.s process escape escape escape escape data mat assign \
		$_MatName
    }

    # Disable Matlist Conbobox
    $w.f.matlist configure -state disabled

    set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
    FinishButton $w $w.bf.btn_assign "Press 'Finish' when selection\nis finished" $endaction disableall

}

proc materials::unassignmaterial { w book type Material_Name } {

    global ProgramName

    switch $book {
	"MATERIALS_(FLUID)" {
	    set tty Material
	}
	"MATERIALS_(SOLID)" {
	    set tty Material
	}
	"BOUNDARIES_(FLUID)" {
	    set tty Boundary
	}
	"BOUNDARIES_(SOLID)" {
	    set tty Boundary
	}
    }

    if { $type == "Surfaces" || $type == "Volumes" || $type == "Lines" } {
	.central.s process escape escape escape escape Data Materials UnAssign Entities $type
	
	# Disable Matlist Conbobox
	$w.f.matlist configure -state disabled

	set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
	FinishButton $w $w.bf.btn_assign "Press 'Finish' when selection\nis finished" $endaction disableall

    } elseif { $type == "allthismat" } {
	if { $Material_Name == ""} {
	    WarnWin "Please select a $tty first." $w
	    return
	}
	set aa [tk_dialogRAMFull $w.tempwin {Unassign materials} \
		"Are you sure you want to unassign all $tty [DWUnder2Space $Material_Name]?" \
		"" "" gidquestionhead 0 OK Cancel]
	if { $aa == "0" } {
	    .central.s process escape escape escape escape Data Materials UnAssign SomeMat $Material_Name
	}
    } elseif { $type == "allthisbook" } {
	if { $book == ""} {
	    WarnWin "Please select a Book first." $w
	    return
	}
	set aa [tk_dialogRAMFull $w.tempwin {Unassign materials} \
		"Are you sure you want to unassign all Book [DWUnder2Space $book]?" \
		"" "" gidquestionhead 0 OK Cancel]
	if { $aa == "0" } {
	    .central.s process escape escape escape escape Data Materials UnAssign SomeBook $book
	}
    } elseif { $type == "allmaterials" } {
	# Confirmation qeustion is not here because GiD posts it itself.
	.central.s process escape escape escape escape Data Materials UnAssign AllMat
    }
}

proc materials::drawmaterial { w type Material_Name } {

    switch $type {
	thismaterial {
	    .central.s process escape escape escape escape data material draw $Material_Name
	}
	allmaterials {
	    .central.s process escape escape escape escape data material draw -drawall-
	}
    }

    # Disable Matlist Conbobox
    $w.f.matlist configure -state disabled

    set endaction ".central.s process escape escape escape; $w.f.matlist configure -state normal"
    FinishButton $w $w.bf.btn_assign "Press 'Finish' when finished\ndrawing" $endaction disableall
}