#tkwidget.tcl  -*- TCL -*- Created: JGE Jun-2002, Modified: JGE Jun-2002

###################################################
# UTILIDAD TKWIDGET PARA CONTROL DE CONDICIONES
###################################################
namespace eval ConditionsWidget {
    variable HelpFunc "Function used to calculate boundary condition\
	    value.\nEdit this function in Data>Initial Data."
    variable HelpCond "Function used to update boundary\
	    conditions where conditional condition is applied.\
	    If the value given by the function is > 0 , boundary\
	    condition is applied. If < 0 no boundary condition\
	    is applied. If = 0, nothing is done.\nEdit this\
	    function in Data>Initial Data (FuncCond)."
    variable HelpField "Function used to calculate boundary condition\
	    value.\nEdit this function in Data>Initial Data.\
	    If only Fix_Initial is selected, function will\
	    be evaluated for t=0."
    variable HelpUnits1 "Units may be defined for all the conditions\
	    in Data>Units Data>General Units"
    variable HelpUnits2 "Units may be defined for all the conditions\
	    in Data>Solver Data>ADVECT"
    variable HelpUnits3 "Units may be defined for all the conditions\
	    in Data>Solver Data>URSOLVER"
    variable Temperature_Help "Temperature will be fixed to the given\
	    value."
    variable Pressure_Help "Pressure will be fixed to the given value."
    variable Velocity_Help "Velocity components will be fixed to the\
	    given value."
    variable SpeciesList ""
    variable SpeciesMenu "" 
    variable SpeciesSelc ""
    variable SpeciesFields "" 
    variable SpeciesField "" 
    variable SpeciesConds ""
    variable SpeciesCond ""
    variable SpeciesValue ""
    variable SpeciesFIni ""
    variable SpeciesFFld ""
    variable SpeciesUnits ""
    variable SpeciesUnit ""
    variable PhisList ""
    variable PhisMenu "" 
    variable PhisSelc ""
    variable PhisFields "" 
    variable PhisField "" 
    variable PhisConds ""
    variable PhisCond ""
    variable PhisValue ""
    variable PhisFIni ""
    variable PhisFFld ""
    variable PhisUnits ""
    variable PhisUnit ""
    variable Temperature_Value
    variable Temperature_Units
    variable Pressure_Value
    variable Pressure_Units
    variable VelocityX_Value
    variable VelocityY_Value
    variable VelocityZ_Value
    variable VelocityX_Fix
    variable VelocityY_Fix
    variable VelocityZ_Fix
    variable Velocity_Units
}

proc ConditionsWidget::ShowFieldPhiFunction { cas op args } {
    # cas puede ser field (condiciones tipo Field),
    #  cond (condiciones tipo Conditional) o value
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable PhisList
    variable PhisMenu
    variable PhisSelc
    variable PhisFields
    variable PhisField
    variable PhisConds
    variable PhisCond
    variable PhisValue 
    variable PhisFIni
    variable PhisFFld
    variable PhisUnits
    variable PhisUnit
    variable HelpFunc
    variable HelpCond
    variable HelpUnits3
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]
	    set f [frame $PARENT.f]
	    set gendata [.central.s info gendata] 
	    set idx [lsearch -regexp $gendata URVariables_Names]
	    set data [lindex $gendata [expr $idx+1]]
	    set PhisList [lrange $data 2 end]
	    set PhisSelc [lindex $PhisList [expr \
		    [DWLocalGetValue $GDN $STRUCT Variable_index]-1]]
	    label $f.lbi -text "Phis Name:"
	    set PhisMenu [eval [ concat tk_optionMenu $f.uni \
		    ConditionsWidget::PhisSelc $PhisList]] 
	    foreach Phis $PhisList {
		$PhisMenu entryconfigure $Phis \
		        -command ConditionsWidget::UpdatePhisData
	    }
	    GidHelp $f.lbi "Phis selection" 
	    GidHelp $f.uni "Phis selection"
	    grid $f.lbi $f.uni -sticky nw
	    if { $cas=="field" } {
		set PhisFIni [DWLocalGetValue $GDN $STRUCT Fix_Initial]
		set PhisFFld [DWLocalGetValue $GDN $STRUCT Fix_Field]
		checkbutton $f.ch1 -text "Fix Initial" \
		        -variable ConditionsWidget::PhisFIni -relief flat \
		        -command "ConditionsWidget::Link $f.ch1 \$ConditionsWidget::PhisFFld" 
		checkbutton $f.ch2 -text "Fix Field" \
		        -variable ConditionsWidget::PhisFFld -relief flat \
		        -command "ConditionsWidget::Link $f.ch1 \$ConditionsWidget::PhisFFld" 
		ConditionsWidget::Link $f.ch1 $PhisFFld
		GidHelp $f.ch1 "Fix the variable to the initial value (t=0) \
		        given in the \"Value Field\" field."
		GidHelp $f.ch2 "Fix the variable to the (time dependant) \
		        field given in \"Value Field\"."
		grid $f.ch1 -sticky nw
		grid $f.ch2 -sticky nw
	    }
	    if { $cas=="field" || $cas=="cond"} {
		set idx [lsearch -regexp $gendata URSolver_Initdata]
		set data [lindex $gendata [expr $idx+1]]
		set PhisFields [lstep $data 2 2]
		set PhisField [lindex $PhisFields 0]
		set idx [lsearch -regexp $gendata URSolver_Solverdata]
		set data [lindex $gendata [expr $idx+1]]
		set PhisUnits [lstep $data 7 8]
		set PhisUnit [TUI_formatunits Nums2Unicode [lindex $PhisUnits 0]]
		label $f.lb1 -text "Value Field:"
		label $f.lb2 -relief sunken -borderwidth 1 -anchor w \
		        -wraplength 85 -width 14 -justify left \
		        -textvariable ConditionsWidget::PhisField
		label $f.lbu -textvariable ConditionsWidget::PhisUnit
		GidHelp $f.lb1 $HelpFunc
		GidHelp $f.lb2 $HelpFunc
		GidHelp $f.lbu $HelpUnits3
		grid $f.lb1 $f.lb2 $f.lbu -sticky nw
	  } else {
	      set PhisFields ""
	  }
	  if { $cas=="cond"} {
		set idx [lsearch -regexp $gendata URSolver_Initdata]
		set data [lindex $gendata [expr $idx+1]]
		set PhisConds [lstep $data 2 3]
		set PhisCond [lindex $PhisConds 0]
		label $f.lb3 -text "Conditional:"
		label $f.lb4 -relief sunken -borderwidth 1 -anchor w \
		        -wraplength 85 -width 14 -justify left \
		        -textvariable ConditionsWidget::PhisCond
		GidHelp $f.lb3 $HelpCond
		GidHelp $f.lb4 $HelpCond
		grid $f.lb3 $f.lb4 -sticky nw
	    }  else {
		set PhisConds ""
	    }
	    if { $cas=="value"} {
		set PhisValue [DWLocalGetValue $GDN $STRUCT {Value:}]
		set idx [lsearch -regexp $gendata URSolver_Solverdata]
		set data [lindex $gendata [expr $idx+1]]
		set PhisUnits [lstep $data 7 8]
		set PhisUnit [TUI_formatunits Nums2Unicode [lindex $PhisUnits 0]]
		label $f.lb5 -text "Value:"
		entry $f.lb6 -relief sunken -borderwidth 1 -width 14 \
		        -textvariable ConditionsWidget::PhisValue
		label $f.lbu -textvariable ConditionsWidget::PhisUnit
		GidHelp $f.lb5 "Given variable value"
		GidHelp $f.lb6 "Given variable value"
		GidHelp $f.lbu $HelpUnits3
		grid $f.lb5 $f.lb6 $f.lbu -sticky nw
	    }
	    grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		    -pady 3 -padx 2
	    grid rowconf $PARENT $ROW -weight 1
	    grid columnconf $PARENT 1 -weight 1

	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    set PhisIndex [lsearch $PhisList $PhisSelc]
	    DWLocalSetValue $GDN $STRUCT Variable_index [expr $PhisIndex+1]
	    if { $cas=="value"} {
		if { [TUI_valuecheck $PhisValue "float"]!="ok" } {
		    set PhisValue [tk_dialogEntryRAM .gid.tempwine [_ "Enter value window"] \
		            "Enter correct value" questhead real $PhisValue]
		}
		DWLocalSetValue $GDN $STRUCT Value $PhisValue
	    }
	    if { $cas=="field" } {
		DWLocalSetValue $GDN $STRUCT Fix_Field $PhisFFld
		DWLocalSetValue $GDN $STRUCT Fix_Initial $PhisFIni
	    }
	    return ""
	}
    }
}

proc ConditionsWidget::UpdatePhisData { } {
    variable PhisList
    variable PhisMenu
    variable PhisSelc
    variable PhisFields
    variable PhisField
    variable PhisConds
    variable PhisCond
    variable PhisUnits
    variable PhisUnit

    set PhisIndex [lsearch $PhisList $PhisSelc]
    if { $PhisFields != "" } {
	set PhisField [lindex $PhisFields $PhisIndex]
    }
    if { $PhisConds != "" } {
	set PhisCond [lindex $PhisConds $PhisIndex]
    }
    if { $PhisUnits != "" } {
	set PhisUnit [lindex $PhisUnits $PhisIndex]
    }
}

proc ConditionsWidget::ShowFieldSpecFunction { cas op args } {
    # cas puede ser field (condiciones tipo Field),
    #  cond (condiciones tipo Conditional) o value
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable SpeciesList
    variable SpeciesMenu
    variable SpeciesSelc
    variable SpeciesFields
    variable SpeciesField
    variable SpeciesConds
    variable SpeciesCond
    variable SpeciesValue 
    variable SpeciesFIni
    variable SpeciesFFld
    variable SpeciesUnits
    variable SpeciesUnit
    variable HelpFunc
    variable HelpCond 
    variable HelpUnits2
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    set f [frame $PARENT.f]
	    set gendata [.central.s info gendata] 
	    set idx [lsearch -regexp $gendata Species_Name]
	    set data [lindex $gendata [expr $idx+1]]
	    set SpeciesList [lrange $data 2 end]
	    set SpeciesSelc [lindex $SpeciesList [expr \
		    [DWLocalGetValue $GDN $STRUCT Concentration_index]-1]]
	    label $f.lbi -text "Species Name:"
	    set SpeciesMenu [eval [ concat tk_optionMenu $f.uni \
		    ConditionsWidget::SpeciesSelc $SpeciesList]] 
	    foreach Species $SpeciesList {
		$SpeciesMenu entryconfigure $Species \
		        -command ConditionsWidget::UpdateSpeciesData
	    }
	    GidHelp $f.lbi "Species selection" 
	    GidHelp $f.uni "Species selection"
	    grid $f.lbi $f.uni -sticky nw
	    if { $cas=="field" } {
		set SpeciesFIni [DWLocalGetValue $GDN $STRUCT Fix_Initial]
		set SpeciesFFld [DWLocalGetValue $GDN $STRUCT Fix_Field]
		checkbutton $f.ch1 -text "Fix Initial" \
		        -variable ConditionsWidget::SpeciesFIni -relief flat \
		        -command "ConditionsWidget::Link $f.ch1 \$ConditionsWidget::SpeciesFFld" 
		checkbutton $f.ch2 -text "Fix Field" \
		        -variable ConditionsWidget::SpeciesFFld -relief flat \
		        -command "ConditionsWidget::Link $f.ch1 \$ConditionsWidget::SpeciesFFld" 
		ConditionsWidget::Link $f.ch1 $SpeciesFFld
		GidHelp $f.ch1 "Fix the concentration to the initial value (t=0) \
		        given in the \"Value Field\" field."
		GidHelp $f.ch2 "Fix the concentration to the (time dependant) \
		        field given in \"Value Field\"."
		grid $f.ch1 -sticky nw
		grid $f.ch2 -sticky nw
	    }
	    if { $cas=="field" || $cas=="cond"} {
		set idx [lsearch -regexp $gendata Advect_Initdata]
		set data [lindex $gendata [expr $idx+1]]
		set SpeciesFields [lstep $data 2 2]
		set SpeciesField [lindex $SpeciesFields 0]
		set idx [lsearch -regexp $gendata Advect_Solverdata]
		set data [lindex $gendata [expr $idx+1]]
		set SpeciesUnits [lstep $data 7 8]
		set SpeciesUnit [TUI_formatunits Nums2Unicode [lindex $SpeciesUnits 0]]
		label $f.lb1 -text "Value Field:"
		label $f.lb2 -relief sunken -borderwidth 1 -anchor w \
		        -wraplength 85 -width 14 -justify left \
		        -textvariable ConditionsWidget::SpeciesField
		label $f.lbu -textvariable ConditionsWidget::SpeciesUnit
		GidHelp $f.lb1 $HelpFunc
		GidHelp $f.lb2 $HelpFunc
		GidHelp $f.lbu $HelpUnits2
		grid $f.lb1 $f.lb2 $f.lbu -sticky nw
	  } else {
	      set SpeciesFields ""
	  }
	  if { $cas=="cond"} {
		set idx [lsearch -regexp $gendata Advect_Initdata]
		set data [lindex $gendata [expr $idx+1]]
		set SpeciesConds [lstep $data 2 3]
		set SpeciesCond [lindex $SpeciesConds 0]
		label $f.lb3 -text "Conditional:"
		label $f.lb4 -relief sunken -borderwidth 1 -anchor w \
		        -wraplength 85 -width 14 -justify left \
		        -textvariable ConditionsWidget::SpeciesCond
		GidHelp $f.lb3 $HelpCond
		GidHelp $f.lb4 $HelpCond
		grid $f.lb3 $f.lb4 -sticky nw
	    }  else {
		set SpeciesConds ""
	    }
	    if { $cas=="value"} {
		set SpeciesValue [DWLocalGetValue $GDN $STRUCT {Value:}]
		set idx [lsearch -regexp $gendata Advect_Solverdata]
		set data [lindex $gendata [expr $idx+1]]
		set SpeciesUnits [lstep $data 7 8]
		set SpeciesUnit [TUI_formatunits Nums2Unicode [lindex $SpeciesUnits 0]]
		label $f.lb5 -text "Value:"
		entry $f.lb6 -relief sunken -borderwidth 1 -width 17 \
		        -textvariable ConditionsWidget::SpeciesValue
		GidHelp $f.lb5 "Given concentration"
		GidHelp $f.lb6 "Given concentration"
		label $f.lbu -textvariable ConditionsWidget::SpeciesUnit
		grid $f.lb5 $f.lb6 $f.lbu -sticky nw
	    }
	    grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		    -pady 3 -padx 2
	    grid rowconf $PARENT $ROW -weight 1
	    grid columnconf $PARENT 1 -weight 1

	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    set SpeciesIndex [lsearch $SpeciesList $SpeciesSelc]
	    DWLocalSetValue $GDN $STRUCT Concentration_index [expr $SpeciesIndex+1]
	    if { $cas=="value"} {
		if { [TUI_valuecheck $SpeciesValue "float"]!="ok" } {
		    set SpeciesValue [tk_dialogEntryRAM .gid.tempwine [_ "Enter value window"] \
		            "Enter correct value" questhead real $SpeciesValue]
		}
		DWLocalSetValue $GDN $STRUCT Value $SpeciesValue
	    }
	    if { $cas=="field" } {
		DWLocalSetValue $GDN $STRUCT Fix_Field $SpeciesFFld
		DWLocalSetValue $GDN $STRUCT Fix_Initial $SpeciesFIni
	    }
	    return ""
	}
    }
}

proc ConditionsWidget::UpdateSpeciesData { } {
    variable SpeciesList
    variable SpeciesMenu
    variable SpeciesSelc
    variable SpeciesFields
    variable SpeciesField
    variable SpeciesConds
    variable SpeciesCond
    variable SpeciesUnits
    variable SpeciesUnit

    set SpeciesIndex [lsearch $SpeciesList $SpeciesSelc]
    if { $SpeciesFields != "" } {
	set SpeciesField [lindex $SpeciesFields $SpeciesIndex]
    }
    if { $SpeciesConds != "" } {
	set SpeciesCond [lindex $SpeciesConds $SpeciesIndex]
    }
    if { $SpeciesUnits != "" } {
	set SpeciesUnit [lindex $SpeciesUnits $SpeciesIndex]
    }
}

proc ConditionsWidget::Link { w val } {
    if { $val } {
	$w deselect 
	$w configure -state disabled
    } else {
	$w configure -state normal
    }
}

proc ConditionsWidget::ShowFieldFunction { func units op args } {
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable HelpField
    variable HelpUnits1
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    if { $args != "" } {
		set gendata [.central.s info gendata]
		set idx [lsearch -regexp $gendata $func]
		set tx [lindex $gendata [expr $idx+1]]
		set f [frame $PARENT.f]
		set l1 [label $f.lb1 -text "Value Field:"]
		set l2 [label $f.lb2 -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $tx]
		set idx [lsearch -regexp $gendata $units]
		set tx [TUI_formatunits Nums2Unicode [lindex $gendata [expr $idx+1]]]
		set l3 [label $f.lb3 -text $tx]
		GidHelp $l1 $HelpField
		GidHelp $l2 $HelpField
		GidHelp $l3 $HelpUnits1
		grid $l1 $l2 $l3 -sticky nw
		grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		        -pady 3 -padx 2
		grid rowconf $PARENT $ROW -weight 1
		grid columnconf $PARENT 1 -weight 1
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    return ""
	}
    }
}

proc ConditionsWidget::ShowCondFunction { func cond units op args } {
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable HelpFunc
    variable HelpCond
    variable HelpUnits1
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]
	    if { $args != "" } {
		set gendata [.central.s info gendata]
		set idx [lsearch -regexp $gendata $func]
		set tx [lindex $gendata [expr $idx+1]]
		set f [frame $PARENT.f]
		set l1 [label $f.lb1 -text "Value Field:"]
		set l2 [label $f.lb2 -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $tx]
		set idx [lsearch -regexp $gendata $cond]
		set tx [lindex $gendata [expr $idx+1]]
		set l3 [label $f.lb3 -text "Conditional:"]
		set l4 [label $f.lb4 -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $tx]
		set idx [lsearch -regexp $gendata $units]
		set tx [TUI_formatunits Nums2Unicode [lindex $gendata [expr $idx+1]]]
		set lu [label $f.lu -text $tx]
		GidHelp $lu $HelpUnits1
		GidHelp $l1 $HelpFunc
		GidHelp $l2 $HelpFunc
		GidHelp $l3 $HelpCond
		GidHelp $l4 $HelpCond
		grid $l1 $l2 $lu -sticky nw
		grid $l3 $l4 -sticky nw
		grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		        -pady 3 -padx 2
		grid rowconf $PARENT $ROW -weight 1
		grid columnconf $PARENT 1 -weight 1
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    return ""
	}
    }
}

proc ConditionsWidget::Show2DFieldFunction { fieldX fieldY units op args } {
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable HelpField 
    variable HelpUnits1
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]
	    if { $args != "" } {
		set gendata [.central.s info gendata]
		set idx [lsearch -regexp $gendata $fieldX]
		set txX [lindex $gendata [expr $idx+1]]
		set idx [lsearch -regexp $gendata $fieldY]
		set txY [lindex $gendata [expr $idx+1]]
		set f [frame $PARENT.f]
		set l1X [label $f.lb1x -text "Value Field X:"]
		set l2X [label $f.lb2x -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txX]
		set l1Y [label $f.lb1y -text "Value Field Y:"]
		set l2Y [label $f.lb2y -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txY]
		set idx [lsearch -regexp $gendata $units]
		set tx [TUI_formatunits Nums2Unicode [lindex $gendata [expr $idx+1]]]
		set lux [label $f.lux -text $tx]
		set luy [label $f.luy -text $tx]
		GidHelpRecursive $f $HelpField
		GidHelp $lux $HelpUnits1
		GidHelp $luy $HelpUnits1
		grid $l1X $l2X $lux
		grid $l1Y $l2Y $luy
		grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		        -pady 3 -padx 2
		grid rowconf $PARENT $ROW -weight 1
		grid columnconf $PARENT 1 -weight 1
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    return ""
	}
    }
}

proc ConditionsWidget::Show3DFieldFunction { fieldX fieldY fieldZ units op args } {
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable HelpField
    variable HelpUnits1
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    if { $args != "" } {
		set gendata [.central.s info gendata]
		set idx [lsearch -regexp $gendata $fieldX]
		set txX [lindex $gendata [expr $idx+1]]
		set idx [lsearch -regexp $gendata $fieldY]
		set txY [lindex $gendata [expr $idx+1]]
		set idx [lsearch -regexp $gendata $fieldZ]
		set txZ [lindex $gendata [expr $idx+1]]
		set f [frame $PARENT.f]
		set l1X [label $f.lb1x -text "Value Field X:"]
		set l2X [label $f.lb2x -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txX]
		set l1Y [label $f.lb1y -text "Value Field Y:"]
		set l2Y [label $f.lb2y -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txY]
		set l1Z [label $f.lb1z -text "Value Field Z:"]
		set l2Z [label $f.lb2z -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txZ]
		set idx [lsearch -regexp $gendata $units]
		set tx [TUI_formatunits Nums2Unicode [lindex $gendata [expr $idx+1]]]
		set lux [label $f.lux -text $tx]
		set luy [label $f.luy -text $tx]
		set luz [label $f.luz -text $tx]
		GidHelpRecursive $f $HelpField
		GidHelp $lux $HelpUnits1
		GidHelp $luy $HelpUnits1 
		GidHelp $luz $HelpUnits1
		grid $l1X $l2X $lux
		grid $l1Y $l2Y $luy
		grid $l1Z $l2Z $luz
		grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		        -pady 3 -padx 2
		grid rowconf $PARENT $ROW -weight 1
		grid columnconf $PARENT 1 -weight 1
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    return ""
	}
    }
}

proc ConditionsWidget::Show2DCondFunction { fieldX condX fieldY condY units op args } {
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable HelpFunc
    variable HelpCond 
    variable HelpUnits1
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]
	    if { $args != "" } {
		set gendata [.central.s info gendata]
		set f [frame $PARENT.f]
		set idx [lsearch -regexp $gendata $fieldX]
		set txX [lindex $gendata [expr $idx+1]]
		set l1X [label $f.lb1x -text "Value Field X:"]
		set l2X [label $f.lb2x -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txX]
		set idx [lsearch -regexp $gendata $condX]
		set txX [lindex $gendata [expr $idx+1]]
		set l3X [label $f.lb3x -text "Conditional X:"]
		set l4X [label $f.lb4x -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txX]
		set idx [lsearch -regexp $gendata $fieldY]
		set txY [lindex $gendata [expr $idx+1]]
		set l1Y [label $f.lb1y -text "Value Field Y:"]
		set l2Y [label $f.lb2y -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txY]
		set idx [lsearch -regexp $gendata $condY]
		set txY [lindex $gendata [expr $idx+1]]
		set l3Y [label $f.lb3y -text "Conditional Y:"]
		set l4Y [label $f.lb4y -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 85 -width 14 -justify left \
		        -text $txY]
		set idx [lsearch -regexp $gendata $units]
		set tx [TUI_formatunits Nums2Unicode [lindex $gendata [expr $idx+1]]]
		set lux [label $f.lux -text $tx]
		set luy [label $f.luy -text $tx]
		GidHelp $lux $HelpUnits1
		GidHelp $luy $HelpUnits1 
		GidHelp $l1X $HelpFunc
		GidHelp $l2X $HelpFunc        
		GidHelp $l3X $HelpCond
		GidHelp $l4X $HelpCond
		GidHelp $l1Y $HelpFunc
		GidHelp $l2Y $HelpFunc        
		GidHelp $l3Y $HelpCond
		GidHelp $l4Y $HelpCond
		grid $l1X $l2X $lux    
		grid $l3X $l4X
		grid $l1Y $l2Y $luy
		grid $l3Y $l4Y
		grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		        -pady 3 -padx 2
		grid rowconf $PARENT $ROW -weight 1
		grid columnconf $PARENT 1 -weight 1
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    return ""
	}
    }
}

proc ConditionsWidget::Show3DCondFunction { fieldX condX fieldY condY fieldZ condZ units op args } {
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable HelpFunc
    variable HelpCond
    variable HelpUnits1
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]
	    if { $args != "" } {
		set gendata [.central.s info gendata]
		set f [frame $PARENT.f]
		set idx [lsearch -regexp $gendata $fieldX]
		set txX [lindex $gendata [expr $idx+1]]
		set l1X [label $f.lb1x -text "Value Field X:"]
		set l2X [label $f.lb2x -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 100 -width 17 -justify left \
		        -text $txX]
		set idx [lsearch -regexp $gendata $condX]
		set txX [lindex $gendata [expr $idx+1]]
		set l3X [label $f.lb3x -text "Conditional X:"]
		set l4X [label $f.lb4x -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 100 -width 17 -justify left \
		        -text $txX]
		set idx [lsearch -regexp $gendata $fieldY]
		set txY [lindex $gendata [expr $idx+1]]
		set l1Y [label $f.lb1y -text "Value Field Y:"]
		set l2Y [label $f.lb2y -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 100 -width 17 -justify left \
		        -text $txY]
		set idx [lsearch -regexp $gendata $condY]
		set txY [lindex $gendata [expr $idx+1]]
		set l3Y [label $f.lb3y -text "Conditional Y:"]
		set l4Y [label $f.lb4y -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 100 -width 17 -justify left \
		        -text $txY]
		set idx [lsearch -regexp $gendata $fieldZ]
		set txZ [lindex $gendata [expr $idx+1]]
		set l1Z [label $f.lb1z -text "Value Field Z:"]
		set l2Z [label $f.lb2z -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 100 -width 17 -justify left \
		        -text $txZ]
		set idx [lsearch -regexp $gendata $condZ]
		set txZ [lindex $gendata [expr $idx+1]]
		set l3Z [label $f.lb3z -text "Conditional Z:"]
		set l4Z [label $f.lb4z -relief sunken -borderwidth 1 \
		        -anchor w -wraplength 100 -width 17 -justify left \
		        -text $txZ]
		set idx [lsearch -regexp $gendata $units]
		set tx [TUI_formatunits Nums2Unicode [lindex $gendata [expr $idx+1]]]
		set lux [label $f.lux -text $tx]
		set luy [label $f.luy -text $tx]
		set luz [label $f.luz -text $tx]
		GidHelp $lux $HelpUnits1
		GidHelp $luy $HelpUnits1 
		GidHelp $luz $HelpUnits1
		GidHelp $l1X $HelpFunc
		GidHelp $l2X $HelpFunc        
		GidHelp $l3X $HelpCond
		GidHelp $l4X $HelpCond
		GidHelp $l1Y $HelpFunc
		GidHelp $l2Y $HelpFunc        
		GidHelp $l3Y $HelpCond
		GidHelp $l4Y $HelpCond
		GidHelp $l1Z $HelpFunc
		GidHelp $l2Z $HelpFunc        
		GidHelp $l3Z $HelpCond
		GidHelp $l4Z $HelpCond
		grid $l1X $l2X $lux      
		grid $l3X $l4X
		grid $l1Y $l2Y $luy
		grid $l3Y $l4Y
		grid $l1Z $l2Z $luz
		grid $l3Z $l4Z
		grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		        -pady 3 -padx 2
		grid rowconf $PARENT $ROW -weight 1
		grid columnconf $PARENT 1 -weight 1
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    return ""
	}
    }
}


proc ConditionsWidget::ShowValueType { type op args } {
    # type es el tipo de condicion Temperature, Pressure, ...
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    variable ${type}_Value
    variable ${type}_Units
    variable ${type}_Help
    variable HelpUnits1

    switch $op {
	"INIT" {
	    set gendata [.central.s info gendata]
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    set f [frame $PARENT.f]
	    set idx [lsearch -regexp $gendata Boundary_${type}_Units]
	    set ${type}_Units [TUI_formatunits Nums2Unicode [lindex $gendata [expr $idx+1]]]
	    set ${type}_Value [DWLocalGetValue $GDN $STRUCT {Value:}]
	    label $f.lbv -text "Value:"
	    entry $f.val -relief sunken -borderwidth 1 -width 17 \
		    -textvariable ConditionsWidget::${type}_Value
	    label $f.lbu -textvariable ConditionsWidget::${type}_Units
	    grid $f.lbv $f.val $f.lbu -sticky nw
	    GidHelpRecursive $f [set ${type}_Help]
	    GidHelp $f.lbu $HelpUnits1
	    grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		    -pady 3 -padx 2
	    grid rowconf $PARENT $ROW -weight 1
	    grid columnconf $PARENT 1 -weight 1

	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]
	    if { [TUI_valuecheck [set ${type}_Value] "float"]!="ok" } {
		set ${type}_Value [tk_dialogEntryRAM .gid.tempwine [_ "Enter value window"] \
			"Enter correct value" questhead real [set ${type}_Value]]
	    }
	    DWLocalSetValue $GDN $STRUCT Value [set ${type}_Value]
	    
	    return ""
	}
    }
}

proc ConditionsWidget::ShowValueVelocity { type op args } {
    # type puede ser 2D o 3D
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    
    variable VelocityX_Value
    variable VelocityY_Value
    variable VelocityZ_Value
    variable VelocityX_Fix
    variable VelocityY_Fix
    variable VelocityZ_Fix
    variable Velocity_Units
    variable Velocity_Help
    variable HelpUnits1

    switch $op {
	"INIT" {
	    set gendata [.central.s info gendata]
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	    set STRUCT [lindex $args 3]
	    set f [frame $PARENT.f]
	    set idx [lsearch -regexp $gendata Boundary_Velocity_Units]
	    set Velocity_Units [TUI_formatunits Nums2Unicode [lindex \
		    $gendata [expr $idx+1]]]
	    set VelocityX_Value [DWLocalGetValue $GDN $STRUCT X_Component]
	    set VelocityY_Value [DWLocalGetValue $GDN $STRUCT Y_Component]
	    set VelocityX_Fix [DWLocalGetValue $GDN $STRUCT Fix_X]
	    set VelocityY_Fix [DWLocalGetValue $GDN $STRUCT Fix_Y]
	    set enX [entry $f.vlx -relief sunken -borderwidth 1 -width 14 \
		    -state disabled -textvariable ConditionsWidget::VelocityX_Value]
	    set fxX [checkbutton $f.fxx -text "Fix X:" \
		    -variable ConditionsWidget::VelocityX_Fix\
		    -command "ConditionsWidget::LinkVlc $enX \$ConditionsWidget::VelocityX_Fix"]
	    set lbX [label $f.lbx -textvariable ConditionsWidget::Velocity_Units]
	    set enY [entry $f.vly -relief sunken -borderwidth 1 -width 14 \
		    -state disabled -textvariable ConditionsWidget::VelocityY_Value]
	    set fxY [checkbutton $f.fxy -text "Fix Y:" \
		    -variable ConditionsWidget::VelocityY_Fix\
		    -command "ConditionsWidget::LinkVlc $enY \$ConditionsWidget::VelocityY_Fix"]
	    set lbY [label $f.lby -textvariable ConditionsWidget::Velocity_Units]

	    grid $fxX $enX $lbX -sticky nw
	    grid $fxY $enY $lbY -sticky nw

	    if { $type == "3D" } {
		set VelocityZ_Value [DWLocalGetValue $GDN $STRUCT Z_Component]
		set VelocityZ_Fix [DWLocalGetValue $GDN $STRUCT Fix_Z]
		set enZ [entry $f.vlz -relief sunken -borderwidth 1 -width 14 \
		        -state disabled -textvariable ConditionsWidget::VelocityZ_Value]
		set fxZ [checkbutton $f.fxz -text "Fix Z:" \
		        -variable ConditionsWidget::VelocityZ_Fix\
		        -command "ConditionsWidget::LinkVlc $enZ \$ConditionsWidget::VelocityZ_Fix"]
		set lbZ [label $f.lbz -textvariable ConditionsWidget::Velocity_Units]
		grid $fxZ $enZ $lbZ -sticky nw
	    }
	    GidHelpRecursive $f $Velocity_Help
	    GidHelp $lbX $Velocity_Help
	    GidHelp $lbY $Velocity_Help
	    if { $type == "3D" } {
		GidHelp $lbZ $Velocity_Help
	    }
	    grid $f -row $ROW -column 0 -sticky nw -columnspan 2 \
		    -pady 3 -padx 2
	    grid rowconf $PARENT $ROW -weight 1
	    grid columnconf $PARENT 1 -weight 1

	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	    set STRUCT [lindex $args 1]

	    DWLocalSetValue $GDN $STRUCT Fix_X $ConditionsWidget::VelocityX_Fix
	    if { [TUI_valuecheck $VelocityX_Value "float"]!="ok" } {
		set VelocityX_Value [tk_dialogEntryRAM .gid.tempwine [_ "Enter value window"] \
			"Enter correct value for X component" questhead real $VelocityX_Value]
	    }
	    DWLocalSetValue $GDN $STRUCT X_Component $ConditionsWidget::VelocityX_Value
	    DWLocalSetValue $GDN $STRUCT Fix_Y $ConditionsWidget::VelocityY_Fix
	    if { [TUI_valuecheck $VelocityY_Value "float"]!="ok" } {
		set VelocityY_Value [tk_dialogEntryRAM .gid.tempwine [_ "Enter value window"] \
			"Enter correct value for Y component" questhead real $VelocityY_Value]
	    }
	    DWLocalSetValue $GDN $STRUCT Y_Component $ConditionsWidget::VelocityY_Value

	    if { $type == "3D" } {
		DWLocalSetValue $GDN $STRUCT Fix_Z $ConditionsWidget::VelocityZ_Fix
		if { [TUI_valuecheck $VelocityZ_Value "float"]!="ok" } {
		    set VelocityZ_Value [tk_dialogEntryRAM .gid.tempwine [_ "Enter value window"] \
			    "Enter correct value for Z component" questhead real $VelocityZ_Value]
		}
		DWLocalSetValue $GDN $STRUCT Z_Component $ConditionsWidget::VelocityZ_Value
	    }

	    return ""
	}
    }
}

proc ConditionsWidget::LinkVlc { w val } {
    if { $val==0 } {
	$w configure -state disabled
    } else {
	$w configure -state normal
    }
}

###################################################
# UTILIDAD TKWIDGET PARA CONTROL DE DATOS GENERALES
###################################################
#
# Requiere ser llamado desde el archivo de definicin del 
# material, datos, ... en la forma:
# TKWIDGET: CheckDataWidget::Control "Field1 type1 Field2 type2 ..."
# Donde Field1, Field2, etc. son los diferentes campos QUESTION y
# type1, type2, etc. son los tipos de entrada, que pueden ser:
#
#      word:    word without spaces
#      any:     anything
#      real:    real
#      real+:   positive real
#      real0+:  positive real, including 0
#      int:     int
#      int+:    positive int
#      int0+:   positive int, including 0
#
# Note: Types must be lowercase
#
namespace eval CheckDataWidget {        
}

proc CheckDataWidget::Control { flds op args } {
    # op puede ser INIT,SYNC o CLOSE
    # INIT es enviado al abrir la ventana
    # SYNC es enviado al pulsar AcceptData
    # CLOSE es enviado al cerrar la ventana
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]

	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	      set STRUCT [lindex $args 1]
   
	    for { set i 1 } { $i < [llength $flds] } { incr i 2 } {
		set type [lindex $flds $i]
		set QUESTION [lindex $flds [expr $i-1]]
		if { [catch {set ifld $GidData($STRUCT,LABEL,$QUESTION)}] } {
		    set ifld [LabelField $GDN $STRUCT $QUESTION]
		}
		if { $ifld != -1 } {
		    set val [DWLocalGetValue $GDN $STRUCT $QUESTION]
		    set errmessage [IsType $type $val .gid]
		    if { $errmessage != "" } {
		        append errmessage " in field $QUESTION" 
		        set w .gid.cdata
		        while { [winfo exists $w] } { append w a }
		        set newval [tk_dialogEntryType $w \
		                "Enter correct value" $errmessage "" $type $val]
		        DWLocalSetValue $GDN $STRUCT $QUESTION $newval
		    } 
		}
	    }
	    return ""
	}
    }
}

###################################################
# UTILIDAD TKWIDGET PARA EDICIN DE FUNCIONES EN 
# UN MATERIAL Y CONTROL DE DATOS
###################################################

namespace eval MaterialWidget {        
    # Array Ventana de material 
    # (todas las que se han abierto se guardan aqui)
    variable PARENT
    # Array Cdigo de la estructura del material
    variable GDN
}

proc MaterialWidget::Control { geom flds op args } {
    # flds es la lista de campos y tipos para el editor de funciones y control
    # ffunc indica una funcin para fluidos
    # sfunc indica una funcin para slidos
    # func es una funcin bsica
    # real[0,+] int[0,+] se utilizan para controlar reales o enteros

    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]

	    # STRUCT es el cdigo (HEAD) del Material Ej. MAT,Steel
	    set MaterialWidget::PARENT($STRUCT) $PARENT
	    set MaterialWidget::GDN($STRUCT) $GDN

	    # Controles para evitar errores detectados en GiD
	    if { [llength $flds] < 1 } { return }
	    set ifld -1
	    catch { set ifld [LabelField $GDN $STRUCT [lindex $flds 0]] }
	    if { $ifld == -1 } { return }

	    # Modificamos la ventana de materiales para poner un boton 
	    # que abre el editor de funciones y cambiamos el boton Assign
	    upvar \#0 $GDN GidData
	    set w $GidData(WINDOW)

	    # Cambiamos las entradas del menu Assign, para adaptarlas a Tdyn
	    set MenuAssign $w.but.assign.menu
	    if { [winfo exists $MenuAssign] } {
		$MenuAssign configure -postcommand \
		        "PostCmdAssignMatTdyn $GDN $w $MenuAssign $geom"
	    }
	    # Aadimos un menubutton para el editor de funciones
	    set func_fields ""
	    set func_mattype ""
	    for { set i 1 } { $i < [llength $flds] } { incr i 2 } {
		set TYPE [lindex $flds $i]
		if { [string compare -nocase -length 4 $TYPE "FUNC"]==0 } {
		    lappend func_fields [lindex $flds [expr $i-1]]
		    lappend func_mattype Basic
		} elseif { [string compare -nocase -length 5 $TYPE "FFUNC"]==0 } {
		    lappend func_fields [lindex $flds [expr $i-1]]        
		    lappend func_mattype Fluid
		} elseif { [string compare -nocase -length 5 $TYPE "SFUNC"]==0 } {
		    lappend func_fields [lindex $flds [expr $i-1]]        
		    lappend func_mattype Solid
		}
	    }
	    if { $func_fields!="" && [winfo exists $w.but] } {
		set menufields $w.but.fx.field
		if { ![winfo exists $w.but.fx] } {
		    menubutton $w.but.fx -image [imagefromtdyn sym-fx.gif] -und 0 \
		            -relief raised -width 40 -height 20 -bd 2 -highlightthickness 1\
		            -menu $menufields
		    menu $menufields -bd 1 -activeborderwidth 1
		    set len [llength $func_fields]
		    for { set i 0 } { $i < $len } { incr i } {
		        set field [lindex $func_fields $i]
		        set mattype [lindex $func_mattype $i]
		        if { $field != "" } {
		            $menufields add command -label $field \
		                    -command "FunctionEditor::Start $PARENT $GDN $STRUCT $field $mattype"
		        }
		    }
		    if { ![catch { set manager [winfo manager $w.but.assign] }] } {
		        if { $manager=="grid" } {
		            grid $w.but.fx -row 1 -column 6 -sticky e -padx 5 -pady 5
		        } elseif { $manager=="pack" } {
		            pack $w.but.fx -side right
		        }
		    }
		    GidHelp $w.but.fx "Open Function editor" 
		} elseif { [winfo exists $menufields] } {
		$menufields delete 0 end
		    set len [llength $func_fields]
		    for { set i 0 } { $i < $len } { incr i } {
		        set field [lindex $func_fields $i]
		        set mattype [lindex $func_mattype $i]
		        if { $field != "" } {
		            $menufields add command -label $field \
		                    -command "FunctionEditor::Start $PARENT $GDN $STRUCT $field $mattype"
		        }
		    }
		}
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	      set STRUCT [lindex $args 1]

	    # Controles para evitar errores detectados en GiD
	    if { [llength $flds] < 1 } { return }
	    set ifld -1
	    catch { set ifld [LabelField $GDN $STRUCT [lindex $flds 0]] }
	    if { $ifld == -1 } { return } 

	    for { set i 1 } { $i < [llength $flds] } { incr i 2 } {
		set type [lindex $flds $i]
		set field [lindex $flds [expr $i-1]]
		if { ![info exists GidData($STRUCT,VALUE,$field)] } {
		    set ifld -1
		    catch { set ifld [LabelField $GDN $STRUCT $field] }
		    if { $ifld != -1 } {
		        if { ![catch { set val [DWLocalGetValue $GDN $STRUCT $field]}] } {
		            set errmessage [IsType $type $val .gid]
		            if { $errmessage != "" } {
		                append errmessage " in field $field"
		                set newval [tk_dialogEntryType $MaterialWidget::PARENT($STRUCT) \
		                        "Enter correct value" $errmessage "" $type $val]
		                DWLocalSetValue $GDN $STRUCT $field $newval
		            } 
		        }
		    }
		}
	    }
	    return ""
	}
    }
}

proc PostCmdAssignMatTdyn { GDN w Menu {geom "Lines"}} {

    $Menu del 0 end
    if { [catch { set whatuse [lindex [.gid.central.s info Project] 4] }] } {
	set whatuse GEOMETRYUSE
    }
    switch $whatuse {
	GEOMETRYUSE {
	    $Menu add command -label $geom -command "DWAssignMaterials $GDN $w $geom"
	}
	MESHUSE {
	    $Menu add command -label Elements -command "DWAssignMaterials $GDN $w Elements"
	}
    }
}


###################################################
# UTILIDAD TKWIDGET PARA EDICIN DE FUNCIONES EN 
# UN MATERIAL Y CONTROL DE DATOS
###################################################

namespace eval ProblemDataWidget {        
    # Array Ventana de problem data
    # (todas las que se han abierto se guardan aqui)
    variable PARENT
    # Array Cdigo de la estructura del problem data
    variable GDN
}

proc ProblemDataWidget::Control { flds op args } {
    # flds es la lista de campos y tipos para el editor de funciones y control
    # ffunc indica una funcin para fluidos
    # sfunc indica una funcin para slidos
    # func es una funcin bsica
    # real[0,+] int[0,+] se utilizan para controlar reales o enteros

    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW
	    set GDN [lindex $args 2]
	      set STRUCT [lindex $args 3]

	    # STRUCT es el cdigo (HEAD) del Menu
	    set ProblemDataWidget::PARENT($STRUCT) $PARENT
	    set ProblemDataWidget::GDN($STRUCT) $GDN

	    # Controles para evitar errores detectados en GiD
	    if { [llength $flds] < 1 } { return }
	    set ifld -1
	    catch { set ifld [LabelField $GDN $STRUCT [lindex $flds 0]] }
	    if { $ifld == -1 } { return } 

	    # Modificamos la ventana de problem para poner un boton 
	    # que abre el editor de funciones
	    upvar \#0 $GDN GidData
	    set w $GidData(WINDOW)

	    # Aadimos un menubutton para el editor de funciones
	    set func_fields ""
	    set func_mattype ""
	    for { set i 1 } { $i < [llength $flds] } { incr i 2 } {
		set TYPE [lindex $flds $i]
		if { [string compare -nocase -length 4 $TYPE "FUNC"]==0 } {
		    lappend func_fields [lindex $flds [expr $i-1]]
		    lappend func_mattype Basic
		} elseif { [string compare -nocase -length 5 $TYPE "FFUNC"]==0 } {
		    lappend func_fields [lindex $flds [expr $i-1]]        
		    lappend func_mattype Fluid
		} elseif { [string compare -nocase -length 5 $TYPE "SFUNC"]==0 } {
		    lappend func_fields [lindex $flds [expr $i-1]]        
		    lappend func_mattype Solid
		}
	    }
	    if { $func_fields!="" && [winfo exists $w.but] } {
		set menufields $w.but.fx.field
		if { ![winfo exists $w.but.fx] } {
		    menubutton $w.but.fx -image [imagefromtdyn sym-fx.gif] -und 0 \
		            -relief raised -width 40 -height 20 -bd 2 -highlightthickness 1\
		            -menu $menufields
		    menu $menufields -bd 1 -activeborderwidth 1
		    set len [llength $func_fields]
		    for { set i 0 } { $i < $len } { incr i } {
		    set field [lindex $func_fields $i]
		        set mattype [lindex $func_mattype $i]
		        if { $field != "" } {
		            $menufields add command -label $field \
		                    -command "FunctionEditor::Start $PARENT $GDN $STRUCT $field $mattype"
		        }
		    }
		    if { ![catch { set manager [winfo manager $w.but.close] }] } {
		        if { $manager=="grid" } {
		            grid $w.but.fx -row 1 -column 4 -sticky e -padx 5 -pady 5
		        } elseif { $manager=="pack" } {
		            pack $w.but.fx -side right
		        }
		    }
		    GidHelp $w.but.fx "Open Function editor" 
		} elseif { [winfo exists $menufields] } {
		    $menufields delete 0 end
		    set len [llength $func_fields]
		    for { set i 0 } { $i < $len } { incr i } {
		        set field [lindex $func_fields $i]
		        set mattype [lindex $func_mattype $i]
		        if { $field != "" } {
		            $menufields add command -label $field \
		                -command "FunctionEditor::Start $PARENT $GDN $STRUCT $field $mattype"
		        }
		    }
		}
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
	      set STRUCT [lindex $args 1]

	    # Controles para evitar errores detectados en GiD
	    if { [llength $flds] < 1 } { return }
	    set ifld -1
	    catch { set ifld [LabelField $GDN $STRUCT [lindex $flds 0]] }
	    if { $ifld == -1 } { return } 

	    for { set i 1 } { $i < [llength $flds] } { incr i 2 } {
		set type [lindex $flds $i]
		set field [lindex $flds [expr $i-1]]
		if { ![info exists GidData($STRUCT,VALUE,$field)] } {
		    if { ![catch { set val [DWLocalGetValue $GDN $STRUCT $field]}] } {
		        set errmessage [IsType $type $val .gid]
		        if { $errmessage != "" } {
		            append errmessage " in field $field"
		            set newval [tk_dialogEntryType $ProblemDataWidget::PARENT($STRUCT) \
		                        "Enter correct value" $errmessage "" $type $val]
		            DWLocalSetValue $GDN $STRUCT $field $newval
		        }
		    }
		}
	    }
	    return ""
	}
    }
}

###################################################
# UTILIDAD TKWIDGET PARA EDICIN DE FUNCIONES EN UN
# CAMPO (MATERIALES, PROBLEM DATA, ETC.)
###################################################

namespace eval FunctionEditor {
    # Datos del editor de funciones
    variable used_functions ""
    variable type "Basic"
    variable available_functions [list sqrt abs ln log fac sin cos \
	    tan asin acos atan sinh cosh tanh exp srand if_then \
	    if_else interpolate int]
    variable example_functions {"sqrt()" "abs()" "ln()" "log()" "fac()" \
	    "sin()" "cos()" "tan()" "asin()" "acos()" "atan()" "sinh()" \
	    "cosh()" "tanh()" "exp()" "srand()" "if()then()else()endif" \
	    "if()then()elif()then()else()endif" "interpolate(# # )" "int()"}
    variable variables ""
    variable example_variables ""  
    variable basic_variables "t it rand dt e pi"
    variable example_basic_variables "t it tm rand dt exp(1) pi" 
    variable fluid_variables "x y z nx ny nz t it rand dt e pi vx vy vz pr tm\
	    vt v_ tau wc ds ke ep om kt pk dn bt sp#" 
    variable example_fluid_variables "x y z nx ny nz t it rand dt exp(1)\
	    pi vx vy vz pr tm vt v_ tau wc ds ke ep om kt pk dn bt sp#"
    variable solid_variables "x y z t it tm rand dt e pi"
    variable example_solid_variables "x y z t it tm rand dt exp(1) pi"
    variable operators ""
    variable basic_operators "+ - * / ^ mod div rmod rdiv max min ~ < <= >\
	    >= = != & |"
    variable fluid_operators "+ - * / ^ mod div rmod rdiv max min Dx() Dy() Dz()\
	    ~ < <= > >= = != & |"
    variable solid_operators "+ - * / ^ mod div rmod rdiv max min Dx() Dy() Dz()\
	    ~ < <= > >= = != & |"
}

proc FunctionEditor::Start { wn gdn struct {field ""} {type "Basic"} } {
    global tkPriv tcl_platform

    if { $type=="Fluid" } {
	set FunctionEditor::variables $FunctionEditor::fluid_variables
	set FunctionEditor::example_variables $FunctionEditor::example_fluid_variables
	set FunctionEditor::operators $FunctionEditor::fluid_operators
    } elseif { $type=="Solid" } {
	set FunctionEditor::variables $FunctionEditor::solid_variables
	set FunctionEditor::example_variables $FunctionEditor::example_solid_variables
	set FunctionEditor::operators $FunctionEditor::solid_operators
    } else {
	set FunctionEditor::variables $FunctionEditor::basic_variables
	set FunctionEditor::example_variables $FunctionEditor::example_basic_variables
	set FunctionEditor::operators $FunctionEditor::basic_operators
    }

    set w $wn.feditor
    if { $field != "" } {
	set title "Function editor: $field"
    } else {        
	set title "Function editor: $field"
    }
    if { [catch {
	InitWindow $w $title PreEditFunctionWindowGeom ""
    }] } { 
	toplevel $w -height 200 -width 200
	wm title $w $title
	wm iconname $w Dialog        
	update idletasks
	
	set pare [winfo toplevel [winfo parent $w]]
	set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
	set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
	if { $x < 0 } { set x 0 }
	if { $y < 0 } { set y 0 }
	wm geom $w +$x+$y
    }
    wm minsize $w 200 100

    # Frame que lo encierra todo
    frame $w.f -bd 2 -relief groove

    # Barra de botones
    set upbar [frame $w.f.upbar]
    frame $upbar.west
    frame $upbar.east
    grid $upbar.west -row 0 -column 0 -sticky w
    grid $upbar.east -row 0 -column 1 -sticky e
    grid columnconf $upbar {0 1} -weight 1

    button $upbar.west.cb1 -image [imagefromtdyn sym-fx.gif] -command \
	    "FunctionEditor::openfunclist $w"
    button $upbar.west.cb2 -image [imagefromtdyn sym-x.gif] -command \
	    "FunctionEditor::openvarlist $w"
    button $upbar.west.cb3 -image [imagefromtdyn sym-up.gif] -command \
	    "FunctionEditor::openoperlist $w"
    button $upbar.east.cb4 -image [imagefromtdyn questionarrow.gif] -command \
	    "PickHelp $w" -highlightthickness 0 
    set upbarls [tk_ListMenu $upbar.east "History" $FunctionEditor::used_functions \
	    "$w.f.t insert insert"]

    grid $upbar.west.cb1 -row 0 -column 0 -sticky w -pady 2 -padx 5 
    grid $upbar.west.cb2 -row 0 -column 1 -sticky w -pady 2 -padx 5
    grid $upbar.west.cb3 -row 0 -column 2 -sticky w -pady 2 -padx 5
    grid $upbarls  -row 0 -column 4 -sticky e -pady 2 -padx 10
    grid $upbar.east.cb4 -row 0 -column 5 -sticky e -pady 2 -padx 5

    GidHelp $upbar.cb1 "List of available functions" 
    GidHelp $upbar.cb2 "List of available variables and constants" 
    GidHelp $upbar.cb3 "List of available operators"
    GidHelp $upbar.cb4 "History of used funcions"

    # Editor texto
    text $w.f.t -xscroll "$w.f.hscr set" -yscroll "$w.f.vscr set" \
	    -width 60    
    scrollbar $w.f.vscr -orient vertical -comm "$w.f.t yview"
    scrollbar $w.f.hscr -orient horizontal -comm "$w.f.t xview"

    bind $w.f.t <KeyPress> {
	if { [regexp {[a-zA-Z]} %A] } {
	    catch {
		if {[%W compare sel.first <= insert]
		&& [%W compare sel.last >= insert]} {
		    %W delete sel.first sel.last
		}
	    }
	    %W insert insert %A ""
	    break 
	}
    }

    grid $w.f.upbar -row 0 -column 0 -sticky ew -pady 5 -columnspan 2
    grid $w.f.t -row 1 -column 0 -padx 2 -sticky nsew
    grid $w.f.vscr -row 1 -column 1 -sticky ns
    grid $w.f.hscr -row 2 -column 0 -sticky ew -padx 2 
    grid columnconf $w.f 0 -weight 1
    grid rowconf $w.f 1 -weight 1

    frame $w.dbuts
    button $w.dbuts.ok -text OK -width 5 -command "set tkPriv(ed_button) ok" -und 0
    button $w.dbuts.clear -text Clear -width 5 -command "$w.f.t del 1.0 end" -und 1
    button $w.dbuts.cancel -text Cancel -width 5 -command "set tkPriv(ed_button) cancel" -und 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $w.dbuts.cancel"
    bind $w.dbuts.ok <Return> "tkButtonInvoke $w.dbuts.ok"
    bind $w <Escape> "tkButtonInvoke $w.dbuts.cancel"
    bind $w.f.t <Alt-o> "tkButtonInvoke $w.dbuts.ok"
    bind $w.f.t <Alt-l> "tkButtonInvoke $w.dbuts.clear"
    bind $w.f.t <Alt-c> "tkButtonInvoke $w.dbuts.cancel"

    grid $w.f -row 0 -column 0  -pady 5 -sticky ewns \
	    -padx 2
    grid $w.dbuts -row 1 -column 0
    grid $w.dbuts.ok -row 0 -column 0 -sticky ew -padx 3 -pady 2
    grid $w.dbuts.clear -row 0 -column 1 -sticky ew -padx 3 -pady 2
    grid $w.dbuts.cancel -row 0 -column 2 -sticky ew -padx 3 -pady 2

    # Nueva entrada del menu, para seleccionar el campo sobre el que actua
    #if { [llength $field] > 1 } {
    #        eval tk_optionMenu $w.dbuts.field sel_field $field
    #        grid $w.dbuts.field -row 0 -column 3 -sticky ew -padx 3 -pady 2
    #        GidHelp $w "Insert the function, Select the field to edit \
    #                and press OK to update the material data"
    #        GidHelp $w.f.t "Insert the function, Select the field to edit \
    #                and press OK to update the material data"
    #  }
    if { $field == "" } {
	set field $MaterialWidget::EDIT($struct)
    }
    # Lee la informacin actual y la pone en la ventana
    catch { $w.f.t ins end [DWLocalGetValue $gdn $struct $field] }
    GidHelp $w "Insert or edit the function and press OK to update material data" 
    GidHelp $w.f.t "Insert or edit the function and press OK to update material data"

    grid columnconf $w 0 -weight 1
    grid rowconf $w 0 -weight 1

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    update
    grab $w
    focus $w.f.t
    after 100 catch [list "focus -force $w.f.e"]

    # Se ha pulsado un botn?
    while 1 {
	tkwait variable tkPriv(ed_button)

	set retval ""
	if { $tkPriv(ed_button) == "ok" } {
	    set retval [$w.f.t get 1.0 end]
	    break
	} else { break }
    }
    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }
    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    catch { grab -global $oldGrab }
	} else {
	    catch { grab $oldGrab }
	}
    }
    # Actualiza el valor del campo y antes elimina los cdigos incmodos
#    regsub -all { } $retval {} retval
#    regsub -all {\t} $retval {} retval 
    regsub -all {\n} $retval {} retval  
    regsub -all {;} $retval {} retval 
    if { $field != "" && $retval != "" } { 
	catch { DWLocalSetValue $gdn $struct $field $retval }
	lappend FunctionEditor::used_functions $retval
	if { [llength $FunctionEditor::used_functions] > 15 } {
	    set FunctionEditor::used_functions \
		    [lrange $FunctionEditor::used_functions 1 end] 
	}
    }
    return $retval
}

proc FunctionEditor::openfunclist { w } {
    tk_ListBox $w {Function list} {Select the function to be inserted} \
	    "$FunctionEditor::available_functions" \
	    "FunctionEditor::sendfuncselc $w.f.t" 
}

proc FunctionEditor::sendfuncselc { w idx } {
    if { $idx!="" } {
	set text [lindex $FunctionEditor::example_functions $idx]
	$w insert insert $text
    } else {
	WarnWin "Error in sendfunclist $idx"
    }
}

proc FunctionEditor::openvarlist { w } {
    tk_ListBox $w {Variable and constant list} \
    {Select the variable or constant to be inserted} \
	    "$FunctionEditor::variables" \
	    "FunctionEditor::sendvarselc $w.f.t" 
}

proc FunctionEditor::sendvarselc { w idx } {
    if { $idx!="" } {
	set text [lindex $FunctionEditor::example_variables $idx]
	$w insert insert $text
    } else {
	WarnWin "Error in sendvarselc $idx"
    }
}

proc FunctionEditor::openoperlist { w } {
    tk_ListBox $w {Operator list} \
    {Select the operator to be inserted} \
	    "$FunctionEditor::operators" \
	    "FunctionEditor::sendoperselc $w.f.t" 
}

proc FunctionEditor::sendoperselc { w idx } {
    if { $idx!="" } {
	set text [lindex $FunctionEditor::operators $idx]
	$w insert insert $text
    } else {
	WarnWin "Error in sendoperselcs $idx"
    }
}

###################################################
# UTILIDADES CREACIN DE LISTBOX, ENTRY Y OTROS
###################################################

# proc IsType
#
# w: parent window
# type -        Can be:
#                        word:    word without spaces
#                        any:     anything
#                        real:    real
#                        real+:   positive real
#                        real0+:  positive real, including 0
#                        int:     int
#                        int+:    positive int
#                        int0+:   positive int, including 0

proc IsType { type word { w .gid } { errorText "" } } {

    set errorRes ""
    switch $type {
	word {
	    if { ![regexp {^[  ]*[^ \t]+[  ]*$} $word] } {
		set errorRes "A single word must be entered"
	    }
	}
	real {
	    if { ![regexp \
		    {^[  ]*([+-]?([0-9]+\.?[0-9]*|[0-9]*\.?[0-9]+)([eE][+-]?[0-9]+)?)[  ]*$} \
		    $word] } {
		set errorRes "One number must be entered"
	    }           
	}
	real+ {
	    if { ![regexp \
		    {^[  ]*([+]?([0-9]+\.?[0-9]*|[0-9]*\.?[0-9]+)([eE][+-]?[0-9]+)?)[  ]*$} \
		    $word] } {
		set errorRes "One positive number must be entered"
	    } elseif { $word == 0 } {
		set errorRes "One positive number must be entered"
	    }
	}
	real0+ {
	    if { ![regexp \
		    {^[  ]*([+]?([0-9]+\.?[0-9]*|[0-9]*\.?[0-9]+)([eE][+-]?[0-9]+)?)[  ]*$} \
		    $word] } {
		set errorRes "One non negative number must be entered"
	    }
	}
	int {
	    if { ![regexp \
		    {^[  ]*[+-]?[0-9]+[  ]*$} \
		    $word] } {
		set errorRes "One integer number must be entered"
	    }           
	}
	int+ {
	    if { ![regexp \
		    {^[  ]*[+]?[0-9]*[1-9]+[0-9]*[  ]*$} \
		    $word] } {
		set errorRes "One positive integer number must be entered"
	    }   
	}
	int0+ {
	    if { [regexp  {^[  ]*[+-]?[0-9]+[  ]*$} \
		    $word] } {
		if { $word < 0 } {
		    set errorRes "One non negative integer number must be entered"
		}
	    } else {
		set errorRes "One non negative integer number must be entered"
	    }
	}
    }
    if { $errorRes!="" } {
	if { $errorText!="" } {
	    tk_dialogRAM $w.tempwinerr {Error window} "$errorRes $errorText" \
		    error 0 OK
	}
	return "$errorRes"
    } 
    return ""
}

#
# This procedure displays a dialog box, with an entry, waits for a button in 
# the dialog to be invoked, then returns the entry.
#
# Arguments:
# w -                Window to use for dialog top-level.
# title -        Title to display in dialog's decorative frame.
# text -        Message to display in dialog.
# bitmap -        Bitmap to display in dialog (empty string means none).
# type -        Can be any of IsType types and:
#                        text:    text with spaces and '\n'
#                        textro:  text with spaces and '\n' read-only
# default -    default value for entry

proc tk_dialogEntryType {w title text bitmap type default } {
    global tkPriv tcl_platform

    # 1. Create the top-level window and divide it into top, middle
    # and bottom parts.

    while { [winfo exists $w] } { append w a }
    catch {destroy $w}
    toplevel $w -class OverlayWindow

    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW { }

    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    frame $w.bot -relief raised -bd 1
    pack $w.bot -side bottom -fill both
    frame $w.top -relief raised -bd 1

    if { $type != "text" && $type != "textro"} {
	pack $w.top -side top -fill both -expand 1
    } else {
	pack $w.top -side top -fill both
    }

    frame $w.middle -relief raised -bd 1
    if { $type != "text" && $type != "textro"} {
	entry $w.middle.e -relief sunken -xscroll "$w.middle.xscroll set" \
		-textvariable this_entry -bd 1
	if { $type == "password" } { $w.middle.e conf -show * }

	$w.middle.e del 0 end
	if { $default != "" } {
	    #kike,correccin si $default == "", no hara caso a la tecla retroceso
	      $w.middle.e ins end $default
	     $w.middle.e sel from 0 
	    $w.middle.e sel to end
	}

	scrollbar $w.middle.xscroll -relief sunken \
		-orient horizontal -command "$w.middle.e xview" \
		-bd 1 -elementborderwidth 1
#     pack $w.middle.xscroll -side bottom -fill x
#     pack $w.middle.e -side top

	grid $w.middle.e -sticky ew
	grid $w.middle.xscroll -sticky ew

	pack $w.middle

    } else {
	set this_entry ""

	text $w.middle.t -relief sunken -width 40 -height 8 \
		-yscroll "$w.middle.yscroll set" -bd 1
	$w.middle.t delete 0.0 end
	$w.middle.t ins end $default
	#$w.middle.t tag add sel 0.0 end
	if { $type == "textro"} {
	    $w.middle.t conf -state disabled
	}

	scrollbar $w.middle.yscroll -relief sunken \
		-orient vertical -command "$w.middle.t yview" \
		-bd 1 -elementborderwidth 1

	# grid $w.middle.t $w.middle.yscroll -sticky ewns -ipady 3
	# 
	# grid rowconfigure $w.middle 0 -weight 1
	# grid rowconfigure $w.middle 1 -weight 1
	# grid columnconfigure $w.middle 0 -weight 1
	# grid columnconfigure $w.middle 1 -weight 0
	pack $w.middle.t -side left -expand 1 -fill both
	pack $w.middle.yscroll -expand 1 -fill y

	pack $w.middle -expand 1 -fill both
    }

    # 2. Fill the top part with bitmap and message (use the option
    # database for -wraplength so that it can be overridden by
    # the caller).

#    option add *Dialog.msg.wrapLength 3i widgetDefault
    label $w.msg -justify left -text $text -wraplength 75m -font BigFont

    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$bitmap != ""} {
	label $w.bitmap -bitmap $bitmap
	pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }

    if { $type != "text" && $type != "textro"} {
	bind $w <Return> "
	  $w.button0 configure -state active -relief sunken
	  update idletasks
	  after 100
	  set tkPriv(button) 0
	"
    }

    # 3. Create a row of buttons at the bottom of the dialog.

    set UnderLineList ""
    set i 0
    set default 0
#    set args "OK Cancel" 
    set args "OK"
    set lentext 0
    foreach but $args {
	if { [string length $but] > $lentext } {
	    set lentext [string length $but]
	}
    }
   foreach but $args {
	button $w.button$i -text $but -command "set tkPriv(button) $i" \
		-width $lentext -bd 1
	pack $w.button$i -in $w.bot -side left -expand 1 \
		-padx 3m -pady 2m

	bind $w.button$i <Return> "
	$w.button$i configure -state active -relief sunken
	update idletasks
	after 100
	set tkPriv(button) $i
	break
	"

	if { [regexp -nocase cancel $but] } "
	    bind $w <Escape> \"
	    $w.button$i configure -state active -relief sunken
	    update idletasks
	    after 100
	    set tkPriv(button) $i
	\""
	incr i
    }

    # 5. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

#    if { [winfo parent $w] == ".gid" } {
#        bind $w <Destroy> [list DestroyWinInDialogs $w GeomDialog]
#    }

#    if { [info exists GidPriv(GeomDialog)] && [winfo parent $w] == ".gid"} {
#        WmGidGeom  $w $GidPriv(GeomDialog)
#    } else {
	wm withdraw $w
	update idletasks
	
	# extrange errors with updates
	if { ![winfo exists $w] } { return 0}
	
	set pare [winfo toplevel [winfo parent $w]]
	set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
	set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
	if { $x < 0 } { set x 0 }
	if { $y < 0 } { set y 0 }
	WmGidGeom $w +$x+$y
	update
	wm deiconify $w
 #   }

    # 6. Set a grab and claim the focus too.

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w
    
    if { $type != "text" && $type != "textro" } {
	focus $w.middle.e
	after 100 catch [list "focus -force $w.middle.e"]
    } elseif { $type == "textro" } {
	focus $w.button0
	after 100 catch [list "focus -force $w.button0"]
    } else {
	focus $w.middle.t 
	after 100 catch [list "focus -force $w.middle.t"]
    }
 
    # 7. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.

    set errorRes ""
    while 1 {
	tkwait variable tkPriv(button)
#        if { $tkPriv(button) == 1} { break }
	if { $type != "text" && $type != "textro" } {
	    set result [$w.middle.e get]
	} else {
	    set result [ $w.middle.t get 0.0 end]
	}
	set errorRes [IsType $type $result .gid "!!!"]
	set this_entry "$result"
	if { $errorRes == "" } { break }
    }

    if { $oldFocus == "" || [catch {focus $oldFocus}] } {
	if { [winfo exists .gid] } {
	    focus [focus -lastfor .gid]
	}
    }
    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    grab -global $oldGrab
	} else {
	    grab $oldGrab
	}
    }
    return $this_entry
}

# proc tk_ListBox
#
# wn: parent window
# title: title of the listbox
# subtitle: lateral text of the listbox
# names: names of entries of the list
# command: to be executed when ready

proc tk_ListBox { wn title subtitle names { command "" } } { 

    set w $wn.lsbox
    while { [winfo exists $w] } { append w a }

    toplevel $w -height 200 -width 200
    wm title $w $title
#    wm iconname $w Dialog        
    wm transient $w $wn

    set pare [winfo toplevel [winfo parent $w]]
    set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
    set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    wm geom $w +$x+$y

#   Primera fila del grid, compuesta por el subttulo y la listbox 
    frame $w.main -bd 2 -relief groove
    label $w.main.msg -wraplength 1.5i -justify left -text $subtitle -font Normal -padx 10
    grid $w.main.msg -row 0 -column 0 -sticky nsew -pady 3

    frame $w.main.frame -bd .1c
    grid $w.main.frame -row 0 -column 1 -sticky nsew -pady 1

    scrollbar $w.main.frame.scroll -command "$w.main.frame.list yview"
    listbox $w.main.frame.list -yscroll "$w.main.frame.scroll set" -selectmode single \
	    -setgrid 1 -height 12 -bg white
    grid $w.main.frame.scroll -row 0 -column 1 -sticky nse
    grid $w.main.frame.list -row 0 -column 0 -sticky nsew
    grid columnconfigure $w.main.frame 0 -weight 1
    grid rowconfigure $w.main.frame 0 -weight 1
    grid $w.main -row 0 -column 0 -sticky nsew
    grid columnconfigure $w.main 0 -weight 1
    grid rowconfigure $w.main 0 -weight 1

    proc tk_ListBox_thisprocess { w list command } {
	eval $command [$list curselection]
	destroy $w
    } 

    bind $w.main.frame.list <Return> \
	    "tk_ListBox_thisprocess $w $w.main.frame.list \"$command\""
    bind $w.main.frame.list <Double-1> \
	    "tk_ListBox_thisprocess $w $w.main.frame.list \"$command\""

    eval $w.main.frame.list insert 0 $names

#   Segunda fila del grid, compuesta por los botones
    frame $w.buttons
    grid $w.buttons -row 1 -column 0 -sticky nsew -padx 3 -pady 1
    button $w.buttons.ok -text OK -width 5 -command \
	    "tk_ListBox_thisprocess $w $w.main.frame.list \"$command\""
    button $w.buttons.cancel -text Cancel -width 5 -command "destroy $w"
    grid $w.buttons.ok -row 0 -column 0 -sticky s -padx 3 -pady 1
    grid $w.buttons.cancel -row 0 -column 1 -sticky s -padx 3 -pady 1  
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    update
}

# proc tk_ListMenu
#
# wn: parent window
# text: text of the buttonmenu
# names: names of commands of the menu
# command: list of commands of the menu

proc tk_ListMenu { wn text names { command "" } } { 
    set w $wn.lsmenu
    menubutton $w -text $text -underline 0 -direction below -menu $w.ls -relief raised
    menu $w.ls -tearoff 0
    foreach name $names {
	if { $name != "" } {
	    set nmtext $name
	    if { [string length $nmtext]>10 } {
		set nmtext [string range $name 0 9]
	    }
	    $w.ls add command -label $nmtext -command "$command \"$name\""
	}
    }
    return $w
}
