namespace eval unitdatwin {

    global ProgramName
    global tkPriv tcl_platform

    # Tdyn settings
    variable show_fluid [IsVarOnTdynDefaults show_fluid 1]
    variable show_solid [IsVarOnTdynDefaults show_solid 1]
    variable show_ursolver [IsVarOnTdynDefaults show_ursolver 1]
    variable show_alemesh [IsVarOnTdynDefaults show_alemesh 1]
    variable show_ransol [IsVarOnTdynDefaults show_ransol 1]
    variable show_heatrans [IsVarOnTdynDefaults show_heatrans 1]
    variable show_advect [IsVarOnTdynDefaults show_advect 1]
    variable show_naval [IsVarOnTdynDefaults show_naval 1]

    variable Aux_Length_Units [list m dm cm mm um pm in pica ft yd A Nmi]
    variable Aux_Mass_Units [list Kg g ton lb]
    variable Aux_Time_Units [list s ds cs ms min hoy day]
    variable Aux_Velocity_Units [list m/s cm/s mm/s Km/s knot mach]
    variable Aux_Pressure_Units [list Pa mmHg atm bar KPa]
    variable Aux_KEnergy_Units [list m\u00b2/s\u00b2 dm\u00b2/s\u00b2 mm\u00b2/s\u00b2 \
	    m\u00b2/cs\u00b2 m\u00b2/ms\u00b2]
    variable Aux_Temperature_Units [list Gt dGt cGt]

    # IN/OUT Units
   
    variable InOut_UnitsData [list \
	    [list Output_Length_Units "Output Length Units" string 0 Aux_Length_Units] \
	    [list Output_Mass_Units "Output Mass Units" string 0 Aux_Mass_Units] \
	    [list Output_Time_Units "Output Time Units" string 0 Aux_Time_Units] \
	    [list Internal_Length_Units "Internal Length Units" string 0 Aux_Length_Units] \
	    [list Internal_Mass_Units "Internal Mass Units" string 0 Aux_Mass_Units] \
	    [list Internal_Time_Units "Internal Time Units" string 0 Aux_Time_Units] \
	    ]
    foreach item $InOut_UnitsData {
	variable [lindex $item 0]
    }

    # Geometry Units
    variable Geometry_UnitsData [list \
	    [list Geometry_Units "Geometry Units" string 0 Aux_Length_Units] \
	    ]
    foreach item $Geometry_UnitsData {
	variable [lindex $item 0]
    }

    # General Units
    variable General_UnitsData [list \
	    [list Boundary_Length_Units "Boundary Length Units" string 0 Aux_Length_Units All] \
	    [list Boundary_Mass_Units "Boundary Mass Units" string 0 Aux_Mass_Units All] \
	    [list Boundary_Time_Units "Boundary Time Units" string 0 Aux_Time_Units All] \
	    [list Boundary_Velocity_Units "Boundary Velocity Units" string 0 Aux_Velocity_Units Ransol] \
	    [list Boundary_Pressure_Units "Boundary Pressure Units" string 0 Aux_Pressure_Units Ransol] \
	    [list Boundary_Kenr_Units "Boundary Kenr Units" string 0 Aux_KEnergy_Units Ransol] \
	    [list Boundary_ELen_Units "Boundary ELen Units" string 0 Aux_Length_Units Ransol] \
	    [list Boundary_Temperature_Units "Boundary Temperature Units" string 0 Aux_Temperature_Units Heatrans] \
	    ]
    foreach item $General_UnitsData {
	variable [lindex $item 0]
    }

    # Function Editor variables
    variable fluid_variables "x y z nx ny nz t it rand dt exp(1) pi vx vy vz pr tm\
	    vt v_ tau wc ds ke ep om kt pk dn bt sp#"
    variable example_fluid_variables "x y z nx ny nz t it rand dt exp(1)\
	    pi vx vy vz pr tm vt v_ tau wc ds ke ep om kt pk dn bt sp#"
    variable fluid_operators "+ - * / ^ mod div rmod rdiv max min Dx() Dy() Dz()\
	    ~ < <= > >= = != & |"
    variable predefined_ff [list $fluid_variables $example_fluid_variables $fluid_operators]
}

proc unitdatwin::init { parent {show "all"} {tags ""} } {

    global ProgramName
    global tkPriv tcl_platform

    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_naval
    variable show_alemesh

    set unw $parent.unw

    if { [TUI_closeotherwindows $unw 1] == 1 } {

	InitWindow $unw "Tdyn Units Data" PreTdynUnitsDataWindowGeom ""

	set closecmd "unitdatwin::checkclose $unw 1 1 1"
	set acceptcmd "unitdatwin::checkclose $unw 1 0 0"
	wm protocol $unw WM_DELETE_WINDOW "$closecmd"

	# Reading values from Problem Data
	unitdatwin::getvalues

	# Top frame
	set topf [frame $unw.f -bg lightblue]
	# Picture Box
	set l_icon [label $topf.l_icon -image [imagefromtdyn units.gif] -bg lightblue]
	grid $topf -row 0 -column 0 -sticky nwe
	grid $l_icon -row 0 -column 0 -sticky nw
	grid columnconf $topf 1 -weight 1

	# Main Notebook
	set pages [NoteBook $unw.pages -homogeneous 1 -arcradius 0]
	grid $pages -row 1 -column 0 -sticky nwes -pady 5

	grid columnconf $unw 0 -weight 1
	grid rowconf $unw 1 -weight 1

	set pnum 0
	set firstpage ""

	# In/Out Page
	incr pnum
	set pvar "pg_inout"
	if { [string length $firstpage] < 1 } { 
	    set firstpage $pvar
	}
	set fpvar "fpg_inout"
	set $pvar [$pages insert $pnum $pvar -text "IN/OUT"]
	set $fpvar [$pages getframe $pvar]
	unitdatwin::inout_data $pg_inout $parent
	$pages raise $pvar

	# Geometry Page
	incr pnum
	set pvar "pg_geometry"
	if { [string length $firstpage] < 1 } { 
	    set firstpage $pvar
	}
	set fpvar "fpg_geometry"
	set $pvar [$pages insert $pnum $pvar -text "GEOMETRY"]
	set $fpvar [$pages getframe $pvar]
	unitdatwin::geometry_data $pg_geometry $parent
	$pages raise $pvar

	# General Page
	incr pnum
	set pvar "pg_general"
	if { [string length $firstpage] < 1 } { 
	    set firstpage $pvar
	}
	set fpvar "fpg_general"
	set $pvar [$pages insert $pnum $pvar -text "GENERAL"]
	set $fpvar [$pages getframe $pvar]
	unitdatwin::general_data $pg_general $parent
	$pages raise $pvar

	# Buttons frame
	set bf [frame $unw.bf -relief raised -takefocus 1]
	# Accept button
	set btn_accept [button $bf.btn_accept -text "Accept Data" -underline 0 -relief raised \
		-bd 1 -highlightthickness 1 -bg lightblue -takefocus 1 \
		-command "$acceptcmd"]
	# Close button
	set btn_close [button $bf.btn_close -text "Close" -underline 0 -relief raised \
		-bd 1 -highlightthickness 1 -bg lightblue -takefocus 1 \
		-command "$closecmd"]

	# GRIDS
	grid $bf -row 2 -column 0 -sticky ne
	grid $btn_accept -row 0 -column 0 -sticky e -pady 1
	grid $btn_close -row 0 -column 1 -sticky e -pady 1

	# BINDS
	bind $unw <Escape> "tkButtonInvoke $btn_close"
	bind $unw <Alt-c> "tkButtonInvoke $btn_close"
	bind $unw <Alt-a> "tkButtonInvoke $btn_accept"

	$pages compute_size

	focus $unw

    }
}

################################################################################
#
#   In/Out
#
################################################################################

proc unitdatwin::inout_data { page tags } {
 
    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_alemesh
    variable show_naval

    variable InOut_UnitsData
    foreach item $InOut_UnitsData {
	variable [lindex $item 0]
    }

    # Title Frame
    set tframe [TitleFrame $page.f -relief ridge -text "Input/Output units"]
    set frame [$tframe getframe]
    set irow 0 
    
    foreach item $InOut_UnitsData {
	set name [lindex $item 0]
	set l_$name [Label $frame.l_$name  -text "[lindex $item 1] "]
	set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		-textvariable unitdatwin::$name -editable 1 -width \
		[TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
	grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
	grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
	incr irow
	if { [info exists help_$name] } {
	    GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
	}
    }
    grid $tframe -row 0 -column 0 -sticky ewn
    grid columnconfigure $page 0 -weight 1
    grid rowconfigure $page 2 -weight 1
    GidHelpRecursive $tframe "Select Input/Output units" 
}

################################################################################
#
#   Geometry
#
################################################################################

proc unitdatwin::geometry_data { page tags } {
 
    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_alemesh
    variable show_naval

    variable Geometry_UnitsData
    foreach item $Geometry_UnitsData {
	variable [lindex $item 0]
    }

    # Title Frame
    set tframe [TitleFrame $page.f -relief ridge -text "Geometry units"]
    set frame [$tframe getframe]
    set irow 0 
    
    foreach item $Geometry_UnitsData {
	set name [lindex $item 0]
	set l_$name [Label $frame.l_$name  -text "[lindex $item 1] "]
	set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		-textvariable unitdatwin::$name -editable 1 -width \
		[TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
	grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
	grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
	incr irow
	if { [info exists help_$name] } {
	    GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
	}
    }
    grid $tframe -row 0 -column 0 -sticky ewn
    grid columnconfigure $page 0 -weight 1
    grid rowconfigure $page 2 -weight 1
    GidHelpRecursive $tframe "Select the units for the geometry of the analysis" 
}

################################################################################
#
#   General
#
################################################################################

proc unitdatwin::general_data { page tags } {
 
    # Tdyn settings
    variable show_fluid
    variable show_solid
    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver
    variable show_alemesh
    variable show_naval

    variable General_UnitsData
    foreach item $General_UnitsData {
	variable [lindex $item 0]
    }

    # Title Frame
    set tframe [TitleFrame $page.f -relief ridge -text "General units"]
    set frame [$tframe getframe]
    set irow 0 
    
    foreach item $General_UnitsData {
	if { ($show_ransol && [lindex $item 5]=="Ransol") || \
	     ($show_heatrans && [lindex $item 5]=="Heatrans") || [lindex $item 5]=="All" } {
	    set name [lindex $item 0]
	    set l_$name [Label $frame.l_$name  -text "[lindex $item 1] "]
	    set e_$name [ComboBox $frame.e_$name -values [eval set temp \$unitdatwin::[lindex $item 4]] \
		    -textvariable unitdatwin::$name -editable 1 -width \
		    [TDYN_getcbxsize 5 [eval set temp \$unitdatwin::[lindex $item 4]]]]
	    grid [eval set temp \$l_$name] -row $irow -column 0 -sticky en -pady 2
	    grid [eval set temp \$e_$name] -row $irow -column 1 -sticky wn -pady 2
	    incr irow
	    if { [info exists help_$name] } {
		GidHelp [eval set temp \$l_$name] [eval set temp \$help_$name]
	    }
	}
    }
    grid $tframe -row 0 -column 0 -sticky ewn
    grid columnconfigure $page 0 -weight 1
    grid rowconfigure $page 2 -weight 1
    GidHelpRecursive $tframe "Select the units used in the definition of initial and boundary conditions" 
}

################################################################################
#
#   Get Values
#
################################################################################

proc unitdatwin::getvalues { } {

    set unilist [list InOut_UnitsData Geometry_UnitsData General_UnitsData]
    set gendata [.central.s info gendata]

    # Reading units
    for {set i 0} {$i < [llength $unilist]} {incr i} {
	set thisvar [lindex $unilist $i]
	variable $thisvar
	foreach item [eval set nothing $$thisvar] {
	    variable [lindex $item 0]
	}
	foreach item [eval set nothing $$thisvar] {
	    set itt [lindex $item 0]
	    set nitt [lindex $item 1]
	    set $itt [TUI_formatunits Nums2Unicode \
		    [DWUnder2Space [join [lindex $gendata [expr [lsearch -regexp $gendata \
		                        [eval set nothing ^${itt}(\$|\#CB)]] + 1]]]] 0]
	}
    }
}

proc unitdatwin::checkclose { win {dumpit 1} {askdump 1} {close 1} } {

    global tkPriv tcl_platform

    variable show_ransol
    variable show_heatrans
    variable show_advect
    variable show_ursolver

    # Checking info 
    set valueschange 0
    set wtsave 0
    set usr_err ""
    set num_usr_err 0
    set unilist [list InOut_UnitsData Geometry_UnitsData General_UnitsData]
    set paglist [list {IN/OUT} GEOMETRY GENERAL]
    
    set gendata [.central.s info gendata]

    # Checking units
    for {set i 0} {$i < [llength $unilist]} {incr i} {
	set thisuni [lindex $unilist $i]
	set thispag [lindex $paglist $i]
	if { $thisuni != "" } {
	    variable $thisuni
	    foreach item [eval set nothing $$thisuni] {
		variable [lindex $item 0]
	    }
	    foreach item [eval set nothing $$thisuni] {
		set itt [lindex $item 0]
		set jtt temp_$itt
		set $jtt [TUI_formatunits Unicode2Nums [eval set temp $$itt] 1]
		set temp [join [lindex $gendata [expr [lsearch -regexp $gendata \
		        [eval set nothing ^${itt}(\$|\#CB)]] + 1]]]
		if { [DWUnder2Space $temp] != [eval set nothing $$jtt] } {
		    set valueschange 1
		    if { [lindex $item 3] == 1} {
		        set errcheck [TUI_valuecheck [DWSpace2Under [eval set nothing $$jtt]] \
		                [lindex $item 2] 0 ""]
		        if { $errcheck != "ok"} {
		            incr num_usr_err
		            if { [eval set temp \$uerrpg_$thispag] != 1 } {
		                append usr_err "\n$thispag PAGE\n"
		            set uerrpg_$thispag 1
		            }
		            append usr_err "$num_usr_err. " [DWUnder2Space $itt] ": $errcheck\n"
		        }
		    }
		}
	    }
	}
    }

    if { $valueschange == 1 } {
	if { $askdump == 0 } {
	    set answer 0
	} else {
	    set answer [tk_dialogRAM $win "Tdyn Units Data " \
		    "Units data has changed.\nDo you want to save changes?" question 0 Yes No]
	}

	if { $answer == 0 && $dumpit == 1} {
	    set wtsave 1
	    if { $num_usr_err > 0 } {
		if { $num_usr_err > 1 } {
		    set err_display "The following $num_usr_err error"
		    append err_display "(s) were found in the data input:\n"
		} else {
		    set err_display "The following error was found in the data input:\n"
		}
		append err_display $usr_err
		WarnWin $err_display
		set close 0
	    } else {
		unitdatwin::dump
		FloatMessage $win.flm "Units Data were saved."
	    }
	}
    } else {
	if { $dumpit == 1 } {
	    FloatMessage $win.flm "No changes need to be saved."
	}
    }

    if { $close == 1 } {
	destroy $win
	return $valueschange
    } else {
	if { $num_usr_err == 0 } {
	    return $valueschange
	} else {
	    if { $wtsave == 1 } {
		return "dataerror"
	    } else {
		return "nosave.dataerror"
	    }
	}
    }
}

proc unitdatwin::dump { } {

    variable InOut_UnitsData
    variable Geometry_UnitsData
    variable General_UnitsData

    TUI_dumpsingleunits unitdatwin $InOut_UnitsData ProblemData
    TUI_dumpsingleunits unitdatwin $Geometry_UnitsData ProblemData
    TUI_dumpsingleunits unitdatwin $General_UnitsData ProblemData

    return 1
}
