# tdynwizard.tcl  -*- TCL -*- Created: JGE Jan-2002, Modified: JGE Feb-2002

# Los siguientes procedimientos estn pensados para la existencia de las
# siguentes capas:
#  body_wet    cuerpo en el fluido
#  body_dry    parte del cuerpo fuera del fluido
#  free_surf   superficie libre
#  symm_oy     plano de simetra perpendicular a Oy
#  volume      volumen de control
#  inlet_ox    entrada de fluido
#  outlet_ox   salida de fluido
#  lateral_oy  lateral perpendicular a Oy
#  lateral_oz  lateral perpendicular a Oz

proc OpenTdynWizardWindow { } {
    global TdynWizPriv ProblemTypePriv
 
    # create the wizard widget
    set wizwin .gid.tdynwizard
    if { [winfo exists $wizwin] } {
	$wizwin show
	return
    }
    catch { unset TdynWizPriv }

    set dir $ProblemTypePriv(problemtypedir)
    tkwizard::tkwizard $wizwin -icondir "$dir/images/tdynwiz.gif" -title "Tdyn Wizard" 
 
    GidHelp $wizwin  "This wizard can be used to automatically generate"\
	    "the geometry and data for simple problems.\nThis kind of problems must"\
	    "consist on one or several bodies within a control volume, symmetric or"\
	    "unsymmetric and with or without free surface.\nThe problem must be"\
	    "oriented in the main cartesian axes, being OX the direction of the"\
	    " free-flow and OZ+ the vertical direction. If any symmetry plane exits,"\
	    "this must be perpendicular to the OY axis."

    # define finish & cancel button action
#    bind $wizwin <<WizFinish>> {StartCalculate}
    bind $wizwin <<WizCancel>> {DestroyWindows}

    # define wizard steps
    $wizwin step "Introduction" -title "Introduction" -layout basic { 
	global TdynWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 1} \
		-subtitle {Introduction and how to create the geometry using the standard \
		layers}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_001.html"] $w 
	DestroyWindows
	CreateLayer "body_wet body_dry free_surf symm_oy"
	ChangeLayers
    }
    $wizwin step "CreateCompBox" -title "Create control volume" -layout basic { 
	global TdynWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 2} \
		-subtitle {Use Create control volume window to define and create the control\
		volume for the analysis}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_002.html"] $w 
	DestroyWindows
	UseCreateCompBox
    }
    $wizwin step "DefineProblemData" -title "Define Data" -layout basic { 
	global TdynWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 3} \
		-subtitle {Use Define data window to define boundary conditions, boundaries,\
		material properties and other data of the problem}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_003.html"] $w 
	DestroyWindows
	UseCreateDefData
    }
    $wizwin step "GenerateMesh" -title "Generate mesh" -layout basic { 
	global TdynWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 4} \
		-subtitle {Use the Generate mesh window to define the element sizes and generate \
		the mesh}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_004.html"] $w 
	DestroyWindows
	UseGenerateMesh
    }
    $wizwin step "Calculate" -title "Calculate" -layout basic { 
	global TdynWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 5} \
		-subtitle {Congratulations! The problem is ready to be solved}
	if { [.central.s info postprocess arewein]!="NO" } {
	    .central.s process escape escape escape escape preprocess yes escape
	}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_005.html"] $w 
	DestroyWindows
	RunWin
    }
    $wizwin step "Postprocessing" -title "Postprocessing" -layout basic { 
	global TdynWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 6} \
		-subtitle {Now, we will start the postprocessing process}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_006.html"] $w 
	if { [.central.s info postprocess arewein]!="YES" } {
	    .central.s process escape escape escape escape postprocess escape
	}
	DestroyWindows
	PostDisplay
	PostResults
    }
    $wizwin step "MakingGraphs" -title "Making Graphs" -layout basic { 
	global TdynWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 7} \
		-subtitle {Making graphs}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_007.html"] $w 
	DestroyWindows
	PostDisplay
	PostGraphResultsWindow
    }
    $wizwin step "Reporting" -title "Reporting" -layout basic { 
	global TdynWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 8} \
		-subtitle {Creating a report of the analysis}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_008.html"] $w 
	DestroyWindows
	UseCreateReportWindow
    }
    # show the wizard
    $wizwin show
}
 
proc DestroyWindows {} {
    if { [.central.s info postprocess arewein]!="YES" } {
	if { [winfo exists .gid.tdynwizard_createbox] } {
	    destroy .gid.tdynwizard_createbox 
	}
	if { [winfo exists .gid.tdynwizard_defdata] } {
	    destroy .gid.tdynwizard_defdata
	}
	if { [winfo exists .gid.tdynwizard_assignsizes] } {
	    destroy .gid.tdynwizard_assignsizes
	} 
	if { [winfo exists .gid.wlay] } {
	    destroy .gid.wlay
	}
	if { [winfo exists .wrun] } {
	    destroy .wrun
	}
    } else {
	if { [winfo exists .wPostDisplay] } {
	    destroy .wPostDisplay
	}
	if { [winfo exists .wPostResults] } {
	    destroy .wPostResults
	}
	if { [winfo exists .gid.createreport] } {
	    destroy .gid.createreport
	}
	if { [winfo exists .gid.graph] } {
	    destroy .gid.graph
	}
    }
}

proc DoTdynWizard {} {
    set ProjectInfo [.central.s info Project]
    set ProjectName [lindex $ProjectInfo 1]
    if { $ProjectName == "UNNAMED" } {          
	tk_dialogRAM .gid.tmpwin {Error window} \
		"Before calculating, a project title is needed. Save project to get it" \
		  error 0 OK 
	return
    } else {
	[.central.s process escape escape escape escape utilities calculate escape]
    }
#   if { [DoCreateCompBox] } { 
#        if { ![DoGenerateMesh] } { 
#            return 
#        }
#    }
#    tk_dialogRAM $w.tempwin {Error window} "Error found trying to define the problem" \
#            error 0 OK
}

############################################
# CREACIN AUTOMTICA DEL VOLUMEN DE CONTROL
############################################
proc CreateCompBoxWindow { w } {
    global TdynWizPriv ProblemTypePriv

    calcLength

    set TdynWizPriv(oxfac) [expr $TdynWizPriv(oxafac)+$TdynWizPriv(oxffac)]
    set TdynWizPriv(oxlen) [expr $TdynWizPriv(length)*$TdynWizPriv(oxfac)]
    set TdynWizPriv(oxres) " x Length = $TdynWizPriv(oxlen)"
    set TdynWizPriv(oylen) [expr $TdynWizPriv(length)*$TdynWizPriv(oyfac)]
    set TdynWizPriv(oyres) " x Length = $TdynWizPriv(oylen)" 
    set TdynWizPriv(ozlen) [expr $TdynWizPriv(length)*$TdynWizPriv(ozfac)]
    set TdynWizPriv(ozres) " x Length = $TdynWizPriv(ozlen)"

    set dir $ProblemTypePriv(problemtypedir)

#    set cancompbox [image create photo \
#            -file [file join $dir images airplane.gif]]

    canvas $w.dib -relief groove -bd 2 -width 250 -height 150
#    $w.dib create image -30 -40 -image $cancompbox -anchor nw

#    $w.dib create bitmap -30 -40 -bitmap @$dir/images/clipplane.xbm \
#            -anchor nw

    ActualizeCreateCompBoxCanvas $w

    GidHelp "$w.dib" "This tool can be used to create a control volume around a body(-ies)."\
	    " The size of the control volume can be defined by the scales. In order"\
	    "to use this tool, the geometry must be separated in different layers:\n"\
	    "1.-Layer body_wet with the surfaces of the body(-ies) within the fluid\n"\
	    "2.-Layer body_dry with the part of the body out of the fluid (if any)\n"\
	    "3.-Layer free_surf with the lines of the body (if any) belonging to the"\
	    "free surface (floating line)\n4.-Layer called symm_oy with the lines of"\
	    "the body (if any) belonging to the symmetry plane and below the floating"\
	    "line (must be perpendicular to the OY axis)."

    proc setox { w } {
	global TdynWizPriv
	setOXLength $w $TdynWizPriv(oxfac)
    }
    frame $w.fx -relief ridge -bd 2
    frame $w.fx.up
    label $w.fx.up.l -text "X factor:"
    entry $w.fx.up.e -relief sunken -textvariable TdynWizPriv(oxfac) -width 5
    bind  $w.fx.up.e <Return> "setox $w"
    label $w.fx.up.o -textvariable TdynWizPriv(oxres) -width 15 -padx 1

    grid $w.fx.up.l $w.fx.up.e $w.fx.up.o -sticky w
    grid conf $w.fx.up.o -sticky ew
    grid columnconf $w.fx.up 2 -weight 1
    grid conf $w.fx.up -sticky ew

    frame $w.fx.dw -relief solid -bd 2
    label $w.fx.dw.l -text "Fore"
    scale $w.fx.dw.s1 -orient h -from 0.1 -to 10.0 \
	    -resolution 0.1 -command "setOXFLength $w"
    scale $w.fx.dw.s2 -orient h -from 0.1 -to 10.0 \
	    -resolution 0.1 -command "setOXALength $w"
    label $w.fx.dw.r -text "Aft"  

    GidHelp "$w.fx" "Define the OX dimension of the control volume."

    grid $w.fx.dw.l $w.fx.dw.s1 $w.fx.dw.s2 $w.fx.dw.r -sticky w
    grid conf $w.fx.dw.s1 $w.fx.dw.s2 -sticky ew
    grid columnconf $w.fx.dw 1 -weight 1 
    grid columnconf $w.fx.dw 2 -weight 1
    grid conf $w.fx.dw -sticky ew -padx 2 -pady 2
    grid columnconf $w.fx 0 -weight 1 

    $w.fx.dw.s1 set $TdynWizPriv(oxffac)
    $w.fx.dw.s2 set $TdynWizPriv(oxafac)

    proc setoy { w } {
	global TdynWizPriv
	setOYLength $w $TdynWizPriv(oyfac)
    }
    frame $w.fy -relief ridge -bd 2
    label $w.fy.l -text "Y factor:"
    entry $w.fy.e -relief sunken -textvariable TdynWizPriv(oyfac) -width 5
    bind  $w.fy.e <Return> "setoy $w"
    label $w.fy.o -textvariable TdynWizPriv(oyres) -width 15 -padx 1
    scale $w.fy.s -relief solid -bd 2 -orient h -from 0.1 -to 10.0 \
	    -resolution 0.1 -command "setOYLength $w"

    GidHelp "$w.fy" "Define the OY dimension of the control volume."

    $w.fy.s set $TdynWizPriv(oyfac)
    grid $w.fy.l -sticky w -row 0 -column 0
    grid $w.fy.e -sticky w -row 0 -column 1
    grid $w.fy.o -sticky ew -row 0 -column 2
    grid $w.fy.s -sticky ew -row 1 -column 0 -columnspan 3 -padx 1 -pady 1
    grid columnconfigure $w.fy 1 -weight 1
    proc setoz { w } {
	global TdynWizPriv
	setOZLength $w $TdynWizPriv(ozfac)
    }
    frame $w.fz -relief ridge -bd 2
    label $w.fz.l -text "Z factor:"
    entry $w.fz.e -relief sunken -textvariable TdynWizPriv(ozfac) -width 5
    bind  $w.fz.e <Return> "setoz $w"
    label $w.fz.o -textvariable TdynWizPriv(ozres) -width 15 -padx 1
    scale $w.fz.s  -relief solid -bd 2 -orient h -from 0.1 -to 10.0 \
	    -resolution 0.1 -command "setOZLength $w"

    GidHelp "$w.fz" "Define the OZ dimension of the control volume."

    $w.fz.s set $TdynWizPriv(ozfac)
    grid $w.fz.l -sticky w -row 0 -column 0
    grid $w.fz.e -sticky w -row 0 -column 1
    grid $w.fz.o -sticky ew -row 0 -column 2
    grid $w.fz.s -sticky ew -row 1 -column 0 -columnspan 3 -padx 1 -pady 1
    grid columnconfigure $w.fz 1 -weight 1

    frame $w.pk1 -relief ridge -bd 2
    label $w.pk1.l -text "Reference fore point:"
    button $w.pk1.b -text Pick -command "SetCompBoxPick $w 0"
    label $w.pk1.lx -text x:
    entry $w.pk1.ex -relief sunken -textvariable TdynWizPriv(xforpnt) -width 4
    label $w.pk1.ly -text y:
    entry $w.pk1.ey -relief sunken -textvariable TdynWizPriv(yforpnt) -width 4
    label $w.pk1.lz -text z:
    entry $w.pk1.ez -relief sunken -textvariable TdynWizPriv(zforpnt) -width 4
    bind  $w.pk1.ex <Return> " calcLength $w " 
    bind  $w.pk1.ey <Return> " calcLength $w "
    bind  $w.pk1.ez <Return> " calcLength $w "
    
    GidHelp "$w.pk1" "Insert the coordinates of the fore extreme (upstream)"\
	    "of the body (if exits it must be  on the free surface) or pick"\
	    "the point.\nIn order this tool to work properly, geometry must"\
	    "be separated in the standard layers."
    
    grid $w.pk1.l -sticky w -row 0 -column 0 -columnspan 4
    grid $w.pk1.b -sticky e -row 0 -column 4 -columnspan 2
    grid $w.pk1.lx  -sticky w -row 1 -column 0
    grid $w.pk1.ex  -sticky ew -row 1 -column 1
    grid $w.pk1.ly  -sticky w -row 1 -column 2
    grid $w.pk1.ey  -sticky ew -row 1 -column 3
    grid $w.pk1.lz  -sticky w -row 1 -column 4
    grid $w.pk1.ez  -sticky ew -row 1 -column 5
    grid columnconfigure $w.pk1 1 -weight 1
    grid columnconfigure $w.pk1 3 -weight 1
    grid columnconfigure $w.pk1 5 -weight 1

    frame $w.pk2 -relief ridge -bd 2
    label $w.pk2.l -text "Reference aft point:"
    button $w.pk2.b -text Pick -command "SetCompBoxPick $w 1"
    label $w.pk2.lx -text x:
    entry $w.pk2.ex -relief sunken -textvariable TdynWizPriv(xaftpnt) -width 4
    label $w.pk2.ly -text y:
    entry $w.pk2.ey -relief sunken -textvariable TdynWizPriv(yaftpnt) -width 4
    label $w.pk2.lz -text z:
    entry $w.pk2.ez -relief sunken -textvariable TdynWizPriv(zaftpnt) -width 4
    bind  $w.pk2.ex <Return> " calcLength $w " 
    bind  $w.pk2.ey <Return> " calcLength $w "
    bind  $w.pk2.ez <Return> " calcLength $w "

    GidHelp "$w.pk2" "Insert the coordinates of the aft extreme (downstream)"\
	    "of the body (if exists it must be on the free surface) or pick"\
	    "the point.\nIn order this tool to work properly, geometry must"\
	    "be separated in the standard layers."

    grid $w.pk2.l -sticky w -row 0 -column 0 -columnspan 4
    grid $w.pk2.b -sticky e -row 0 -column 4 -columnspan 2
    grid $w.pk2.lx  -sticky w -row 1 -column 0
    grid $w.pk2.ex  -sticky ew -row 1 -column 1
    grid $w.pk2.ly  -sticky w -row 1 -column 2
    grid $w.pk2.ey  -sticky ew -row 1 -column 3
    grid $w.pk2.lz  -sticky w -row 1 -column 4
    grid $w.pk2.ez  -sticky ew -row 1 -column 5
    grid columnconfigure $w.pk2 1 -weight 1
    grid columnconfigure $w.pk2 3 -weight 1
    grid columnconfigure $w.pk2 5 -weight 1

    frame $w.ln -relief ridge -bd 2
    label $w.ln.l -textvariable TdynWizPriv(txtlen)
    grid  $w.ln.l -sticky w -row 0 -column 0

    GidHelp "$w.ln" "Characteristic length of the problem."

    grid $w.dib -padx 3 -pady 1
    grid $w.fx -sticky ew -padx 3 -pady 3
    grid $w.fy -sticky ew -padx 3 -pady 3
    grid $w.fz -sticky ew -padx 3 -pady 3
#    grid $w.din -sticky w -padx 3 -pady 3
    grid $w.ln -sticky ew -padx 3 -pady 3
    grid $w.pk1 -sticky ew -padx 3 -pady 3
    grid $w.pk2 -sticky ew -padx 3 -pady 3
}

proc UseCreateCompBox { {w ""} } {
    global TdynWizPriv
    set TdynWizPriv(length)   [LayerLength "body_wet body_dry symm_oy" X]
    if { $TdynWizPriv(length)<=0 } { 
	set TdynWizPriv(length) 1.0
    }
    set TdynWizPriv(xforpnt)  $TdynWizPriv(length)
    set TdynWizPriv(yforpnt)  0.0
    set TdynWizPriv(zforpnt)  0.0
    set TdynWizPriv(xaftpnt)  0.0
    set TdynWizPriv(yaftpnt)  0.0
    set TdynWizPriv(zaftpnt)  0.0 
    set TdynWizPriv(oxffac)   0.5  
    set TdynWizPriv(oxafac)   1.0 
    set TdynWizPriv(oyfac)    1.0
    set TdynWizPriv(ozfac)    1.0
    set TdynWizPriv(dofrees)  0

    if { $w == "" } { set w .gid.tdynwizard_createbox }
    
#    set TdynWizPriv(window) $w

    if { [info procs InitWindow] != "" } {
	InitWindow $w "Create control volume window" PreCreateBoxWindowGeom ""
    } else {
	toplevel $w
    }

    CreateCompBoxWindow $w

    set b $w.buts
    frame $b
    button $b.ok -text OK -width 5 -und 0 -command "DoCreateCompBox $w"
    button $b.cancel -text Cancel -width 5 -und 0 -command "destroy $w" 
    button $b.help -image [imagefromtdyn questionarrow.gif] -command "PickHelp $b.help" \
	    -highlightthickness 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $b.cancel"
    bind $b.ok <Return> "tkButtonInvoke $b.ok"
    bind $w <Escape> "tkButtonInvoke $b.cancel"
    bind $w <Alt-o> "tkButtonInvoke $b.ok"
    bind $w <Alt-c> "tkButtonInvoke $b.cancel"
    
    grid $b.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $b.cancel -row 2 -column 2 -pady 2 -sticky w 
    grid $b.help -row 2 -column 3 -pady 2 -sticky w
    grid $b -padx 3 -pady 3
    grid rowconfigure $w {1 2 3} -weight 1
    grid columnconfigure $w 0 -weight 1 
    wm geom $w ""
}

proc calcLength { { w "" } } {
    global TdynWizPriv
    if {![IsWord real $TdynWizPriv(xforpnt) $w ""] } { return }
    if {![IsWord real $TdynWizPriv(yforpnt) $w ""] } { return }
    if {![IsWord real $TdynWizPriv(zforpnt) $w ""] } { return }
    if {![IsWord real $TdynWizPriv(xaftpnt) $w ""] } { return }
    if {![IsWord real $TdynWizPriv(yaftpnt) $w ""] } { return }
    if {![IsWord real $TdynWizPriv(zaftpnt) $w ""] } { return }
    set d0 [expr double($TdynWizPriv(xforpnt))-$TdynWizPriv(xaftpnt)]
    set d1 [expr double($TdynWizPriv(yforpnt))-$TdynWizPriv(yaftpnt)]
    set d2 [expr double($TdynWizPriv(zforpnt))-$TdynWizPriv(zaftpnt)]
    set TdynWizPriv(length) [expr sqrt($d0*$d0+$d1*$d1+$d2*$d2)]
    set TdynWizPriv(txtlen) [format {Length = %.2g} $TdynWizPriv(length)]
    if { $w != "" } {
	setOXLength $w $TdynWizPriv(oxfac) 
	setOYLength $w $TdynWizPriv(oyfac)
	setOZLength $w $TdynWizPriv(ozfac)
    }
}

proc setOXFLength { w fac } {
    global TdynWizPriv
    set TdynWizPriv(oxffac) $fac
    set TdynWizPriv(oxfac) [format {%.1f} [expr \
	    double($TdynWizPriv(oxafac))+$TdynWizPriv(oxffac)]]
    set TdynWizPriv(oxlen) [format {%.1f} [expr \
	    double($TdynWizPriv(length))*$TdynWizPriv(oxfac)]]
    set TdynWizPriv(oxres) " x Length = $TdynWizPriv(oxlen)"
    ActualizeCreateCompBoxCanvas $w
}

proc setOXALength { w fac } {
    global TdynWizPriv
    set TdynWizPriv(oxafac) $fac
    set TdynWizPriv(oxfac) [format {%.1f} [expr \
	    double($TdynWizPriv(oxafac))+$TdynWizPriv(oxffac)]]
    set TdynWizPriv(oxlen) [format {%.1f} [expr \
	    double($TdynWizPriv(length))*$TdynWizPriv(oxfac)]]
    set TdynWizPriv(oxres) " x Length = $TdynWizPriv(oxlen)"
    ActualizeCreateCompBoxCanvas $w
}

proc setOXLength { w fac } {
    global TdynWizPriv
    if { ($TdynWizPriv(length) <= 0.0) } {
	tk_dialogRAM .tempwin {Error window} "Length value out of limits" \
		error 0 OK 
	set TdynWizPriv(length) 1.0
    } elseif {![IsWord real+ $fac . ""] } {
	return
    }
    if { $fac <= 0.0 } { set fac 0.2 } 
    set facrat [expr double($fac)/($TdynWizPriv(oxafac)+$TdynWizPriv(oxffac))]
    set TdynWizPriv(oxffac) [expr $facrat*$TdynWizPriv(oxffac)]
    set TdynWizPriv(oxafac) [expr $facrat*$TdynWizPriv(oxafac)]
    $w.fx.dw.s1 set $TdynWizPriv(oxffac)
    $w.fx.dw.s2 set $TdynWizPriv(oxafac)
    set TdynWizPriv(oxfac) [format {%.1f} $fac]
    set TdynWizPriv(oxlen) [format {%.1f} [expr $TdynWizPriv(length)*$fac]]
    set TdynWizPriv(oxres) " x Length = $TdynWizPriv(oxlen)"
    ActualizeCreateCompBoxCanvas $w
}

proc setOYLength { w fac } {
    global TdynWizPriv
    if { ($TdynWizPriv(length) <= 0.0) } {
	tk_dialogRAM .tempwin {Error window} "Length value out of limits" \
		error 0 OK 
	set TdynWizPriv(length) 1.0
    } elseif {![IsWord real+ $fac . ""] } {
	return
    }
    if { $fac <= 0.0 } { set fac 0.1 } 
    $w.fy.s set $fac
    set TdynWizPriv(oyfac) [format {%.1f} $fac]
    set TdynWizPriv(oylen) [format {%.1f} [expr $TdynWizPriv(length)*$fac]]
    set TdynWizPriv(oyres) " x Length = $TdynWizPriv(oylen)"
    ActualizeCreateCompBoxCanvas $w
}

proc setOZLength { w fac } {
    global TdynWizPriv
     if { ($TdynWizPriv(length) <= 0.0) } {
	tk_dialogRAM .tempwin {Error window} "Length value out of limits" \
		error 0 OK 
	set TdynWizPriv(length) 1.0
    }  elseif {![IsWord real+ $fac . ""] } {
	return
    }
    if { $fac <= 0.0 } { set fac 0.1 } 
    $w.fz.s set $fac
    set TdynWizPriv(ozfac) [format {%.1f} $fac]
    set TdynWizPriv(ozlen) [format {%.1f} [expr $TdynWizPriv(length)*$fac]]
    set TdynWizPriv(ozres) " x Length = $TdynWizPriv(ozlen)"
    ActualizeCreateCompBoxCanvas $w
}

#   A---B
#   |\  |\
#   | \   \
#   |  D---C   Aqui se crea la caja. Su estructura es la del dibujo del margen.
#   E--|F  |
#    \ | \ |
#     \|  \|
#      H---G
proc DoCreateCompBox { { window "" } } {
    global TdynWizPriv BoxInfo

    # Se inicializa la caja y capas
    if { [setUpBoxInfo] == 0 } { return 0 }
    # Forma de creacin de la superficie libre
    set free_surf "bylines"
    # Se destruye la ventana y deshabilitan los grficos
    if { $window != "" } { destroy $window }
    GraphicsOFF
    # Se enva todo hacia atrs y se crea la capa free_surf
    set TdynWizPriv(tolerance) [expr 1.0e-5*$TdynWizPriv(length)]
    set BoxIndex(PFP) [SearchPointInSphere $BoxInfo(PFP) $TdynWizPriv(tolerance)]
    set BoxIndex(PAP) [SearchPointInSphere $BoxInfo(PAP) $TdynWizPriv(tolerance)]
    CreateLayer free_surf
    .central.s process escape escape escape utilities layer SendToBack Volumes 1: escape
    .central.s process escape escape escape utilities layer SendToBack Surfaces 1: escape
    .central.s process escape escape escape utilities layer SendToBack Lines 1: escape
    .central.s process escape escape escape utilities layer SendToBack Points 1: escape
    # Create box points
    set points [list "A" "B" "C" "D" "E" "F" "G" "H"]
    foreach pnt $points {
	.central.s process escape escape escape escape geometry create point $BoxInfo($pnt) old
	set BoxIndex($pnt) [SearchPointInSphere $BoxInfo($pnt) $TdynWizPriv(tolerance)]
    }
    # Lineas A-B-E-F y cara ABEF
    CreateLayer inlet_ox
    DrawLineByIndex $BoxIndex(A) $BoxIndex(B)
    set BoxInfo(A_B) [SearchLine $BoxIndex(A) $BoxIndex(B) inlet_ox]
    DrawLineByIndex $BoxIndex(B) $BoxIndex(F)
    set BoxInfo(B_F) [SearchLine $BoxIndex(B) $BoxIndex(F) inlet_ox]
    DrawLineByIndex $BoxIndex(E) $BoxIndex(F)
    set BoxInfo(E_F) [SearchLine $BoxIndex(E) $BoxIndex(F) inlet_ox]
    DrawLineByIndex $BoxIndex(A) $BoxIndex(E)
    set BoxInfo(A_E) [SearchLine $BoxIndex(A) $BoxIndex(E) inlet_ox]
    .central.s process escape escape escape escape geometry create nurbssurface \
	    $BoxInfo(A_B) $BoxInfo(B_F) $BoxInfo(E_F) $BoxInfo(A_E) escape
    if { $BoxInfo(A_B)=="" || $BoxInfo(B_F)=="" || $BoxInfo(E_F)=="" || $BoxInfo(A_E)=="" } {
	GiveError "Error creating inlet surface. Check layers definition."
	return 0
    }
    # Lineas B-C-F-G y cara BCFG
    CreateLayer lateral_oy
    DrawLineByIndex $BoxIndex(B) $BoxIndex(C)
    set BoxInfo(B_C) [SearchLine $BoxIndex(B) $BoxIndex(C) lateral_oy]
    DrawLineByIndex $BoxIndex(C) $BoxIndex(G)
    set BoxInfo(C_G) [SearchLine $BoxIndex(C) $BoxIndex(G) lateral_oy]
    DrawLineByIndex $BoxIndex(F) $BoxIndex(G)
    set BoxInfo(F_G) [SearchLine $BoxIndex(F) $BoxIndex(G) lateral_oy]
    .central.s process escape escape escape escape geometry create nurbssurface \
	    $BoxInfo(B_C) $BoxInfo(C_G) $BoxInfo(F_G) $BoxInfo(B_F) escape 
    if { $BoxInfo(B_C)=="" || $BoxInfo(C_G)=="" || $BoxInfo(F_G)=="" } {
	GiveError "Error creating lateral surface. Check layers definition."
	return 0
    }
    # Lineas C-D-G-H y cara CDGH
    CreateLayer outlet_ox
    DrawLineByIndex $BoxIndex(C) $BoxIndex(D)
    set BoxInfo(C_D) [SearchLine $BoxIndex(C) $BoxIndex(D) outlet_ox]
    DrawLineByIndex $BoxIndex(D) $BoxIndex(H)
    set BoxInfo(D_H) [SearchLine $BoxIndex(D) $BoxIndex(H) outlet_ox]
    DrawLineByIndex $BoxIndex(G) $BoxIndex(H)
    set BoxInfo(G_H) [SearchLine $BoxIndex(G) $BoxIndex(H) outlet_ox]
    .central.s process escape escape escape escape geometry create nurbssurface \
	    $BoxInfo(C_D) $BoxInfo(D_H) $BoxInfo(G_H) $BoxInfo(C_G) escape 
    if { $BoxInfo(C_D)=="" || $BoxInfo(D_H)=="" || $BoxInfo(G_H)=="" } {
	GiveError "Error creating outlet surface. Check layers definition."
	return 0
    }
    # Lineas E-F-G-H y cara EFGH
    CreateLayer lateral_oz
    DrawLineByIndex $BoxIndex(E) $BoxIndex(H)
    set BoxInfo(E_H) [SearchLine $BoxIndex(E) $BoxIndex(H) lateral_oz]
    .central.s process escape escape escape escape geometry create nurbssurface \
	    $BoxInfo(E_F) $BoxInfo(F_G) $BoxInfo(G_H) $BoxInfo(E_H) escape
    if { $BoxInfo(E_H)=="" } {
	GiveError "Error creating bottom surface. Check layers definition."
	return 0
    }
    # Resto de superficies
    .central.s process escape escape escape utilities layers bringtofront symm_oy
    if { [IsLayerEmpty symm_oy lines] } {
	# Caso sin simetra y sin superficie libre
	if { $TdynWizPriv(dofrees)==0 } {
	    # Lineas A-D-E-H y cara ADEH
	    ToUseLayer lateral_oy
	    DrawLineByIndex $BoxIndex(A) $BoxIndex(D)
	    set BoxInfo(A_D) [SearchLine $BoxIndex(A) $BoxIndex(D) lateral_oy]
	    .central.s process escape escape escape escape geometry create nurbssurface \
		    $BoxInfo(A_D) $BoxInfo(A_E) $BoxInfo(E_H) $BoxInfo(D_H) escape
	    if { $BoxInfo(A_D)=="" } {
		GiveError "Error creating lateral surface. Check layers definition."
		return 0
	    }
	    # Lineas A-B-C-D y cara ABCD
	    ToUseLayer lateral_oz
	    .central.s process escape escape escape escape geometry create nurbssurface \
		    $BoxInfo(A_B) $BoxInfo(B_C) $BoxInfo(C_D) $BoxInfo(A_D) escape
	} else {
	    # Caso sin simetra y con superficie libre 
	    # Lineas A-D-E-H y cara ADEH
	    ToUseLayer lateral_oy
	    DrawLineByIndex $BoxIndex(A) $BoxIndex(D)
	    set BoxInfo(A_D) [SearchLine $BoxIndex(A) $BoxIndex(D) lateral_oy]
	    .central.s process escape escape escape escape geometry create nurbssurface \
		    $BoxInfo(A_D) $BoxInfo(A_E) $BoxInfo(E_H) $BoxInfo(D_H) escape
	    if { $BoxInfo(A_D)=="" } {
		GiveError "Error creating lateral surface. Check layers definition."
		return 0
	    }
	    # Lineas A-B-C-D y cara ABCD
	    ToUseLayer free_surf
	    .central.s process escape escape escape utilities layers bringtofront free_surf
	    set FreeSLines [GiveLines free_surf]
	    .central.s process escape escape escape escape geometry create nurbssurface \
		$BoxInfo(A_B) $BoxInfo(B_C) $BoxInfo(C_D) $BoxInfo(A_D) $FreeSLines escape
	    set BoxInfo(A_B_C_D) [GiveOneSurface free_surf]
	    .central.s process escape escape escape escape utilities collapse lines \
		$BoxInfo(A_B) $BoxInfo(B_C) $BoxInfo(C_D) $BoxInfo(A_D) layer:free_surf escape
	    if { $BoxInfo(A_B_C_D)=="" } {
		.central.s process escape escape escape escape utilities layers bringtofrontall
		.central.s process escape escape escape escape
		GraphicsON
		WarnWin "Error found generating free surface.\n It must be created by hand."
		return 1
	    } 
	}
    } else {
	if { $TdynWizPriv(dofrees)==0 } {
	    # Caso con simetra y sin superficie libre
	    # Lineas A-D-E-H y cara ADEH con el cuerpo
	    ToUseLayer lateral_oy
	    DrawLineByIndex $BoxIndex(A) $BoxIndex(D)
	    set BoxInfo(A_D) [SearchLine $BoxIndex(A) $BoxIndex(D) lateral_oy]
	    .central.s process escape escape escape escape geometry create nurbssurface \
		    $BoxInfo(A_D) $BoxInfo(A_E) $BoxInfo(E_H) $BoxInfo(D_H) escape
	    if { $BoxInfo(A_D)=="" } {
		GiveError "Error creating lateral surface. Check layers definition."
		return 0
	    } 
	    set BoxInfo(A_D_E_H) [GiveOneSurface lateral_oy]
	    set LateralLines [GiveLines symm_oy]
	    .central.s process escape escape escape escape geometry edit holenurb \
		    $BoxInfo(A_D_E_H) $LateralLines escape
	    # Lineas A-B-C-D y cara ABCD
	    ToUseLayer lateral_oz
	    .central.s process escape escape escape escape geometry create nurbssurface \
		    $BoxInfo(A_B) $BoxInfo(B_C) $BoxInfo(C_D) $BoxInfo(A_D) escape
	} else { 
	    # Caso con simetra y con superficie libre
	    # Lineas A-PFP y D-PAP
	    ToUseLayer symm_oy
	    DrawLineByIndex $BoxIndex(PFP) $BoxIndex(A)            
	    set BoxInfo(A_PFP) [SearchLine $BoxIndex(A) $BoxIndex(PFP) symm_oy]
	    if { $BoxInfo(A_PFP)=="" } {        
		GiveError "Error creating free surface (A_PFP point). Check layers definition."
		return 0
	    }
	    DrawLineByIndex $BoxIndex(PAP) $BoxIndex(D)
	    set BoxInfo(D_PAP) [SearchLine $BoxIndex(D) $BoxIndex(PAP) symm_oy]
	    if { $BoxInfo(D_PAP)=="" } {        
		GiveError "Error creating free surface (D_PAP point). Check layers definition."
		return 0
	    }
	    # Lineas A-D-E-H y cara ADEH
	    .central.s process escape escape escape escape geometry create nurbssurface \
		    $BoxInfo(D_H) $BoxInfo(E_H) $BoxInfo(A_E) layer:symm_oy escape
	    # Lineas A-B-C-D y cara ABCD 
	    ToUseLayer free_surf
	    .central.s process escape escape escape utilities layers bringtofront free_surf
	    .central.s process escape escape escape escape geometry create nurbssurface \
		    $BoxInfo(A_B) $BoxInfo(B_C) $BoxInfo(C_D) \
		    $BoxInfo(D_PAP) $BoxInfo(A_PFP) layer:free_surf escape
	}  
    } 
    # Creamos el volumen
    CreateLayer volume
    .central.s process escape escape escape escape utilities layers bringtofront body_wet
    .central.s process escape escape escape escape geometry create volume layer:inlet_ox\
	    layer:outlet_ox layer:lateral_oy layer:lateral_oz layer:symm_oy\
	    layer:free_surf layer:body_wet escape
    # Se comprueba si se ha creado el volumen y se borra todo lo innecesario
    if { [GiveVolumes volume]=="" } {
	GiveError "Error creating control volume. Check free_surf and symm_oy layers definition."
	return 0
    }
    .central.s process escape escape escape escape utilities layers bringtofrontall
    .central.s process escape escape escape escape
    GraphicsON
    return 1
}

proc GiveError { msg } {
    .central.s process escape escape escape escape utilities layers sendtoback surfaces layer:body_wet
    .central.s process escape escape escape escape geometry delete surface 1:
    .central.s process escape escape escape escape geometry delete line 1:
    .central.s process escape escape escape escape geometry delete point 1: 
    .central.s process escape escape escape escape utilities layers bringtofrontall
    .central.s process escape escape escape escape
    GraphicsON
    tk_dialogRAM .gid.tempwin {Error window} "$msg" error 0 OK 
}

# 
#   La caja se inicializa en este procedimiento.
#  
proc setUpBoxInfo {} {
    global TdynWizPriv BoxInfo
#   Comprobamos que los puntos son correctos 
    if { ![IsWord real $TdynWizPriv(xforpnt)] || ![IsWord real $TdynWizPriv(yforpnt)] || \
	    ![IsWord real $TdynWizPriv(zforpnt)] || ![IsWord real $TdynWizPriv(xaftpnt)] || \
	    ![IsWord real $TdynWizPriv(yaftpnt)] || ![IsWord real $TdynWizPriv(zaftpnt)] } { 
	tk_dialogRAM .gid.tempwin {Error window} "Wrong reference points data" \
		error 0 OK 
	return 0 
    }
    calcLength
    if { $TdynWizPriv(length) <= 0.0 } {
	tk_dialogRAM .gid.tempwin {Error window} \
		"Length value out of limits\nSelect correct reference points" \
		error 0 OK 
	return 0
    }
    if { [IsLayerEmpty body_wet surfaces] } {
	WarnWin "In order this tool to understand your problem, follow\
		next steps:\n\n 1.-Create or import the body (bodies) geometry\n\
		2.-Sent to the layer called body_wet the surfaces of the body within\
		the fluid\n 3.-Sent to the layer called body_dry the part of the\
		body out of the fluid (if any)\n 4.-Sent to the layer called\
		free_surf the lines of the body (if any) belonging to the free\
		surface (floating line)\n 5.-Sent to the layer the called symm_oy\
		the lines of the body (if any) belonging to the symmetry plane and\
		\n    below the floating line (must be perpendicular to the OY axis)\
		\n\nLayers window can be accessed through the menu option Utilities>Layers."
	return 0 
    }
    # Comprobamos la orientacin Ox
    if { $TdynWizPriv(xforpnt) > $TdynWizPriv(xaftpnt) } {
	set TdynWizPriv(oxdir) 1
    } elseif { $TdynWizPriv(xforpnt) < $TdynWizPriv(xaftpnt) } {
	set TdynWizPriv(oxdir) -1
    } else {
	tk_dialogRAM .gid.tempwin {Error window} \
		"Create control volume tool only works if the body is oriented \
		in the x direction" \
		error 0 OK 
	return 0
    }
#   Calculamos las coordenadas de los puntos de referencia de la caja
    set xforlen [expr $TdynWizPriv(length)*$TdynWizPriv(oxffac)]
    set xaftlen [expr $TdynWizPriv(length)*$TdynWizPriv(oxafac)]
    set xmin [expr $TdynWizPriv(xaftpnt) - $TdynWizPriv(oxdir)*$xaftlen]
    set xmax [expr $TdynWizPriv(xforpnt) + $TdynWizPriv(oxdir)*$xforlen]
    if {$TdynWizPriv(yaftpnt) > $TdynWizPriv(yforpnt)}  {
	set ymin [expr $TdynWizPriv(yforpnt) - $TdynWizPriv(oylen)]
	set ymax [expr $TdynWizPriv(yaftpnt) + $TdynWizPriv(oylen)]
    } else {        
	set ymin [expr $TdynWizPriv(yaftpnt) - $TdynWizPriv(oylen)]
	set ymax [expr $TdynWizPriv(yforpnt) + $TdynWizPriv(oylen)]
    }
    if {$TdynWizPriv(zaftpnt) > $TdynWizPriv(zforpnt)}  {
	set zmin [expr $TdynWizPriv(zforpnt) - $TdynWizPriv(ozlen)]
	set zmax [expr $TdynWizPriv(zaftpnt) + $TdynWizPriv(ozlen)]
    } else {        
	set zmin [expr $TdynWizPriv(zaftpnt) - $TdynWizPriv(ozlen)]
	set zmax [expr $TdynWizPriv(zforpnt) + $TdynWizPriv(ozlen)]
    }
#   Comprobamos si es necesario crear la superficie libre y si existe espejo
    set TdynWizPriv(transom) 0
    if { ![IsLayerEmpty free_surf lines] } { 
	set TdynWizPriv(dofrees) 1
	if { $TdynWizPriv(zaftpnt) < $TdynWizPriv(zforpnt) } { 
	    set zmax $TdynWizPriv(zforpnt)
	    set TdynWizPriv(transom) 1
	} else {
	    set zmax $TdynWizPriv(zaftpnt)
	}
    }
#   Comprobamos la orientacin Oy si el caso es simtrico 
    set TdynWizPriv(yorient) 0.0
    if { ![IsLayerEmpty symm_oy lines] && [ExistsLayer body_wet] } {
#        global GidPriv TdynWizPriv
#        .central.s process escape escape escape escape SelectEntities JustOne Surfaces\
#                layer:body_wet escape
#        trace var GidPriv(selection) w [OrientaOY]
	set TdynWizPriv(yorient) [expr [LayerLength body_wet YC]-0.5*($ymin+$ymax)]
	set bodyylen [expr 2.0*abs($TdynWizPriv(yaftpnt)-$TdynWizPriv(yorient))]
	if { $TdynWizPriv(yorient) == 0.0 } { 
	    tk_dialogRAM .gid.tempwin {Error window} \
		"Error trying to obtain the y orientation of the geometry. Check geometry" \
		error 0 OK 
	    return 0 
	} elseif { $TdynWizPriv(yaftpnt) != $TdynWizPriv(yforpnt) } {
	    tk_dialogRAM .gid.tempwin {Error window} \
		"Wrong y = yo plane of symmetry. Check Reference points" \
		error 0 OK 
	    return 0
	} elseif { $TdynWizPriv(yorient) > 0.0 } {  
	    set ymin [expr $TdynWizPriv(yaftpnt) + $TdynWizPriv(oylen)]
	    set ymax [expr $TdynWizPriv(yaftpnt)] 
	} else {
	    set ymin [expr $TdynWizPriv(yaftpnt) - $TdynWizPriv(oylen)]
	    set ymax [expr $TdynWizPriv(yaftpnt)] 
	}
    }
#   Se calculan los puntos de la caja
    set BoxInfo(PFP) "$TdynWizPriv(xforpnt) $TdynWizPriv(yforpnt) $TdynWizPriv(zforpnt)"  
    set BoxInfo(PAP) "$TdynWizPriv(xaftpnt) $TdynWizPriv(yaftpnt) $TdynWizPriv(zaftpnt)"  
 
    set BoxInfo(A) "$xmax $ymax $zmax" 
    set BoxInfo(B) "$xmax $ymin $zmax"  
    set BoxInfo(C) "$xmin $ymin $zmax"  
    set BoxInfo(D) "$xmin $ymax $zmax"  
    set BoxInfo(E) "$xmax $ymax $zmin" 
    set BoxInfo(F) "$xmax $ymin $zmin"  
    set BoxInfo(G) "$xmin $ymin $zmin"  
    set BoxInfo(H) "$xmin $ymax $zmin"

    return 1
}

proc OrientaOY {} {
    global GidPriv TdynWizPriv
    set num [lindex [lindex $GidPriv(selection) 0] 1]
    set nnum ""
    regexp {[0-9]*} $num nnum
    if { $nnum != "" } {
	set retval [.central.s info list_entities -more Surfaces $nnum] 
	regexp {Center:[ ]*([+-]*[0-9.]*)[ ]*([+-]*[0-9.]*)} $retval {} {} TdynWizPriv(yorient)
    }        
}

proc CheckVolume {} {
    global GidPriv TdynWizPriv
    set num [lindex [lindex $GidPriv(selection) 0] 1]
    set nnum ""
    regexp {[0-9]*} $num nnum
    if { $nnum != "" } {
	set TdynWizPriv(volume) 1
    }
}

proc SetCompBoxPick { win pnt } {
    global WarnLineVar CompBoxWin aftRefP
    set aftRefP $pnt  
    set CompBoxWin $win 
    gidmakecurrent
    .central.s process escape escape escape escape utilities id
    trace var WarnLineVar w ProcessCompBoxPick
}

#proc ProcessCompBoxPick {} {
#    global WarnLineVar
#    gidmakecurrent
#    .central.s process escape escape escape escape utilities id
#    trace var WarnLineVar w ProcessCompBoxPick
#}

proc ProcessCompBoxPick {name1 name2 op} {
    global WarnLineVar
    upvar #0 $name1 cad
    global aftRefP CompBoxWin TdynWizPriv

    if { $cad == "Leaving" } { 
	trace vdelete WarnLineVar w ProcessCompBoxPick
	return 
    }
    
    if { ![regexp  {Point number[ ]*([^.]*)[^=]*=([^, ]*)[^=]*=([^, ]*)[^=]*=([^, ]*)} \
	    $cad trash pntnum xval yval zval]} { return }
    
#    tk_messageBox -message "Aqui estn... $xval $yval $zval"

    if { $aftRefP == 0 } {
	set TdynWizPriv(xforpnt) $xval
	set TdynWizPriv(yforpnt) $yval
	set TdynWizPriv(zforpnt) $zval
	calcLength $CompBoxWin
    } else {
	set TdynWizPriv(xaftpnt) $xval
	set TdynWizPriv(yaftpnt) $yval
	set TdynWizPriv(zaftpnt) $zval
	calcLength $CompBoxWin
    } 
    trace vdelete WarnLineVar w ProcessCompBoxPick
    .central.s process escape
}

proc ActualizeCreateCompBoxCanvas { w } {
    global TdynWizPriv

    set xc 130.0
    set yc 80.0
    set xl 9.0
    set yl 4.0
    set zl 5.5
    set x1 [expr $xc-$xl*(2.0+$TdynWizPriv(oxffac))]
    set x2 [expr $xc+$xl*(2.0+$TdynWizPriv(oxafac))]
    set y1 [expr $yc-$zl*(2.0+$TdynWizPriv(ozfac))]
    set y2 [expr $yc+$zl*(2.0+$TdynWizPriv(ozfac))]
    set xl1 [expr $xc-2.0*$yl*(2.0+$TdynWizPriv(oyfac))]
    set xl2 [expr $xc+2.0*$yl*(2.0+$TdynWizPriv(oyfac))]
    set yl1 [expr $yc-$yl*(2.0+$TdynWizPriv(oyfac))]
    set yl2 [expr $yc+$yl*(2.0+$TdynWizPriv(oyfac))]
    set xo1 [expr $xc-11]
    set xo2 [expr $xc+11]
    set yo1 [expr $yc-6]
    set yo2 [expr $yc+6]

    $w.dib delete compbox

    $w.dib create line $x1 $yc $x2 $yc -width 3 -arrow both \
	    -arrowshape "8 8 3" -tags compbox 
    $w.dib create line $xc $y1 $xc $y2 -width 3 -arrow both \
	    -arrowshape "8 8 3" -tags compbox 
    $w.dib create line $xl1 $yl1 $xl2 $yl2 -width 3 -arrow both \
	    -arrowshape "8 8 3" -tags compbox

    set x1 [expr $xc-$xl*12.0]
    set x2 [expr $xc+$xl*12.0]
    set y1 [expr $yc-$zl*12.0]
    set y2 [expr $yc+$zl*12.0]
    set xl1 [expr $xc-$yl*24.0]
    set xl2 [expr $xc+$yl*24.0]
    set yl1 [expr $yc-$yl*12.0]
    set yl2 [expr $yc+$yl*12.0]

    $w.dib create line $x1 $yc $x2 $yc -width 1 -dash {2 4} \
	    -tags compbox
    $w.dib create line $xc $y1 $xc $y2 -width 1 -dash {2 4} \
	    -tags compbox
    $w.dib create line $xl1 $yl1 $xl2 $yl2 -width 1 \
	    -dash {2 4} -tags compbox

    $w.dib create text 145 10 -text "z axis"
    $w.dib create text 20 85 -text "x axis"
    $w.dib create text 40 20 -text "y axis"

    $w.dib create oval $xo1 $yo1 $xo2 $yo2 -width 1 -outline red \
	    -dash {1 1} -tags compbox
}

################################################################
# ASIGNACIN AUTOMTICA DE CONDICIONES DE CONTORNO Y OTROS DATOS
################################################################

proc CreateDefDataWindow { w } {
    global TdynWizPriv

    frame $w.f0 -relief ridge -bd 2
    label $w.f0.lb1 -text "Velocity" -width 14 -anchor w
    entry $w.f0.en1 -textvariable TdynWizPriv(velocity) 
    tk_optionMenu $w.f0.un1 TdynWizPriv(velocunits) m/s knot cm/s mm/s mach
    $w.f0.un1 conf -padx 5 -width 6
    label $w.f0.lb2 -text "Simulation Time" -width 14 -anchor w
    entry $w.f0.en2 -textvariable TdynWizPriv(simultime)
    tk_optionMenu $w.f0.un2 TdynWizPriv(stimeunits) s cs ms min hour
    $w.f0.un2 conf -padx 5 -width 6
    grid $w.f0.lb1 $w.f0.en1 $w.f0.un1 -sticky w  
    grid $w.f0.lb2 $w.f0.en2 $w.f0.un2 -sticky w 
    grid conf $w.f0.en1 $w.f0.en2 -sticky ew
    grid conf $w.f0.un1 $w.f0.un2 -sticky e
    grid columnconfigure $w.f0 1 -weight 1
   
    GidHelp "$w.f0" "Insert the free velocity (with the correct sign) of the problem"\
	    "and the simulation (physical) time of the problem.\nIn order this tool"\
	    "to work correctly geometry must be separated in the standard layers."

    frame $w.f1 -relief ridge -bd 2
    label $w.f1.lb1 -text "Density" -width 14 -anchor w
    entry $w.f1.en1 -textvariable TdynWizPriv(density)
    tk_optionMenu $w.f1.un1 TdynWizPriv(densiunits) Kg/m3 ton/m3 g/lt Kg/lt Kg/cc g/cc
    label $w.f1.lb2 -text "Viscosity" -width 14 -anchor w
    entry $w.f1.en2 -textvariable TdynWizPriv(viscosity)
    tk_optionMenu $w.f1.un2 TdynWizPriv(viscounits) Pa.s Kg/m.s ton/m.s g/m.s
    $w.f1.un1 conf -padx 5 -width 6
    $w.f1.un2 conf -padx 5 -width 6
    grid $w.f1.lb1 $w.f1.en1 $w.f1.un1 -sticky w
    grid $w.f1.lb2 $w.f1.en2 $w.f1.un2 -sticky w 
    grid conf $w.f1.en1 $w.f1.en2 -sticky ew
    grid conf $w.f1.un1 $w.f1.un2 -sticky e
    grid columnconfigure $w.f1 1 -weight 1

    GidHelp "$w.f1" "Insert density and viscosity of the fluid."

    frame $w.f2 -relief ridge -bd 2
#    if { [info exists TdynWizPriv(length)] } {
#        label $w.f2.lb -text "Geometry units"
#        tk_optionMenu $w.f2.un TdynWizPriv(geomeunits) m cm mm
#        $w.f2.un conf -padx 5 -width 6
#        grid $w.f2.lb $w.f2.un -sticky w 
#        grid conf $w.f2.un -sticky e
#        grid columnconfigure $w.f2 0 -weight 1
#        GidHelp "$w.f2" "Insert the geometrical units of the problem."
#        bind  $w.f0.en1 <Return> { UpdateTdynData }
#    } else {
	label $w.f2.lb -text "Length" -width 14 -anchor w
	entry $w.f2.en -textvariable TdynWizPriv(length)
	tk_optionMenu $w.f2.un TdynWizPriv(geomeunits) m cm mm
	$w.f2.un conf -padx 5 -width 6
	grid $w.f2.lb $w.f2.en $w.f2.un -sticky w 
	grid conf $w.f2.en -sticky we
	grid conf $w.f2.un -sticky e
	grid columnconfigure $w.f2 1 -weight 1
	GidHelp "$w.f2" "Insert the characteristic length of the problem."
#    }
    frame $w.f3 -relief ridge -bd 2
    if { ![IsLayerEmpty free_surf surfaces] } {
	label $w.f3.lb1 -text "Free surface" -width 15 -anchor w
	tk_optionMenu $w.f3.un1 TdynWizPriv(solvefrees) Solve_Beta No_Solve_Beta
    } else { 
	label $w.f3.lb1 -text "Free surface" -width 15 -anchor w
	tk_optionMenu $w.f3.un1 TdynWizPriv(solvefrees) No_Solve_Beta 
	$w.f3.un1 configure -state disabled
    }
    label $w.f3.lb2 -text "Outlet type" -width 15 -anchor w
    tk_optionMenu $w.f3.un2 TdynWizPriv(assignpres) Fix_Pressure Free_Flow Natural
    label $w.f3.lb3 -text "Body type" -width 15 -anchor w
    tk_optionMenu $w.f3.un3 TdynWizPriv(body_wall) YplusWall ITTC_Wall InvisWall DeltaWall RoughWall Cw_U2Wall V_fixWall 
    label $w.f3.lb4 -text "Turbulence model" -width 15 -anchor w
    tk_optionMenu $w.f3.un4 TdynWizPriv(turbulence) Laminar Mixing_Length Smagorinsky Kinetic_Energy \
	    K_Energy_Two_Layers K_E_High_Reynolds K_E_Two_Layers K_E_Lam_Bremhorst K_E_Launder_Sharma \
	    K_Omega K_Omega_SST K_KT Spalart_Allmaras
    $w.f3.un1 conf -padx 5 -width 12
    $w.f3.un2 conf -padx 5 -width 12
    $w.f3.un3 conf -padx 5 -width 12 
    $w.f3.un4 conf -padx 5 -width 12
    grid $w.f3.lb1 $w.f3.un1 -sticky w
    grid $w.f3.lb2 $w.f3.un2 -sticky w  
    grid $w.f3.lb3 $w.f3.un3 -sticky w 
    grid $w.f3.lb4 $w.f3.un4 -sticky w 
    grid conf $w.f3.un1 $w.f3.un2 $w.f3.un3 $w.f3.un4 -sticky e
    grid columnconfigure $w.f3 0 -weight 1

    GidHelp "$w.f3" "Define the type of the main boundaries, outlet, free surface and body."\
	    "Select if you want to solve free surface or not, if the pressure must be fixed"\
	    "at the outlet and the type of wall of the body.\nIf neccesary, select also a"\
	    "turbulence model, or Laminar for none.\nIn order this tool to work correctly"\
	    "geometry must be separated in the standard layers."

    grid $w.f0 -sticky ew -padx 3 -pady 3 
    grid $w.f1 -sticky ew -padx 3 -pady 3
    grid $w.f2 -sticky ew -padx 3 -pady 3
    grid $w.f3 -sticky ew -padx 3 -pady 3

#    bind $w <Return> { UpdateTdynData }
}

proc UpdateTdynData { } {
    global TdynWizPriv
    if { [info exists TdynWizPriv(body_wall)] } {
	if { $TdynWizPriv(body_wall)=="InvisWall" } {
	    set TdynWizPriv(turbulence) "Laminar"
	}
    }
    if { $TdynWizPriv(velocity)!=0.0 } { 
	set Lun [ConvertLengthUnitsToSI $TdynWizPriv(geomeunits)]
	set Tun [ConvertTimeUnitsToSI $TdynWizPriv(stimeunits)]
	set Vun [ConvertVelocityUnitsToSI $TdynWizPriv(velocunits)]
	set TdynWizPriv(simultime) [format %.3g [expr \
		abs(8.0*$TdynWizPriv(length)/$TdynWizPriv(velocity))*$Lun/$Vun/$Tun]]
    }
}

proc UseCreateDefData { } {
    global TdynWizPriv

    set w .gid.tdynwizard_defdata
    if {![info exists TdynWizPriv(velocity)]  } { set TdynWizPriv(velocity)   1.0 } 
    if {![info exists TdynWizPriv(density)]   } { set TdynWizPriv(density)    1000.0 } 
    if {![info exists TdynWizPriv(viscosity)] } { set TdynWizPriv(viscosity)  0.001 }
    if {![info exists TdynWizPriv(simultime)] } { set TdynWizPriv(simultime)  25 }
    if {![info exists TdynWizPriv(geomeunits)] } { set TdynWizPriv(geomeunits) "m" }
    if {![info exists TdynWizPriv(stimeunits)] } { set TdynWizPriv(stimeunits) "s" }
    if {![info exists TdynWizPriv(velocunits)] } { set TdynWizPriv(velocunits) "m/s" }
    if { [info exists TdynWizPriv(oxdir)] } {
	if { [expr $TdynWizPriv(oxdir)*$TdynWizPriv(velocity)] > 0.0 } { 
	    set TdynWizPriv(velocity) [expr -$TdynWizPriv(velocity)]
	}
    }
    if { ![info exists TdynWizPriv(length)] } {
	set TdynWizPriv(length) [LayerLength "body_wet body_dry" D]
	if { $TdynWizPriv(length)<=0.0 } { set TdynWizPriv(length) 1.0 }
    }
    set TdynWizPriv(length) [format %.3g $TdynWizPriv(length)]
    UpdateTdynData
    if { [info procs InitWindow] != "" } {
	InitWindow $w "Define data window" PreDefDataWindowGeom ""
    } else {
	toplevel $w
    }

    CreateDefDataWindow $w

    set b $w.buts
    frame $b
    button $b.update -text Update -width 5 -und 0 -command "UpdateTdynData" 
    button $b.ok -text OK -width 5 -und 0 -command "DoDefDataWindow $w"
    button $b.cancel -text Cancel -width 5 -und 0 -command "destroy $w"   
    button $b.help -image [imagefromtdyn questionarrow.gif] -command "PickHelp $b.help" \
	    -highlightthickness 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $b.cancel"
    bind $b.ok <Return> "tkButtonInvoke $b.ok"
    bind $w <Escape> "tkButtonInvoke $b.cancel"
    bind $w <Alt-o> "tkButtonInvoke $b.ok"
    bind $w <Alt-c> "tkButtonInvoke $b.cancel"
    bind $w <Return> { UpdateTdynData }
    grid $b.update $b.ok $b.cancel $b.help -sticky e -pady 3 -padx 3
    grid $b -padx 3 -pady 3
    grid rowconfigure $w {0 1 2 3} -weight 1
    grid columnconfigure $w 0 -weight 1
}

proc DoDefDataWindow { w } {
    global TdynWizPriv
    #   Comprobaciones
    if {![IsWord real  $TdynWizPriv(velocity) $w "In velocity field."] } { return }
    if {![IsWord real+ $TdynWizPriv(simultime) $w "In simulation time field."] } { return }
    if {![IsWord real+ $TdynWizPriv(viscosity) $w "In viscosity field."] } { return }
    if {![IsWord real+ $TdynWizPriv(density) $w "In density field."] } { return }
    if {![IsWord real+ $TdynWizPriv(length) $w "In characteristic length."] } { return } 
    if { [info exists TdynWizPriv(oxdir)] } {
	if { [expr $TdynWizPriv(oxdir)*$TdynWizPriv(velocity)] > 0.0 } { 
	    tk_dialogRAM $w.tempwin {Error window} "Wrong sign of the velocity found." error 0 OK  
	    return
	}
    } 
    if { [IsLayerEmpty body_wet surfaces] } {
	WarnWin "In order this tool to understand your problem, the geometry must be\
		separated in different layers:\n\n 1.-Layer body_wet containing the\
		surfaces of the body within the fluid\n 2.-Layer free_surf containing\
		the free surface (if any)\n 3.-Layer symm_oy containing the symmetry plane\
		(if any, perpendicular to OY axis)\n 4.-Layer lateral_oy containing the\
		boundaries perpendicular to OY axis\n 5.-Layer lateral_oz containing the\
		boundaries perpendicular to OZ axis\n 6.-Layer inlet_ox containing the\
		inlet boundary\n 7.-Layer outlet_ox containing the outlet boundary\n\nLayers\
		window can be accessed through the menu option Utilities>Layers."
	return 0 
    }
    if { $w != "" } { destroy $w }
    #   Se desabilitan los grficos y se desactiva la malla
    GraphicsOFF
    .central.s process escape escape escape escape geometry escape
    #   Se desasigna todo
    UnAssignConditions
    UnAssignMaterials
    #   Condicin de inflow
    AssignConditionToLayer inlet_ox Surface_Velocity_Field "1 1 1 0 0 0"
    AssignConditionToLayer inlet_ox Surface_Pressure_Field "1 0"
    #   Condicin de lateral
    if { [IsLayerEmpty free_surf surfaces] } {
	AssignConditionToLayer lateral_oy Surface_Velocity_Field "0 1 0 0 0 0"
    } elseif { $TdynWizPriv(assignpres) ==  "Fix_Pressure" } {
	AssignConditionToLayer lateral_oy Surface_Pressure_Field "1 0"
    }
    AssignConditionToLayer lateral_oz Surface_Velocity_Field "0 0 1 0 0 0"
    #   Condicin de simetra
#    AssignConditionToLayer symm_oy Surface_Velocity_Field "0 1 0 0 0 0"
    if { ![IsLayerEmpty symm_oy] } { 
	set OY_Symm1 [lindex [lindex [.central.s info layers -bbox symm_oy] 0] 1]
	set OY_Symm2 [lindex [lindex [.central.s info layers -bbox symm_oy] 0] 4]
	set OY_Symm [format %.2g [expr 0.5*($OY_Symm1+$OY_Symm2)]]
	EditProblemData "Yplane_Symmetry_in_Fluid Yplane_Symmetry_Position" \
		"Yes $OY_Symm"
    }
    #   Condicin de outflow
    if { $TdynWizPriv(assignpres) ==  "Fix_Pressure" } {
	AssignConditionToLayer outlet_ox Surface_Pressure_Field "1 0"
    } elseif  { $TdynWizPriv(assignpres) ==  "Free_Flow" } {
	AssignMaterialToLayer outlet_ox Out_Flow Surfaces
    }
    #   Se crea el material Fluid y se asigna a la capa volumen
    CreateMaterial Fluid Auto_Fluid \
	    "Density Density_Units Viscosity Viscosity_Units" \
	    "$TdynWizPriv(density) \[$TdynWizPriv(densiunits)\] $TdynWizPriv(viscosity) \
	    \[$TdynWizPriv(viscounits)\]"
    AssignMaterialToLayer volume Auto_Fluid Volumes
    #   Se crea el material Fluid_Body y se asigna a la capa body_wet
    if { $TdynWizPriv(solvefrees) == "No_Solve_Beta" } {
	CreateMaterial Fluid_Body Auto_Body [list BoundType Sharp_Angle Fix_Angle]\
		[list $TdynWizPriv(body_wall) 25 100]
    } else {
	CreateMaterial Fluid_Body Auto_Body [list BoundType Sharp_Angle Fix_Angle SternC_Angle]\
		[list $TdynWizPriv(body_wall) 25 100 70] 
    }
    AssignMaterialToLayer body_wet Auto_Body Surfaces
    #   Condicin de superficie libre
    if { ![IsLayerEmpty free_surf surfaces] } {
	if { $TdynWizPriv(solvefrees) == "No_Solve_Beta" } {
	    AssignConditionToLayer free_surf Surface_Fix_Velocity_Component "-Automatic- Z_Axis 0.0"
	    if { ![IsLayerEmpty body_trn surfaces] } {
		AssignConditionToLayer body_trn Surface_Fix_Velocity_Component "-Automatic- Z_Axis 0.0"
	    }
	    EditProblemData "Solve_Transpiration_in_Fluid" "No"
	} else {
	    CreateMaterial FreeSurface Auto_FreeSurface \
		    "Length Length_units Time_factor" "$TdynWizPriv(length) \[$TdynWizPriv(geomeunits)\] 2.5"
	    AssignMaterialToLayer free_surf Auto_FreeSurface Surfaces
	    if { ![IsLayerEmpty body_trn surfaces] } {
		AssignMaterialToLayer body_trn Auto_FreeSurface Surfaces
	    }
	    EditProblemData "Solve_Transpiration_in_Fluid" "Yes"
	}
    }
    #   Se cambian los datos generales del problema
    set safe_coef 0.05
    if { $TdynWizPriv(solvefrees) == "Solve_Beta" } {
	set safe_coef 0.01
    }
    set Lun [ConvertLengthUnitsToSI $TdynWizPriv(geomeunits)]
    set Tun [ConvertTimeUnitsToSI $TdynWizPriv(stimeunits)]
    set Vun [ConvertVelocityUnitsToSI $TdynWizPriv(velocunits)]
    set dt [format %.2g [expr $safe_coef*$TdynWizPriv(length)/abs($TdynWizPriv(velocity))*$Lun/$Vun/$Tun]]
    set stp [expr int($TdynWizPriv(simultime)/$dt)]
    if { $stp < 1 } { set stp 1 }
    set ost [expr int($stp/10.)]
    if { $ost < 1 } { set ost 1 }
    set refpnt 0.0
    set points [.central.s info list_entities points layer:free_surf]
    set points [regexp -linestop -all -inline {Num: ([ 0-9]*)} $points]
    for { set i 1 } { $i < [llength $points] } { incr i 2 } {
	set coord [lindex [.central.s info Coordinates [lindex $points $i]] 0]
	if { $i==1 } { 
	    set refpnt $coord 
	} else {
	    if { $TdynWizPriv(velocity)<0.0 && [lindex $coord 1]>[lindex $refpnt 1] } {
		set refpnt $coord
	    } elseif { $TdynWizPriv(velocity)>0.0 && [lindex $coord 1]<[lindex $refpnt 1] } {
		set refpnt $coord
	    }
	}
    }
    set TdynWizPriv(presorigin) [lindex $refpnt 2]
    if { $TdynWizPriv(presorigin)=="" } {  set TdynWizPriv(presorigin) 0.0 }
    EditProblemData "Number_of_Steps Time_increment Time_increment_Units Max_Iterations Output_Step Initial_Steps \
	    Output_Length_Units Output_Time_Units Geometry_Units Boundary_Velocity_Units Turbulence_Model \
	    Give_Pressure_Origin Pressure_Origin_OZ Initial_VelocityX" \
	    "$stp $dt \[$TdynWizPriv(stimeunits)\] 1 $ost $ost \[$TdynWizPriv(geomeunits)\] \
	    \[$TdynWizPriv(stimeunits)\] \[$TdynWizPriv(geomeunits)\] \[$TdynWizPriv(velocunits)\] \
	    $TdynWizPriv(turbulence) Yes $TdynWizPriv(presorigin) $TdynWizPriv(velocity)"
    #   Se habilitan los grficos
    GraphicsON
}

#######################################################
# ASIGNACIN DE TAMAOS DE MALLADO A LAS CAPAS ESTANDAR
#######################################################

proc GenerateMeshWindow { w } {
    global TdynWizPriv ProblemTypePriv

    set ProblemTypePriv(sizetransit) [.central.s info variable SizeTransitionsFactor]

    frame $w.f0 -relief ridge -bd 2 
    label $w.f0.lb -text {Unstructured size transitions}
    button $w.f0.bt0 \
	    -command {set ProblemTypePriv(sizetransit) [expr $ProblemTypePriv(sizetransit)-0.1]} \
	    -disabledforeground grey60 -text -
    scale $w.f0.sc -highlightbackground black -label slower -orient h -to 1.0 \
	    -resolution 0.1 -variable ProblemTypePriv(sizetransit)
    button $w.f0.bt1 \
	    -command {set ProblemTypePriv(sizetransit) [expr $ProblemTypePriv(sizetransit)+0.1]} \
	    -disabledforeground grey60 -text +
    GidHelp "$w.f0" "It controls whether the transitions between"\
	    "different element sizes are slow or fast."

    grid $w.f0.lb -sticky we -row 0 -column 1
    grid $w.f0.bt0 $w.f0.sc $w.f0.bt1 -sticky w
    grid conf $w.f0.sc -sticky we -padx 0 -pady 2
    grid conf $w.f0.bt1 -sticky e
    grid columnconf $w.f0 1 -weight 1

    set ProblemTypePriv(elemsizefactor) 0.8
    set ProblemTypePriv(prevsizefactor) 0.8

    frame $w.f1 -relief ridge -bd 2 
    label $w.f1.lb -text {Mesh size factor}
    button $w.f1.bt0 \
	    -command {ModifyFactor -0.1} \
	    -disabledforeground grey60 -text -
    scale $w.f1.sc -highlightbackground black -label finer -orient h -from 0.1 -to 1.0 \
	    -resolution 0.1 -variable ProblemTypePriv(elemsizefactor) -command {ApplySizeFactor}
    button $w.f1.bt1 \
	    -command {ModifyFactor +0.1} \
	    -disabledforeground grey60 -text +
    GidHelp "$w.f1" "It controls whether the mesh is coarser or finer, based on standard\
	    criteria.\n The greater factor, the coarser mesh."
    proc ModifyFactor { inc } {
	global TdynWizPriv ProblemTypePriv
	set ProblemTypePriv(elemsizefactor) [expr double($ProblemTypePriv(elemsizefactor))+$inc]
	ApplySizeFactor $ProblemTypePriv(elemsizefactor)
    }
    proc ApplySizeFactor {fact} {
	global TdynWizPriv ProblemTypePriv
	set factold $ProblemTypePriv(prevsizefactor)
	set factnew $ProblemTypePriv(elemsizefactor)
	set resize 1.0
	if { $factold < $factnew } {
	    set resize 1.25
	} elseif { $factold > $factnew } {
	    set resize 0.8
	}
	set TdynWizPriv(bodysize) [format %.2g [expr $TdynWizPriv(bodysize)*$resize]]
	set TdynWizPriv(freesize) [format %.2g [expr $TdynWizPriv(freesize)*$resize]]
	set ProblemTypePriv(genesize) [format %.2g [expr $ProblemTypePriv(genesize)*$resize]]
	set ProblemTypePriv(prevsizefactor) $ProblemTypePriv(elemsizefactor)
    }

    grid $w.f1.lb -sticky we -row 0 -column 1
    grid $w.f1.bt0 $w.f1.sc $w.f1.bt1 -sticky w
    grid conf $w.f1.sc -sticky we -padx 0 -pady 2
    grid conf $w.f1.bt1 -sticky e
    grid columnconf $w.f1 1 -weight 1

    frame $w.f4 -relief ridge -bd 2
    label $w.f4.lb -text "Layer body_wet size"
    entry $w.f4.eb -relief sunken -textvariable TdynWizPriv(bodysize) -width 2
    label $w.f4.lf -text "Layer free_surf size"
    entry $w.f4.ef -relief sunken -textvariable TdynWizPriv(freesize) -width 2
    label $w.f4.lg -text "General size"
    entry $w.f4.eg -relief sunken -textvariable ProblemTypePriv(genesize) -width 2

    grid $w.f4.lb $w.f4.eb -sticky w -padx 3 -pady 2 
    grid $w.f4.lf $w.f4.ef -sticky w -padx 3 -pady 2
    grid $w.f4.lg $w.f4.eg -sticky w -padx 3 -pady 2   
    GidHelp "$w.f4" "Insert the element sizes to be assigned to the corresponding layers and the"\
	    "general size.\nIn order this tool to work properly, geometry must be separated in the"\
	    "standard layers."

    grid conf $w.f4.eb $w.f4.ef $w.f4.eg -sticky ew
    grid columnconfigure $w.f4 1 -weight 1

    grid $w.f4 -sticky new -padx 3 -pady 3
    grid $w.f1 -sticky new -padx 3 -pady 3
    grid $w.f0 -sticky sew -padx 3 -pady 3
}

proc UseGenerateMesh {} {
    global TdynWizPriv ProblemTypePriv
    if { ![info exists ProblemTypePriv(genesize)] } {
	set ProblemTypePriv(genesize) [format {%.3g} [expr 0.1*[LayerLength "" D]]]
    } 
    if { ![info exists TdynWizPriv(bodysize)] } {
	set TdynWizPriv(bodysize) [format {%.3g} [expr 0.02*[LayerLength "body_wet body_dry" D]]]
    } 
    if { ![info exists TdynWizPriv(freesize)] } {
	set TdynWizPriv(freesize) [format {%.3g} [expr 0.04*[LayerLength "body_wet body_dry" D]]]
#        if { [info exists TdynWizPriv(velocity)] && [info exists TdynWizPriv(length)] && \
#                $TdynWizPriv(freesize)>0.0} {
#            set TdynWizPriv(freesize) [format {%.3g} \
#                    [expr abs(0.04*$TdynWizPriv(velocity)*$TdynWizPriv(velocity))]]
#        }
    }

    set w .gid.tdynwizard_assignsizes

    if { [info procs InitWindow] != "" } {
	InitWindow $w "Generate mesh window" PreGenerateMeshWindowGeom ""
    } else {
	toplevel $w
    }  
    GenerateMeshWindow $w
 
    set b $w.buts
    frame $b
    button $b.ok -text OK -width 5 -und 0 -command "DoGenerateMesh $w"
    button $b.cancel -text Cancel -width 5 -und 0 -command "destroy $w" 
    button $b.help -image [imagefromtdyn questionarrow.gif] -command "PickHelp $b.help" \
	    -highlightthickness 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $b.cancel"
    bind $b.ok <Return> "tkButtonInvoke $b.ok"
    bind $w <Escape> "tkButtonInvoke $b.cancel"
    bind $w <Alt-o> "tkButtonInvoke $b.ok"
    bind $w <Alt-c> "tkButtonInvoke $b.cancel"
    
    grid $b.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $b.cancel -row 2 -column 2 -pady 2 -sticky w 
    grid $b.help -row 2 -column 3 -pady 2 -sticky w
    grid $b -padx 3 -pady 3
    grid columnconfigure $w {0 1} -weight 1
    grid rowconfigure $w {0 1 2} -weight 1   
    wm geom $w ""
}

proc DoGenerateMesh { { w "" } } {
    global TdynWizPriv ProblemTypePriv
    if {![IsWord real+ $ProblemTypePriv(genesize) $w "In General size field."] } { return }
    if {![IsWord real $TdynWizPriv(freesize) $w "In Layer free_surf size field."] } { return }
    if {![IsWord real $TdynWizPriv(bodysize) $w "In Layer body_wet size field."] } { return }
    if { $ProblemTypePriv(genesize) == 0.0 } {
	tk_dialogRAM $w.tempwin {Error window} "General size must be greater than 0.0" error 0 OK  
	return 
    } 
    if { [IsLayerEmpty body_wet surfaces] } {
	WarnWin "In order this tool to understand your problem, the geometry must be\
		separated in different layers:\n\n 1.-Layer body_wet containing the\
		surfaces of the body within the fluid\n 2.-Layer free_surf containing\
		the free surface (if any)\n\nLayers window can be accessed through the\
		menu option Utilities>Layers.\nMesh generation may be also carried out\
		done by using the standard Meshing>Generate menu option."
	return 0 
    }
    if { $w != "" } { destroy $w }
    GraphicsOFF
    if { ![IsLayerEmpty free_surf surfaces] } { 
	AssignMeshCriteriaLayer free_surf
	if { $TdynWizPriv(freesize)>=0.0 } {
	    AssignSizeSurfacesAndLower $TdynWizPriv(freesize) \
		    [.central.s info layers -entities surfaces free_surf]
	}
    }
    if { ![IsLayerEmpty body_trn surfaces] } {
	AssignMeshCriteriaLayer body_trn
	if { $TdynWizPriv(freesize)>=0.0 } {
	    AssignSizeSurfacesAndLower $TdynWizPriv(freesize) \
		    [.central.s info layers -entities surfaces body_trn]
	}
    }
    if { ![IsLayerEmpty body_wet surfaces] } {
	AssignMeshCriteriaLayer body_wet
	if { $TdynWizPriv(bodysize)>=0.0 } {
	    AssignSizeSurfacesAndLower $TdynWizPriv(bodysize) \
		    [.central.s info layers -entities surfaces body_wet]
	}
    }
    .central.s process escape escape escape escape Meshing CancelMesh Yes escape escape
    set OldSizeTransitionsFactor [.central.s info variable SizeTransitionsFactor]
    .central.s process escape escape escape escape utilities variables \
	    SizeTransitionsFactor $ProblemTypePriv(sizetransit) escape escape
    GraphicsON
    .central.s process escape escape escape escape Meshing Generate $ProblemTypePriv(genesize)
    .central.s process escape escape escape escape utilities variables \
	    SizeTransitionsFactor $OldSizeTransitionsFactor escape escape
    return 1
}

##################################################
# SINK AND TRIM WIZARD
###################################################

proc OpenSinkTrimWizardWindow { } {
    global SATWizPriv ProblemTypePriv
 
    # create the wizard widget
    set wizwin .gid.satwizard
    if { [winfo exists $wizwin] } {
	$wizwin show
	return
    }
    catch { unset SATWizPriv }

    set dir $ProblemTypePriv(problemtypedir) 
    tkwizard::tkwizard $wizwin -icondir "$dir/images/tdynwiz.gif" -title "Sink&Trim Wizard" 
 
    GidHelp $wizwin  "This wizard can be used to automatically generate"\
	    "the geometry and data for simple problems.\nThis kind of problems must"\
	    "consist on one or several bodies within a control volume, symmetric or"\
	    "unsymmetric and with or without free surface.\nThe problem must be"\
	    "oriented in the main cartesian axes, being OX the direction of the"\
	    " free-flow and OZ+ the vertical direction. If any symmetry plane exits,"\
	    "this must be perpendicular to the OY axis."

    # define finish & cancel button action
#    bind $wizwin <<WizFinish>> {StartCalculate}
    bind $wizwin <<WizCancel>> {DestroySATWindows}

    # define wizard steps
    $wizwin step "Introduction" -title "Introduction" -layout basic { 
	global SATWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 1} \
		-subtitle {Update geometry by Sink & Trim data}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_009.html"] $w 
	DestroySATWindows
	UseFreeSReadReplace
    }  
    $wizwin step "DefineProblemData" -title "Define Data" -layout basic { 
	global SATWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 2} \
		-subtitle {Use Define data window to define boundary conditions, boundaries,\
		material properties and other data of the problem}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_003.html"] $w 
	DestroySATWindows
	UseCreateDefData
    }
    $wizwin step "GenerateMesh" -title "Generate mesh" -layout basic { 
	global SATWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 3} \
		-subtitle {Use the Generate mesh window to define the element sizes and generate \
		the mesh}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_004.html"] $w 
	DestroySATWindows
	UseGenerateMesh
    }
    $wizwin step "Calculate" -title "Calculate" -layout basic { 
	global SATWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 4} \
		-subtitle {Congratulations! The problem is ready to be solved}
	if { [.central.s info postprocess arewein]!="NO" } {
	    .central.s process escape escape escape escape preprocess yes escape
	}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_005.html"] $w 
	DestroySATWindows
	RunWin
    }
    $wizwin step "Postprocessing" -title "Postprocessing" -layout basic { 
	global SATWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 5} \
		-subtitle {Now, we will start the postprocessing process}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_006.html"] $w 
	if { [.central.s info postprocess arewein]!="YES" } {
	    .central.s process escape escape escape escape postprocess escape
	}
	DestroySATWindows
	PostDisplay
	PostResults
    }
    $wizwin step "MakingGraphs" -title "Making Graphs" -layout basic { 
	global SATWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 6} \
		-subtitle {Making graphs}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_007.html"] $w 
	DestroySATWindows
	PostGraphResultsWindow
    }
    $wizwin step "Reporting" -title "Reporting" -layout basic { 
	global SATWizPriv ProblemTypePriv
	set w [$this widget clientArea]
	$this stepconfigure \
		-title {Step 7} \
		-subtitle {Creating a report of the analysis}
	set dir $ProblemTypePriv(problemtypedir)
	HelpViewer::HelpReader [file join $dir "tdyn3dwizard/tdynwizard_008.html"] $w 
	DestroySATWindows
	UseCreateReportWindow
    }
    # show the wizard
    $wizwin show
}

proc DestroySATWindows {} {
    if { [.central.s info postprocess arewein]!="YES" } {
	 if { [winfo exists .gid.tdynsatwizard_freesread] } {
	destroy .gid.tdynsatwizard_freesread
	 }
	if { [winfo exists .gid.tdynwizard_defdata] } {
	    destroy .gid.tdynwizard_defdata
	}
	if { [winfo exists .gid.tdynwizard_assignsizes] } {
	    destroy .gid.tdynwizard_assignsizes
	}
	if { [winfo exists .wrun] } {
	    destroy .wrun
	}
    } else {
	if { [winfo exists .wPostDisplay] } {
	    destroy .wPostDisplay
	}
	if { [winfo exists .wPostResults] } {
	    destroy .wPostResults
	}
	if { [winfo exists .gid.createreport] } {
	    destroy .gid.createreport
	}
	if { [winfo exists .gid.graph] } {
	    destroy .gid.graph
	}
    }
}

##################################################
# CREACIN AUTOMTICA DEL NUEVO VOLUMEN DE CONTROL
##################################################

proc ReadSATData { w  } { 
    global SATWizPriv 

    set fileid [OpenFile "flavia.sat" $SATWizPriv(filename)]
    if { $fileid == "" } { 
	if { $w == "" } {
	    set SATWizPriv(trim) 0.0
	    set SATWizPriv(sink) 0.0
	} else {
	    tk_dialogRAM $w.tmpwin {Error window} \
		    "Cannot find correct Sink and Trim data file in $SATWizPriv(filename)" \
		    error 0 OK 
	}
	return 0
    }

    set SATWizPriv(geom_units) 1.0
    set input [gets $fileid]
    if { [regexp {Sink_and_Trim V 1.0} $input] } {
	set SATWizPriv(version) "1.0"
	while { ![eof $fileid] } {
	    set input [gets $fileid]
	    if { [regexp {Body} $input] } { 
		while { ![regexp {end body} $input] } {
		    set input [gets $fileid]
		    if { [regexp {Rotation(.*),(.*),(.*);} $input {} {} com] } {  
		        set SATWizPriv(trim) $com
		    }
		    if { [regexp {Translation(.*),(.*),(.*);} $input {} {} {} com] } {
		        set SATWizPriv(sink) $com
		    }
		}
	    }
	}
    } elseif { [regexp {Sink_and_Trim V 2.0} $input] } {
	set SATWizPriv(version) "2.0"
	set idx 0
	while { ![eof $fileid] } {
	    set input [gets $fileid]
	    if { [regexp {Body(.*);} $input {} name] } { 
		incr idx
		set SATWizPriv(bodylist,$idx) $name
		while { ![regexp {end body} $input] } {
		    set input [gets $fileid]
		    if { [regexp {Forces(.*),(.*),(.*);} $input {} comx comy comz] } {  
		        set SATWizPriv(force,$idx) [list $comx $comy $comz]
		    } elseif { [regexp {Moments(.*),(.*),(.*);} $input {} comx comy comz] } {
		        set SATWizPriv(moment,$idx) [list $comx $comy $comz]
		    } elseif { [regexp {CdeG(.*),(.*),(.*);} $input {} comx comy comz] } {
		        set SATWizPriv(xcdeg,$idx) [list $comx $comy $comz]
		    } elseif { [regexp {Inertia(.*),(.*),(.*);} $input {} comx comy comz] } {
		        set SATWizPriv(inertia,$idx) [list $comx $comy $comz]
		    } elseif { [regexp {PArea(.*),(.*),(.*);} $input {} comx comy comz] } {
		        set SATWizPriv(parea,$idx) [list $comx $comy $comz]
		    } elseif { [regexp {Displacement(.*);} $input {} comx] } {
		        set SATWizPriv(displacement,$idx) $comx
		    } elseif { [regexp {PressureOrigin} $input {} comx] } {
		        set SATWizPriv(presorigin) $comx
		    }         
		}
	    } elseif { [regexp {UnitsFactor(.*);} $input {} data] } { 
		set SATWizPriv(geom_units) $data
	    }
	}
	set SATWizPriv(numbody) $idx
	CalculateChrfromData 
	grid $w.f2.ls1
	grid $w.f2.ls2 $w.f2.es2 $w.f2.ls2b
	grid $w.f2.ls3 $w.f2.es3 $w.f2.ls3b
	grid $w.f2.ls4 $w.f2.es4 $w.f2.ls4b
    } else {
	tk_dialogRAM $w.tmpwin {Error window} \
		"Sink and Trim data file format not correct" \
		error 0 OK 
	return 0
    }
    close $fileid
}

proc ReadSATFileWindow { w } {
    global SATWizPriv
    # Seleccin del archivo de datos
    set SATWizPriv(filename) [Browser-ramR project read $w {Read previous results}] 
    if { $SATWizPriv(filename) == "" } { return }
    # Cargo los datos del nuevo problema
#    GraphicsOFF
#    .central.s process escape escape escape files save
#    set ProjectName [CopyFile cnd $SATWizPriv(filename)]
#    CopyFile mat $SATWizPriv(filename) $ProjectName
#    CopyFile prb $SATWizPriv(filename) $ProjectName
#    .central.s process escape escape escape files read $ProjectName
#    .central.s process escape escape escape
#    GraphicsON
    # Se leen el resto de datos del archivo *.sat
    ReadSATData $w
}

proc CalculateChrfromData { } {
    global SATWizPriv
    if { $SATWizPriv(version)=="1.0" } { return }

    # Calculating displacement
    set SATWizPriv(displacement) 0.0
    for { set i 1 } { $i <=  $SATWizPriv(numbody) } { incr i } {
	set SATWizPriv(displacement) [format {%.4g} \
		[expr $SATWizPriv(displacement)+$SATWizPriv(displacement,$i)]]
    }
    # Calculating vertforce
    set SATWizPriv(vertforce) 0.0
    for { set i 1 } { $i <=  $SATWizPriv(numbody) } { incr i } {
	set SATWizPriv(vertforce) [format {%.4g} \
		[expr $SATWizPriv(vertforce)+[lindex $SATWizPriv(force,$i) 2]]]
    }
    # Calculating trimmoment
    set SATWizPriv(trimmoment) 0.0
    for { set i 1 } { $i <=  $SATWizPriv(numbody) } { incr i } {
	set SATWizPriv(trimmoment) [format {%.4g} \
		[expr $SATWizPriv(trimmoment)+[lindex $SATWizPriv(moment,$i) 1]]]
    }
    # Calculate totalarea
    set SATWizPriv(totalarea) 0.0
    for { set i 1 } { $i <=  $SATWizPriv(numbody) } { incr i } {
	set SATWizPriv(totalarea) [format {%.4g} \
		[expr $SATWizPriv(totalarea)+[lindex $SATWizPriv(parea,$i) 2]]]
    }
    # Calculate totinertia
    set SATWizPriv(totinertia) 0.0
    for { set i 1 } { $i <=  $SATWizPriv(numbody) } { incr i } {
	set SATWizPriv(totinertia) [format {%.4g} \
		[expr $SATWizPriv(totinertia)+[lindex $SATWizPriv(inertia,$i) 1]]]
    }
    CalculateSATfromData
}
proc CalculateSATfromData { } {
    global SATWizPriv
    if { $SATWizPriv(version)=="1.0" } { return }
    if { $SATWizPriv(totalarea)<=0 || $SATWizPriv(totinertia)<=0 } {
	tk_dialogRAM .gid.tmpwin {Error window} \
		"Wrong data. Cannot calculate Sink&Trim." error 0 OK 
	return 0
    }
    # Calculate sinkage
    set SATWizPriv(sink) [format {%.4g} [expr ($SATWizPriv(vertforce)-\
	    $SATWizPriv(displacement))/$SATWizPriv(totalarea)*$SATWizPriv(geom_units)]]
    # Calculate trim angle
    set SATWizPriv(trim) [format {%.4g} [expr 57.295779513*$SATWizPriv(trimmoment)\
	    /$SATWizPriv(totinertia)]]
}

proc FreeSReadReplaceWindow { w } {
    global GIDDEFAULT SATWizPriv ReadbPriv

    frame $w.f1
    label $w.l -text "File:" 
    entry $w.file -relief sunken -textvariable SATWizPriv(filename)

    bind $w.file <Return> "ReadSATData $w"

    button $w.fileb -image [imagefromtdyn openfold.gif] \
	    -command "ReadSATFileWindow $w"

    grid columnconf $w.f1 1 -weight 1 
    grid  $w.l $w.file $w.fileb -in $w.f1 -sticky ew 

    frame $w.f2 -relief ridge -bd 2 
    label $w.f2.lg1 -text "Geometry data" -font normal
    label $w.f2.lg2 -text "Read hull geometry: "
    checkbutton $w.f2.c2 -state normal -variable SATWizPriv(readhull)
    label $w.f2.lg3 -text "Read free surface: "
    checkbutton $w.f2.c3 -state active -variable SATWizPriv(readfrees) 
    label $w.f2.ls1 -text "Calculation data" -font normal
    label $w.f2.ls2 -text "Displacement / r*g: "
    entry $w.f2.es2 -relief sunken -width 8 -textvariable SATWizPriv(displacement)
    label $w.f2.ls2b -text "m3"
    label $w.f2.ls3 -text "Vertical force / r*g: "
    entry $w.f2.es3 -relief sunken -width 8 -textvariable SATWizPriv(vertforce)  
    label $w.f2.ls3b -text "m3"
    label $w.f2.ls4 -text "Trim moment / r*g: "
    entry $w.f2.es4 -relief sunken -width 8 -textvariable SATWizPriv(trimmoment) 
    label $w.f2.ls4b -text "m4"
    label $w.f2.l1 -text "Sink & trim data" -font normal
    label $w.f2.l2 -text "Sinkage: "
    entry $w.f2.e2 -relief sunken -width 8 -textvariable SATWizPriv(sink)
    label $w.f2.l3 -text "Trim angle (): "
    entry $w.f2.e3 -relief sunken -width 8 -textvariable SATWizPriv(trim)
    button $w.f2.ap -text Apply -width 5 -und 0 -command "DoFreeSReadReplace $w"
    GidHelp $w.f2.lg2 $w.f2.c2 "Check this button to read the hull geometry of the selected file."
    GidHelp $w.f2.lg3 $w.f2.c3 "Check this button to read the free surface geometry of the selected file."
    GidHelp $w.f2.l2  "Dynamic sinkage of the ship,\
	    defining displacement in the OZ direction.\nGiven in geometry units."
    GidHelp $w.f2.l3  "Dynamic trim angle, defined by the rotation around OY axis.\nGiven in degrees."

    grid columnconf $w.f2 1 -weight 1 
    grid $w.f2.lg1 -padx 3 -pady 2 -columnspan 2
    grid $w.f2.lg2 $w.f2.c2 -sticky w -padx 3 -pady 2
    grid $w.f2.lg3 $w.f2.c3 -sticky w -padx 3 -pady 2
    grid $w.f2.ls1 -padx 3 -pady 2 -columnspan 2
    grid $w.f2.ls2 $w.f2.es2 $w.f2.ls2b -sticky w -padx 3 -pady 2
    grid $w.f2.ls3 $w.f2.es3 $w.f2.ls3b -sticky w -padx 3 -pady 2  
    grid $w.f2.ls4 $w.f2.es4 $w.f2.ls4b -sticky w -padx 3 -pady 2  
    grid $w.f2.l1 -padx 3 -pady 2 -columnspan 2
    grid $w.f2.l2 $w.f2.e2 -sticky w -padx 3 -pady 2
    grid $w.f2.l3 $w.f2.e3 -sticky w -padx 3 -pady 2  
    grid $w.f2.ap  -pady 2 -columnspan 2
    grid remove $w.f2.ls1
    grid remove $w.f2.ls2 $w.f2.es2 $w.f2.ls2b
    grid remove $w.f2.ls3 $w.f2.es3 $w.f2.ls3b
    grid remove $w.f2.ls4 $w.f2.es4 $w.f2.ls4b
 
    bind  $w.f2.es2 <Return> "CalculateSATfromData"
    bind  $w.f2.es3 <Return> "CalculateSATfromData"
    bind  $w.f2.es4 <Return> "CalculateSATfromData"

    grid $w.f1 -sticky ew
    grid $w.f2 -sticky ew
}

proc UseFreeSReadReplace { } {
    global SATWizPriv
   
    set w .gid.tdynsatwizard_freesread
    set ProjectName [lindex [.central.s info Project] 1]

    set SATWizPriv(sink) 0.0
    set SATWizPriv(trim) 0.0

    if { [info procs InitWindow] != "" } {
	InitWindow $w "Read previous data" PreFreeSReadWindowGeom ""
    } else {
	toplevel $w
    }
    set SATWizPriv(readhull) 0
    set SATWizPriv(readfrees) 1
    
#    if { [info exists SATWizPriv(filename)] } { 
#        if { $SATWizPriv(filename) != "" } ReadSATData $w
#    } 
#    elseif { $ProjectName != "UNNAMED"  } {
#        set SATWizPriv(filename) $ProjectName
#        ReadSATData $w
#    }
    FreeSReadReplaceWindow $w

    set b $w.buts
    frame $b
    button $b.ok -text OK -width 5 -und 0 -command "DoNewCreateCompBox $w"
    button $b.cancel -text Cancel -width 5 -und 0 -command "destroy $w" 
    button $b.help -image [imagefromtdyn questionarrow.gif] -command "PickHelp $b.help" \
	    -highlightthickness 0

    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $b.cancel"
    bind $b.ok <Return> "tkButtonInvoke $b.ok"
    bind $w <Escape> "tkButtonInvoke $b.cancel"
    bind $w <Alt-o> "tkButtonInvoke $b.ok"
    bind $w <Alt-c> "tkButtonInvoke $b.cancel"
    
    grid $b.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $b.cancel -row 2 -column 2 -pady 2 -sticky w 
    grid $b.help -row 2 -column 3 -pady 2 -sticky w
    grid $b -padx 3 -pady 3
    grid rowconfigure $w {1 2 3} -weight 1
    grid columnconfigure $w 0 -weight 1
    # almaceno el tamao de la ventana
    wm geom $w ""
}

proc DoFreeSReadReplace { { w "" } } {
    global SATWizPriv TdynWizPriv ProblemTypePriv

    if { ![info exists SATWizPriv(filename)] || $SATWizPriv(filename)==""} {
	tk_dialogRAM .gid.tmpwin {Error window} \
		"Input file must be selected" error 0 OK 
	return 0
    } 
    set fileid [OpenFile "flavia.sat" $SATWizPriv(filename)]
    if { $fileid == "" } { 
	tk_dialogRAM .gid.tmpwin {Error window} \
		"Error in input file" error 0 OK 
	return 0
    }
    WarnWin "This action may take several minutes"
#    if { $w != "" } { destroy $w }
    GraphicsOFF
    # Se cancela la malla que pueda existir y se eliminan capas intiles
    .central.s process escape escape escape escape Meshing CancelMesh Yes
    DeleteLayer {volume free_surf symm_oy inlet_ox outlet_ox lateral_oy lateral_oz tfree_surf} 1 
    # Se lee el modelo si as se indica
    if { $SATWizPriv(readhull)!= 0 } {
	DeleteLayer {body_wet body_dry} 1
	.central.s process escape escape escape escape files insertgeom \
		\"$SATWizPriv(filename)\" escape
	DeleteLayer {volume free_surf symm_oy inlet_ox outlet_ox lateral_oy lateral_oz tfree_surf} 1 
    }
    # Sink and trim
    MoveToLayer body_dry body_wet
    if { [IsLayerEmpty body_wet] } { 
	GraphicsON
	tk_dialogRAM .gid.tmpwin {Error window} \
		"Geometry is not correct. body_wet layer doesn't exist." error 0 OK 
	return 0
    }
    if { [IsLayerEmpty body_dry] } { CreateLayer body_dry }
    if { [info exists SATWizPriv(xcdeg,1)] } {
	set xcenter [lindex $SATWizPriv(xcdeg,1) 0]
	.central.s process escape escape escape escape utilities move surfaces rotation \
		$xcenter 0.0 0.0 $xcenter 1.0 0.0 $SATWizPriv(trim) layer:body_wet layer:body_dry escape
	.central.s process escape escape escape escape utilities move surfaces translation \
		0.0 0.0 0.0 0.0 0.0 $SATWizPriv(sink) layer:body_wet layer:body_dry escape 
    }
    # Free Surface
    if { $SATWizPriv(readfrees)!= 0 } {
	if { $SATWizPriv(readfrees)!= 0 } {
	    ToUseLayer free_surf
	    CreateFreeSurfaceFromPrevious $fileid
	}
    }
    close $fileid
    # Calcula la lista de puntos y lneas extremos de la superficie libre
    GraphicsOFF
    set retval [.central.s info list_entities surfaces layer:free_surf] 
    set lines [regexp -linestop -all -inline {Line:([ 0-9]*)} $retval]
    set SATWizPriv(fspoints) ""
    set SATWizPriv(fslines) ""
    for { set i 1 } { $i < [llength $lines] } { incr i 2 } {
	lappend SATWizPriv(fslines) [set line [lindex $lines $i]]
	set retval [.central.s info list_entities lines $line] 
	set points [lindex [regexp -linestop -inline {Points:([ 0-9]*)} $retval] 1]
	set pnt1 [lindex $points 0]
	set pnt2 [lindex $points 1]
	if { [lsearch $SATWizPriv(fspoints) $pnt1]==-1 } {
	    lappend SATWizPriv(fspoints) $pnt1
	}
	if { [lsearch $SATWizPriv(fspoints) $pnt2]==-1 } {
	    lappend SATWizPriv(fspoints) $pnt2
	}
    }
    # Crea la interseccin (primero se busca la superficie libre para borrar la que no sirve)  
    set SATWizPriv(fsline) 0
    .central.s process escape escape escape escape SelectEntities JustOne Surfaces\
	    layer:free_surf escape
    trace var GidPriv(selection) w [SearchFSLines]  
    GraphicsON
    if { [IsLayerEmpty body_trn surfaces] } {
	.central.s process escape escape escape geometry create intmultsurfs\
		layer:free_surf layer:body_dry layer:body_wet escape
    } else {
	.central.s process escape escape escape geometry create intmultsurfs\
		layer:free_surf layer:body_dry layer:body_wet layer:body_trn escape
    }
    GraphicsOFF
    .central.s process escape escape escape escape SelectEntities Surfaces\
	    layer:free_surf escape
    trace var GidPriv(selection) w [DeleteNoFS]
    # Organiza la geometra resultante en capas
    if { [IsLayerEmpty body_trn surfaces] } {
	DryAndWet
    } else {  
	set transurf [.central.s info layers -entities surfaces body_trn]
	MoveToLayer body_trn body_dry 
	DryAndWet
	for { set i 0 } { $i < [llength $transurf] } { incr i } {
	    set surface [lindex $transurf $i]
	    if { [GiveLayer Surfaces $surface] == "body_wet" } {
		.central.s process escape escape escape escape utilities layer\
		        entities body_trn surfaces $surface escape
	    } else {
		.central.s process escape escape escape escape geometry delete\
		        surface $surface escape
	    }
	}
    }
    GraphicsON
    WarnWin "Check the results of the process and press OK button"
}

proc DoNewCreateCompBox { { w "" } } { 
    global SATWizPriv BoxInfo ProblemTypePriv

    # Activa el volcado de informacin del proceso
    # set SATWizPriv(echo_on) 1

    if { ![info exists SATWizPriv(filename)] } {
	tk_dialogRAM .gid.tmpwin {Error window} \
		"Input file must be selected" error 0 OK 
	return 0
    } 
    set fileid [OpenFile "flavia.sat" $SATWizPriv(filename)]
    if { $fileid == "" } { 
	tk_dialogRAM .gid.tmpwin {Error window} \
		"Cannot open Sink & Trim (*.sat) input file" error 0 OK 
	return 0
    }   
    # Read box data and generate points
    while { ![eof $fileid] } {
	set input [gets $fileid]
	if { [regexp {VolumeData} $input] } {
	    for { set i 1 } { $i <= 8 } { incr i 1 } {
		set BoxInfo($i) [lindex [regexp -inline "(.*);" [gets $fileid]] 1];
	    }
	}
    }
    close $fileid
    if { ![info exists BoxInfo(8)] } {
	tk_dialogRAM .gid.tmpwin {Error window} \
		"Error reading Sink & Trim (*.sat) input file" error 0 OK 
	return 0
    }
    GraphicsOFF
    if { [info exists SATWizPriv(echo_on)] } { GraphicsON }
    # Se inicia el proceso de creacin del volumen 
    DeleteLayer {tmp1 tinlet_ox tlateral_oy tlateral_oz tsymm_oy toutlet_ox tvolume} 1
    CreateLayer tlateral_oz
    set SATWizPriv(tolerance) [expr 1.0e-5*[Distance $BoxInfo(2) $BoxInfo(3)]]
    set tmpHeight [expr 0.25*[Distance $BoxInfo(1) $BoxInfo(5)]]
    for { set i 1 } { $i <= 4 } { incr i 1 } {
	set BoxIndex($i) [SearchPointInOZCylinder $BoxInfo($i) \
		$SATWizPriv(tolerance) free_surf $tmpHeight]
	if { [llength $BoxIndex($i)] > 1 } {
	    GraphicsON
	    tk_dialogRAM .gid.tmpwin {Error window} \
		    "Error in free_surf layer. Delete extra lines or points." error 0 OK 
	    return 0
	} elseif { [llength $BoxIndex($i)] == 0 } {
	    GraphicsON
	    tk_dialogRAM .gid.tmpwin {Error window} \
		    "Error in free_surf layer.\
		    Cannot find extreme point near $BoxInfo($i)." error 0 OK 
	    return 0
	}
	set BoxInfo($i) [.central.s info Coordinates $BoxIndex($i)] 
    }
    for { set i 5 } { $i <= 8 } { incr i 1 } {
	.central.s process escape escape escape escape geometry create point $BoxInfo($i) old
	set BoxIndex($i) [SearchPointInSphere $BoxInfo($i) $SATWizPriv(tolerance)]
    }
    # Se toma la decisin de qu lateral es el simtrico
    set SATWizPriv(symm_oy) ""
    if { ![IsLayerEmpty symm_oy lines] } { 
	set SATWizPriv(symm_oy) [LayerLength symm_oy YD]
	set dif_symm [expr abs([lindex [lindex $BoxInfo(1) 0] 1]-[lindex $SATWizPriv(symm_oy) 0])]
	set tol_symm [expr [lindex $SATWizPriv(symm_oy) 1]+$SATWizPriv(tolerance)]
	if { $dif_symm>$tol_symm } { 
	    set tmp $BoxInfo(1); set BoxInfo(1) $BoxInfo(2); set BoxInfo(2) $tmp;
	    set tmp $BoxInfo(4); set BoxInfo(4) $BoxInfo(3); set BoxInfo(3) $tmp;
	    set tmp $BoxInfo(5); set BoxInfo(5) $BoxInfo(6); set BoxInfo(6) $tmp;
	    set tmp $BoxInfo(8); set BoxInfo(8) $BoxInfo(7); set BoxInfo(7) $tmp;
	    set tmp $BoxIndex(1); set BoxIndex(1) $BoxIndex(2); set BoxIndex(2) $tmp;
	    set tmp $BoxIndex(4); set BoxIndex(4) $BoxIndex(3); set BoxIndex(3) $tmp;
	    set tmp $BoxIndex(5); set BoxIndex(5) $BoxIndex(6); set BoxIndex(6) $tmp;
	    set tmp $BoxIndex(8); set BoxIndex(8) $BoxIndex(7); set BoxIndex(7) $tmp;
	}
	if { [info exists SATWizPriv(echo_on)] } { 
	    WarnWin "Simetra OY en plano y = [lindex $SATWizPriv(symm_oy) 0]" 
	}
    }
    # inlet
    CreateLayer tinlet_ox 1
    .central.s process escape escape escape escape geometry create line join \
	    $BoxIndex(1) $BoxIndex(5) $BoxIndex(6) $BoxIndex(2)
    .central.s process escape escape escape escape geometry create nurbssurface \
	    [SearchLine $BoxIndex(1) $BoxIndex(2) free_surf] layer:tinlet_ox escape
    if { [info exists SATWizPriv(echo_on)] } { WarnWin "Creado el inlet" }
    # lateral_oy
    CreateLayer tlateral_oy 1
    .central.s process escape escape escape escape geometry create line join \
	    $BoxIndex(6) $BoxIndex(7) $BoxIndex(3)
    .central.s process escape escape escape escape geometry create nurbssurface \
	    [SearchLine $BoxIndex(2) $BoxIndex(3) free_surf] \
	    [SearchLine $BoxIndex(2) $BoxIndex(6) tinlet_ox] layer:tlateral_oy escape 
    if { [info exists SATWizPriv(echo_on)] } { WarnWin "Creado el lateral_oy" }
    # outlet
    CreateLayer toutlet_ox 1
    .central.s process escape escape escape escape geometry create line join \
	    $BoxIndex(7) $BoxIndex(8) $BoxIndex(4)
    .central.s process escape escape escape escape geometry create nurbssurface \
	    [SearchLine $BoxIndex(3) $BoxIndex(4) free_surf] \
	    [SearchLine $BoxIndex(3) $BoxIndex(7) tlateral_oy] layer:toutlet_ox escape 
    if { [info exists SATWizPriv(echo_on)] } { WarnWin "Creado el outlet" }
    # lateral_oz
    ToUseLayer tlateral_oz
    .central.s process escape escape escape escape geometry create line join \
	    $BoxIndex(5) $BoxIndex(8)
    .central.s process escape escape escape escape geometry create nurbssurface \
	    [SearchLine $BoxIndex(6) $BoxIndex(7) tlateral_oy] \
	    [SearchLine $BoxIndex(7) $BoxIndex(8) toutlet_ox] \
	    [SearchLine $BoxIndex(5) $BoxIndex(6) tinlet_ox] layer:tlateral_oz escape 
    if { [info exists SATWizPriv(echo_on)] } { WarnWin "Creado el lateral_oz" }
    # lateral_oy o symm_oy
    if { [IsLayerEmpty symm_oy lines] } {
	ToUseLayer tlateral_oy
	.central.s process escape escape escape escape geometry create nurbssurface \
		[SearchLine $BoxIndex(1) $BoxIndex(4) free_surf] \
		[SearchLine $BoxIndex(5) $BoxIndex(8) tlateral_oz] \
		[SearchLine $BoxIndex(4) $BoxIndex(8) toutlet_ox] \
		[SearchLine $BoxIndex(1) $BoxIndex(5) tinlet_ox] escape
    } else {
	CreateLayer tsymm_oy 1
	set sy_lines ""
	set lines [.central.s info list_entities lines layer:free_surf]
	set lines [regexp -linestop -all -inline {Num: ([0-9]*)} $lines]
	for { set i 1 } { $i < [llength $lines] } { incr i 2 } {
	    set line [lindex $lines $i]
	    if { $line > 0 } {
		set retval [.central.s info list_entities lines $line] 
		set points [lindex [regexp -linestop -inline {Points: ([ 0-9]*)} $retval] 1]
		set point1 [lindex [.central.s info Coordinates [lindex $points 0]] 0]
		set point2 [lindex [.central.s info Coordinates [lindex $points 1]] 0]
		set pointr [lindex $BoxInfo(1) 0]
		if { [lsearch $points $BoxIndex(1)]!=-1 || [lsearch $points $BoxIndex(4)]!=-1 } {
		    if {[expr abs(double([lindex $point1 1])-[lindex $pointr 1])]<$SATWizPriv(tolerance) && \
		            [expr abs(double([lindex $point2 1])-[lindex $pointr 1])]<$SATWizPriv(tolerance) } {
		        lappend sy_lines $line
		    }
		}
	    }
	}
	.central.s process escape escape escape escape geometry create nurbssurface $sy_lines \
		[SearchLine $BoxIndex(5) $BoxIndex(8) tlateral_oz] \
		[SearchLine $BoxIndex(4) $BoxIndex(8) toutlet_ox] \
		[SearchLine $BoxIndex(1) $BoxIndex(5) tinlet_ox] layer:symm_oy escape
    }
    if { [info exists SATWizPriv(echo_on)] } { WarnWin "Creadas todas las superficies" }
    # tvolume
    CreateLayer tvolume 1  
#    .central.s process escape escape escape escape utilities collapse model Yes escape
    if { [IsLayerEmpty body_trn surfaces] } {
	.central.s process escape escape escape escape geometry create volume layer:tinlet_ox\
		layer:toutlet_ox layer:tlateral_oy layer:tlateral_oz layer:tsymm_oy\
		layer:free_surf layer:body_wet escape
    } else {
	.central.s process escape escape escape escape geometry create volume layer:tinlet_ox\
		layer:toutlet_ox layer:tlateral_oy layer:tlateral_oz layer:tsymm_oy\
		layer:free_surf layer:body_wet layer:body_trn escape
    }
    if { [info exists SATWizPriv(echo_on)] } { WarnWin "Creado el volumen" }
    # Se comprueba si se ha creado el volumen y se borra todo lo innecesario
    set check_volume [.central.s info list_entities volume layer:tvolume]
    if { $check_volume != "" } { 
	MoveToLayer tfree_surf free_surf
	MoveToLayer tvolume volume 
	MoveToLayer tsymm_oy symm_oy
	MoveToLayer tinlet_ox inlet_ox
	MoveToLayer toutlet_ox outlet_ox
	MoveToLayer tlateral_oy lateral_oy
	MoveToLayer tlateral_oz lateral_oz
	MoveLowerToLayer free_surf
	MoveLowerToLayer body_wet
	DeleteLayer {tmp1 tmp2 tmp3} 1
	DeleteLayer {tvolume tfree_surf tsymm_oy tinlet_ox toutlet_ox tlateral_oy tlateral_oz} 1
	.central.s process escape escape escape escape
	GraphicsON
	return 1 
    }
    DeleteLayer {tmp1 tmp2 tmp3} 1
    DeleteLayer {tvolume tfree_surf tsymm_oy tinlet_ox toutlet_ox tlateral_oy tlateral_oz} 1
    GraphicsON
    tk_dialogRAM .gid.tempwin {Error window} "Error creating control volume." error 0 OK  
    return 0
}
#
#   Utilidad para intersectar la geometra del barco por un plano z = 0
#
proc IntersectPlanarFreeSurface { } {
    # Desactivo la ventana
    GraphicsOFF
    # Muevo todo lo que hay a la capa body_wet
    ToUseLayer body_wet
    set layer_list [.central.s info layers]
    foreach layer [lrange $layer_list 0 end] {
	MoveToLayer $layer body_wet
    }
    # Creo una superficie libre temporal
    ToUseLayer free_surf
    set tol [expr 0.3*[LayerLength]]
    set xmin [expr [LayerLength "all" MINX]-$tol]
    set xmax [expr [LayerLength "all" MAXX]+$tol]
    set ymin [expr [LayerLength "all" MINY]-$tol]
    set ymax [expr [LayerLength "all" MAXY]+$tol]
    .central.s process escape escape escape escape geometry create object rectangle\
	    $xmax $ymax 0.0 $xmin $ymin 0.0
    # Intersecto la geometra
    .central.s process escape escape escape geometry create intmultsurfs layer:free_surf\
	    layer:body_wet escape
    # Organiza la geometra resultante en capas
    DryAndWet    
    # Borramos la geometra sobrante
    DeleteLayer free_surf 1
    .central.s process escape escape escape
    # Activo la ventana
    GraphicsON
}
#
#   Abre una listbox con la lista que se enva y devuelve los elementos seleccionados
#
proc CreateSATLayers { listnames { w .gid.wizlay} } {
    global TdynWizPriv

    InitWindow $w [_ Select Sink&Trim Layers] SATLayersWindowGeom ""
    set TdynWizPriv(laywin) $w

    label $w.title -text [_ Name]
    frame $w.txsel -relief ridge -bd 2

    set list $w.txsel
#    text $list.tx -width 7 -height 7 -borderwidth 0 -relief flat \
#            -spacing1 1 -spacing2 1 -spacing3 1.5 -exportselection false \
#            -selectborderwidth 0 -background [$list cget -background]

#    $list.tx conf -state disabled
#    set defback [ $list.tx cget -background]
#    $list.tx configure -selectbackground $defback
    set listbox [listbox $list.lb -xscrollc "$list.xs set" -yscrollc "$list.ys set" \
	    -height 5 -width 0 -borderwidth 0 -relief flat \
	    -selectmode extended -highlightthickness 0]
    set sy [scrollbar $list.ys -orient vertical -comm "$list.lb yview"]
    set sx [scrollbar $list.xs -orient horizontal -comm "$list.lb xview"]
    
    set TdynWizPriv(listbox) $listbox
    set TdynWizPriv(sx) $sx
    set TdynWizPriv(sy) $sy

#    set LayerNames [.central.s info layers]
#    set LayerNames [lsort -dictionary $LayerNames]
#    foreach layer $LayerNames {
#        $list.lb insert end $layer
#    }

    set listnames [lsort -dictionary $listnames]
    foreach layer $listnames {
	$list.lb insert end $layer
    }
    set TdynWizPriv(laynames) $listnames

#    bind $list.tx <ButtonPress-1> "event generate $list.lb <ButtonPress-1> -y %y -x 1"
#    bind $list.tx <ButtonRelease-1> "event generate $list.lb <ButtonRelease-1> -y %y -x 1" 
#    bind $list.tx <B1-Motion> "event generate $list.lb <B1-Motion> -y %y -x 1" 
#    bind $list.tx <Control-Button-1> "event generate $list.lb <Control-Button-1> -y %y -x 1"
#    bind $list.tx <Shift-Button-1> "event generate $list.lb <Shift-Button-1> -y %y -x 1"

 #   bind $listbox <Configure> "LayersConfigureListBox"

    set b $w.buts
    frame $b
    button $b.ok -text OK -width 5 -und 0 -command "SATLayersReturn"

 #   bind $b.ok <Return> "tkButtonInvoke $b.ok"
 #   bind $w <Escape> "tkButtonInvoke $b.ok"
    wm protocol $w WM_DELETE_WINDOW "tkButtonInvoke $b.ok" 

#    grid $list.tx -row 0 -column 0 -sticky ns
    grid $list.lb -row 0 -column 0 -sticky nw -pady 4 -padx 2
    grid $list.ys -row 0 -column 1 -sticky ens
    grid $list.xs -row 1 -column 0 -columnspan 2 -sticky ew

    grid columnconfigure $list 0 -weight 1
    grid rowconfigure $list 0 -weight 1

    grid $w.title -row 0 -column 0 -sticky sw
    grid $w.txsel -row 1 -column 0 -sticky nesw
    grid $b.ok -row 2 -column 1 -sticky e -pady 3 -padx 3
    grid $b -padx 3 -pady 3

    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 1 -weight 1

    bind $listbox <Configure> "SATLayersConfigureListBox" 
    grid remove $list.ys
    grid remove $list.xs
    SATLayersConfigureListBox
}
#
#   Configura las barras de Scroll (las esconde cuando no hacen falta)
#
proc SATLayersConfigureListBox { } {
    global TdynWizPriv
    set listbox $TdynWizPriv(listbox)
    set sx $TdynWizPriv(sx)
    set sy $TdynWizPriv(sy)

    foreach i "x y" {
	if { ![info exists s${i}] || ![winfo exists [set s${i}]] } { continue }
	foreach "${i}1 ${i}2" [$listbox ${i}view] break
	
	if { [set ${i}1] == 0 && [set ${i}2] == 1 } {
	    after idle grid remove [set s${i}]
	} else {
	    after idle grid [set s${i}]
	}
    }
}
#
#   Devuelve los indices de la lista
#
proc SATLayersReturn { } {
    global TdynWizPriv
    set listbox $TdynWizPriv(listbox)

    set ret ""
    foreach i [$listbox curselection] {
	lappend ret [lindex $TdynWizPriv(laynames) $i]
    }
    destroy $TdynWizPriv(laywin)
    return ret
}

proc SearchFSLines { } {
    global GidPriv SATWizPriv
    set num [lindex [lindex $GidPriv(selection) 0] 1]
    set nnum ""
    regexp {[0-9]*} $num nnum
    if { $nnum != "" } {
	set retval [.central.s info list_entities Surfaces $nnum]
	set SATWizPriv(fsline) [regexp -all -inline {Line:[ 0-9]*} $retval]
    }        
}

proc GiveLineNumber { } {
    global GidPriv SATWizPriv

    set num [lindex [lindex $GidPriv(selection) 0] 1]
    set SATWizPriv(linenum) 0
    regexp {[0-9]*} $num SATWizPriv(linenum)
}

proc DeleteNoFS { } {
    global GidPriv SATWizPriv
    set nums [lindex $GidPriv(selection) 0]
    set nums [regexp -all -inline {[0-9]*} $nums]
    foreach num [lrange $nums 0 end] {
	regexp {[0-9]*} $num num
	if { $num != "" } {
	    set retval [.central.s info list_entities Surfaces $num]
	    set thislines [regexp -all -inline {Line:[ 0-9]*} $retval] 
	    set deleteline 1
	    foreach thisline [lrange $thislines 0 end] {
		if { [lsearch $SATWizPriv(fsline) $thisline]!=-1 } {
		    set deleteline 0
		    break
		}
	    }
	    if { $deleteline==1 } {
		.central.s process escape escape escape geometry delete \
		        surface $num escape
	    }
	}
    }
    .central.s process escape escape escape geometry delete line \
	    layer:free_surf escape
}
#
#  Calcula que entidades son de body_dry y cuales de body_wet
#  No funciona en el caso simtrico
#
proc DryAndWet {} {
    # Buscamos las lneas y los puntos de la lnea de flotacin
    if { ![ExistsLayer free_surf] } { CreateLayer free_surf }
    if { ![ExistsLayer body_wet]  } { CreateLayer body_wet  }
    if { ![ExistsLayer body_dry]  } { CreateLayer body_dry  }
    set fsurface [.central.s info layers -entities surfaces free_surf]
    if { $fsurface=="" } { return 0 }
    set fsurface [eval .central.s info list_entities surfaces $fsurface]
    set fs_lines "" 
    set fl_lines "" 
    set bd_lines ""
    set fl_points ""
    # Se almacenan en fs_lines los id de las lneas de la superficie libre
    set lines [regexp -linestop -all -inline {Line:([ 0-9]*)} $fsurface]
    for { set i 1 } { $i < [llength $lines] } { incr i 2 } {
	set line [lindex [lindex $lines $i] 0]
	lappend fs_lines $line
    }
    set bsurface [.central.s info layers -entities surfaces body_wet]
    if { $bsurface=="" } { return 0 }
    set bsurface [concat $bsurface [.central.s info layers -entities surfaces body_dry]]
    set bsurface [eval .central.s info list_entities surfaces $bsurface]
    set lines [regexp -linestop -all -inline {Line:([ 0-9]*)} $bsurface]
    for { set i 1 } { $i < [llength $lines] } { incr i 2 } {
	set line [lindex [lindex $lines $i] 0]
	if { [lsearch $fs_lines $line ]!=-1 && [lsearch $fl_lines $line ]==-1} {
	     lappend fl_lines $line
	}
	if { [lsearch $bd_lines $line ]==-1 && [lsearch $fl_lines $line ]==-1 } {
	    lappend bd_lines $line 
	}
    }   
    set fl_points [ListPointsFromLines $fl_lines]
    # Creo una lista de pares de puntos de las lneas simples de la flotacin para
    # que funcione el caso de polilneas
    set fl_cpoint ""
    for { set i 0 } { $i < [llength $fl_lines] } { incr i } {
	set line [lindex [lindex $fl_lines $i] 0]
	set retval [.central.s info list_entities lines $line] 
	set points [regexp -linestop -all -inline {Points:([ 0-9]*)} $retval]
	set len [llength $points]
	set ini 1
	if { $len > 2 } { set ini 3 }
	for { set j $ini } { $j < $len } { incr j 2 } {
	    set pnt1 [lindex [lindex $points $j] 0]
	    set pnt2 [lindex [lindex $points $j] 1]
	    if { [lsearch $fl_cpoint $pnt1]==-1 } {
		lappend fl_cpoint $pnt1
		set fl_couple($pnt1) ""
	    }
	    if { [lsearch $fl_cpoint $pnt2]==-1 } {
		lappend fl_cpoint $pnt2
		set fl_couple($pnt2) ""
	    }
	    lappend fl_couple($pnt2) $pnt1
	    lappend fl_couple($pnt1) $pnt2
	}
    }
    # Buscamos las lneas y puntos del casco unidas a puntos de la lnea de flotacin
    # y las eliminamos de la lista de elementos del casco
    set tmplines ""
    set tmppoints ""
    for { set i 0 } { $i < [llength $bd_lines] } { incr i } {
	set line [lindex [lindex $bd_lines $i] 0]
	set retval [.central.s info list_entities lines $line]
	set points [regexp -linestop -all -inline {Points:([ 0-9]*)} $retval]
	for { set j 1 } { $j < [llength $points] } { incr j 2 } {
	    set pnt1 [lindex [lindex $points $j] 0]
	    set pnt2 [lindex [lindex $points $j] 1]
	    if { [lsearch $fl_points $pnt1 ]!=-1 || [lsearch $fl_points $pnt2 ]!=-1 } {
		lappend tmplines $line
		lappend tmppoints "$pnt1 $pnt2"
	    }
	}
    }
    set tmpbd_lines ""
    for { set i 0 } { $i < [llength $bd_lines] } { incr i } {
	set line [lindex $bd_lines $i]
	if { [lsearch $tmplines $line ]==-1 } { lappend tmpbd_lines $line }
    }
    set bd_lines $tmpbd_lines
    # Compruebo si cada una de las lineas en $tmplines es dry o wet
    # Para ello las comparo con una lnea de la flotacin a la que estn unidas
    set dry_lines ""
    set wet_lines ""
    set new_lines ""
    for { set i 0 } { $i < [llength $tmplines] } { incr i } {
	set line [lindex $tmplines $i]
	set points [lindex $tmppoints $i]
	set pnt1 [lindex $points 0]
	set fnd1 [lsearch $fl_cpoint $pnt1]
	set pnt2 [lindex $points 1]
	set fnd2 [lsearch $fl_cpoint $pnt2]
	if { $fnd1!=-1 } {
	    set V1 [NormVector $pnt1 $pnt2]
	    set flpnt $pnt1
	    set flcop $fl_couple($pnt1)
	} else {
	    set V1 [NormVector $pnt2 $pnt1]
	    set flpnt $pnt2 
	    set flcop $fl_couple($pnt2)
	}
	for { set j 0 } { $j < [llength $flcop] } { incr j } {
	    set V2 [NormVector $flpnt [lindex $flcop $j]]
	    if { [expr double([lindex $V1 0])*[lindex $V2 0]]>=0 } { break }
	}
	if { double([lindex $V1 2])>double([lindex $V2 2]) } {
	    lappend dry_lines $line
	} else {
	    lappend wet_lines $line
	}        
	lappend new_lines $line        
    }
    # Busco recursivamente todas las lineas 
    set wet_points [ListPointsFromLines $wet_lines]
    set dry_points [ListPointsFromLines $dry_lines]
    for { set it 0 } { $it < 100 } { incr it } {
	for { set i 0 } { $i < [llength $bd_lines] } { incr i } {
	    set line [lindex [lindex $bd_lines $i] 0]
	    set retval [.central.s info list_entities lines $line]
	    set points [regexp -linestop -all -inline {Points:([ 0-9]*)} $retval]
	    for { set j 1 } { $j < [llength $points] } { incr j 2 } {
		set pnt1 [lindex [lindex $points $j] 0]
		set pnt2 [lindex [lindex $points $j] 1]
		if { [lsearch $wet_points $pnt1 ]!=-1 } {
		    lappend wet_lines $line
		    lappend wet_points $pnt1 $pnt2
		} elseif { [lsearch $wet_points $pnt2 ]!=-1 } {
		    lappend wet_lines $line
		    lappend wet_points $pnt1 $pnt2
		} elseif { [lsearch $dry_points $pnt1 ]!=-1 } {
		    lappend dry_lines $line
		    lappend dry_points $pnt1 $pnt2
		} elseif { [lsearch $dry_points $pnt2 ]!=-1 } {
		    lappend dry_lines $line
		    lappend dry_points $pnt1 $pnt2
		}
	    }
	}
	set tmpbd_lines ""
	for { set i 0 } { $i < [llength $bd_lines] } { incr i } {
	    set line [lindex $bd_lines $i]
	    if { [lsearch $tmplines $line ]==-1 } { lappend tmpbd_lines $line }
	}
	set bd_lines $tmpbd_lines
	if { $bd_lines!="" } { break }
    } 
    # Busco todas las superficies y pongo todo en su sitio
    set bsurface [regexp -linestop -all -inline {Num:([ 0-9]*)} $bsurface]
    for { set j 1 } { $j < [llength $bsurface] } { incr j 2 } {
	set surface [.central.s info list_entities surfaces [lindex $bsurface $j]]
	set lines [regexp -linestop -all -inline {Line:([ 0-9]*)} $surface]
	for { set i 1 } { $i < [llength $lines] } { incr i 2 } {
	    set line [lindex [lindex $lines $i] 0]
	    if { [lsearch $dry_lines $line ]!=-1 } {
		.central.s process escape escape escape escape utilities layer entities \
		        body_dry surfaces [lindex $bsurface $j]
	    } elseif { [lsearch $wet_lines $line ]!=-1 } {
		.central.s process escape escape escape escape utilities layer entities \
		        body_wet surfaces [lindex $bsurface $j]
	    }
	}
    }
    if { ![IsLayerEmpty symm_oy lines] } {
	set sy_lines [.central.s info layers -entities lines symm_oy]
	for { set i 0 } { $i < [llength $sy_lines] } { incr i } {
	    set line [lindex $sy_lines $i]
	    if { [lsearch $wet_lines $line ]!=-1 } {
		.central.s process escape escape escape escape utilities layer entities symm_oy \
		        LowerEntities lines $line
	    } else {
		.central.s process escape escape escape escape utilities layer entities body_dry \
		        LowerEntities lines $line
	    }
	}
    }
    MoveLowerToLayer body_dry
    MoveLowerToLayer free_surf
}

proc OpenTipsWizardWindow { } {
    global TipsWizPriv ProblemTypePriv
 
    # create the wizard widget
    set wizwin .gid.tdyntipswizard
    if { [winfo exists $wizwin] } {
	$wizwin show
	return
    }
    catch { unset TipsWizPriv }

    set dir $ProblemTypePriv(problemtypedir)
    set width 400
    set height 250
    set x [expr [winfo screenwidth .]/2-$width/2]
    set y [expr [winfo screenheight .]/2-$height/2]
    set geometry  ${width}x${height}+${x}+${y}
    tkwizard::tkwizard $wizwin -icondir "$dir/images/tips.gif" \
	    -title "Tip of the day" -geom $geometry \
	    -finishtext "" -canceltext "Close" -resize

    # create the tips
    proc setTipShow { } {
	global ProblemTypePriv TipsWizPriv
	set filename [GiveTdynDefaultsFile]
	WriteVarTdynDefaults showtips $TipsWizPriv(show)
    }

    set TipsWizPriv(show) [IsVarOnTdynDefaults showtips 1]
    bind $wizwin <<WizCancel>> {setTipShow}
    set TipsWizPriv(text) [list \
	    "Find some on-line tutorials in Help>Tutorials. They will\
	    guide you in the geometry creation and meshing processes" \
	    "Find some on-line tutorials which will guide you\
	    in the Tdyn analysis process in Help>Tdyn tutorials" \
	    "Start a Tdyn wizard which will guide you in the analysis\
	    process by selecting Utilities>Tdyn wizard>Tdyn wizard" \
	    "Create easily reports of your analyses by usign the utility\
	    available in Utilities>Create Report" \
	    "Update the Sink&Trim data of a naval analysis using the\
	    tool Utilities>Tdyn wizard>Sink&Trim wizard" \
	    "Follow the icons in the Tdyn toolbar downwards to define\
	    all the data, meshing and perform and analysis" \
	    "Define materials data by functions. This can be comfortably\
	    done with the Functions editor, accessible by pressing the\
	    button \"fx\" of the material window." \
	    "Functions can use any of the variable data calculated within\
	    Tdyn (temperature, velocity, pressure, ...). It is also possible\
	    to calcule its Cartesian derivatives." \
	    "Define new results by functions. This can be comfortably\
	    done with the Functions editor, accessible by pressing the\
	    button \"fx\" of the problem data window." \
	    "Select results to be written through the label window Write\
	    Results of the PROBLEM data window."]

    proc ShowTip { w idx } {
	global TipsWizPriv
	frame $w.fr -borderwidth 1 -relief flat
	label $w.fr.lb -wraplength 240 -justify left -text \
		[lindex $TipsWizPriv(text) $idx]
	checkbutton $w.fr.b1 -text "Show tips at start up" -relief flat \
		-variable TipsWizPriv(show)
	grid $w.fr.lb -row 0 -column 0 -sticky nsew
	 grid $w.fr.b1 -row 1 -column 0 -sticky sw
	grid $w.fr -row 0 -column 0 -sticky nsew
	grid columnconfigure $w.fr 0 -weight 1
	grid rowconfigure    $w.fr 0 -weight 1
	grid columnconfigure $w 0 -weight 1
	grid rowconfigure    $w 0 -weight 1 
	grid conf $w -rowspan 4
    }

    # define wizard steps
    for { set i 0 } { $i < [llength $TipsWizPriv(text)] } { incr i } {
	set command1 {set w [$this widget clientArea]}
	set command2 {$this stepconfigure -title "Did you know ..." -subtitle ""}
	set command3 {ShowTip $w}
	append command3 " $i"
	$wizwin step Tip$i -title "Tip of the Day" \
		-layout advanced "$command1; $command2; $command3;"
    }

    # show the wizard
    $wizwin showRandom
}
