proc ListBoxWindow { wn title labeltext entries { command "" } } { 

    # Makes a List Box Window with entries
    # $parent -  Parent window path
    # $title -  ListBox Window Title
    # $labeltext - Label text inserted above of the values
    # $entries - Entries names
    # $command - executed command when finished

    # Defining the size of the window.
    # A minimum of 30 characters is set

    set lwidth 30
    foreach item $entries {
	if { [string length $item] > $lwidth } {
	    set lwidth [string length $item]
	}
    }

    set w $wn.lsbox
    InitWindow $w $title PreFEditorOperatorsWindowGeom ""

    # Label and listbox with entries
    set main [frame $w.main -bd 2 -relief groove]
    set label [label $main.label -justify left -text $labeltext -padx 3]
    set entriesframe [frame $main.entriesframe -bd .1c]
    set scrollbar [scrollbar $entriesframe.scrollbar -command "$entriesframe.listbox yview"]
    set listbox [listbox $entriesframe.listbox -yscroll "$scrollbar set" \
	    -selectmode single -height 12 -bg white -width $lwidth ]

    grid $main -row 0 -column 0 -sticky nsew
    grid $label -row 0 -column 0 -sticky nw
    grid $entriesframe -row 1 -column 0 -sticky nsew
    grid $listbox -row 1 -column 0 -sticky nwes
    grid $scrollbar -row 1 -column 1 -sticky nse
    grid columnconfigure $entriesframe 0 -weight 1
    grid rowconfigure $entriesframe 0 -weight 1
    grid columnconfigure $main 0 -weight 1
    grid rowconfigure $main 0 -weight 1

    proc MakeSelection { w listbox command } {

	set selent [$listbox curselection]
	  if { ![string length $selent] > 0 } {
	      set selent "-1"
	  }

	eval $command $selent
	destroy $w
    } 

    bind $listbox <Return> "MakeSelection $w $listbox \"$command\""
    bind $listbox <Double-1> "MakeSelection $w $listbox \"$command\""

    eval $listbox insert 0 $entries

    # Buttons
    set btnframe [frame $w.btnframe]
    set btn_ok [button $btnframe.btn_ok -text "OK" -width 5 \
	    -command "MakeSelection $w $listbox \"$command\""]
    set btn_cancel [button $btnframe.btn_cancel -text "Cancel" -width 5 \
	    -command "destroy $w"]

    grid $btnframe -row 1 -column 0 -sticky nsew -padx 3 -pady 1
    grid $btn_ok -row 0 -column 0 -sticky s -padx 3 -pady 1
    grid $btn_cancel -row 0 -column 1 -sticky s -padx 3 -pady 1  
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1

    bind $w <Escape> "tkButtonInvoke $btn_cancel"

    update
    focus $w
}